// Copyright (c) 2016, Lawrence Livermore National Security, LLC.  
// Produced at the Lawrence Livermore National Laboratory.
//
// This file is part of Caliper.
// Written by David Boehme, boehme3@llnl.gov.
// LLNL-CODE-678900
// All rights reserved.
//
// For details, see https://github.com/scalability-llnl/Caliper.
// Please also see the LICENSE file for our additional BSD notice.
//
// Redistribution and use in source and binary forms, with or without modification, are
// permitted provided that the following conditions are met:
//
//  * Redistributions of source code must retain the above copyright notice, this list of
//    conditions and the disclaimer below.
//  * Redistributions in binary form must reproduce the above copyright notice, this list of
//    conditions and the disclaimer (as noted below) in the documentation and/or other materials
//    provided with the distribution.
//  * Neither the name of the LLNS/LLNL nor the names of its contributors may be used to endorse
//    or promote products derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS
// OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
// LAWRENCE LIVERMORE NATIONAL SECURITY, LLC, THE U.S. DEPARTMENT OF ENERGY OR CONTRIBUTORS BE
// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
// ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

/// \file CaliperMetadataAccessInterface.h 
/// \brief Abstract Caliper metadata access interface

#pragma once

#include "Attribute.h"
#include "Entry.h"

#include <cstddef>
#include <vector>

namespace cali
{

class Node;

/// \brief Abstract base class for a Caliper metadata database.

class CaliperMetadataAccessInterface
{
public:

    // --- query operations

    virtual Node*
    node(cali_id_t) const = 0;

    virtual Attribute 
    get_attribute(cali_id_t id) const = 0;
    virtual Attribute
    get_attribute(const std::string& str) const = 0;

    /// \brief Return all attributes
    virtual std::vector<Attribute>
    get_attributes() const = 0;

    /// \brief Return all attributes that have a metadata entry \a meta, of any value
    std::vector<Attribute>
    find_attributes_with(const Attribute& meta) const;

    // --- modifying operations

    virtual Attribute
    create_attribute(const std::string& name,
                     cali_attr_type     type,
                     int                prop = CALI_ATTR_DEFAULT,
                     int                meta = 0,
                     const Attribute*   meta_attr = nullptr,
                     const Variant*     meta_data = nullptr) = 0;

    virtual Node* 
    make_tree_entry(std::size_t n, const Node* nodelist[], Node* parent = 0) = 0;

    // --- globals

    /// \brief Return global entries (entries with the \a CALI_ATTR_GLOBAL flag set)
    virtual std::vector<Entry>
    get_globals() = 0;
};

}
