
#include <mpi.h>
#include <stdio.h>
#include <stdlib.h>

#ifndef _EXTERN_C_
#ifdef __cplusplus
#define _EXTERN_C_ extern "C"
#else /* __cplusplus */
#define _EXTERN_C_
#endif /* __cplusplus */
#endif /* _EXTERN_C_ */

#ifdef MPICH_HAS_C2F
_EXTERN_C_ void *MPIR_ToPointer(int);
#endif // MPICH_HAS_C2F

#ifdef PIC
/* For shared libraries, declare these weak and figure out which one was linked
   based on which init wrapper was called.  See mpi_init wrappers.  */
#pragma weak pmpi_init
#pragma weak PMPI_INIT
#pragma weak pmpi_init_
#pragma weak pmpi_init__
#endif /* PIC */

_EXTERN_C_ void pmpi_init(MPI_Fint *ierr);
_EXTERN_C_ void PMPI_INIT(MPI_Fint *ierr);
_EXTERN_C_ void pmpi_init_(MPI_Fint *ierr);
_EXTERN_C_ void pmpi_init__(MPI_Fint *ierr);


/* Macros to enable and disable compiler warnings for deprecated functions.
 * These macros will be used to silent warnings about deprecated MPI functions,
 * as these should be wrapped, even if they are deprecated.
 *
 * Note: The macros support GCC and clang compilers only. For other compilers
 *       just add similar macros for your compiler.
 */
#if (defined(__GNUC__) && !defined(__clang__)) && \
  ((__GNUC__ == 4 && __GNUC_MINOR__ >= 6) || __GNUC__ > 4)
#define  WRAP_MPI_CALL_PREFIX     \
  _Pragma("GCC diagnostic push"); \
  _Pragma("GCC diagnostic ignored \"-Wdeprecated-declarations\"");
#define WRAP_MPI_CALL_POSTFIX _Pragma("GCC diagnostic pop");
#elif defined(__clang__)
#define  WRAP_MPI_CALL_PREFIX       \
  _Pragma("clang diagnostic push"); \
  _Pragma("clang diagnostic ignored \"-Wdeprecated-declarations\"");
#define WRAP_MPI_CALL_POSTFIX _Pragma("clang diagnostic pop");
#else
#define WRAP_MPI_CALL_PREFIX
#define WRAP_MPI_CALL_POSTFIX
#endif

#include "MpiEvents.h"
#include "services/mpiwrap/MpiTracing.h"

#include "caliper/caliper-config.h"

#include "caliper/Caliper.h"

#include "caliper/common/Log.h"
#include "caliper/common/StringConverter.h"
#include "caliper/common/Variant.h"

#include <mpi.h>

#include <algorithm>
#include <cstring>
#include <iterator>
#include <numeric>
#include <string>

namespace cali
{

extern Attribute   mpifn_attr;
extern Attribute   mpirank_attr;
extern Attribute   mpisize_attr;

extern bool        enable_msg_tracing;

}

using namespace cali;
using namespace std;

namespace 
{

bool       enable_wrapper = false;
MpiTracing tracing;


bool enable_MPI_Abort = false;

bool enable_MPI_Accumulate = false;

bool enable_MPI_Add_error_class = false;

bool enable_MPI_Add_error_code = false;

bool enable_MPI_Add_error_string = false;

bool enable_MPI_Address = false;

bool enable_MPI_Allgather = false;

bool enable_MPI_Allgatherv = false;

bool enable_MPI_Alloc_mem = false;

bool enable_MPI_Allreduce = false;

bool enable_MPI_Alltoall = false;

bool enable_MPI_Alltoallv = false;

bool enable_MPI_Alltoallw = false;

bool enable_MPI_Attr_delete = false;

bool enable_MPI_Attr_get = false;

bool enable_MPI_Attr_put = false;

bool enable_MPI_Barrier = false;

bool enable_MPI_Bcast = false;

bool enable_MPI_Bsend = false;

bool enable_MPI_Bsend_init = false;

bool enable_MPI_Buffer_attach = false;

bool enable_MPI_Buffer_detach = false;

bool enable_MPI_Cancel = false;

bool enable_MPI_Cart_coords = false;

bool enable_MPI_Cart_create = false;

bool enable_MPI_Cart_get = false;

bool enable_MPI_Cart_map = false;

bool enable_MPI_Cart_rank = false;

bool enable_MPI_Cart_shift = false;

bool enable_MPI_Cart_sub = false;

bool enable_MPI_Cartdim_get = false;

bool enable_MPI_Close_port = false;

bool enable_MPI_Comm_accept = false;

bool enable_MPI_Comm_call_errhandler = false;

bool enable_MPI_Comm_compare = false;

bool enable_MPI_Comm_connect = false;

bool enable_MPI_Comm_create = false;

bool enable_MPI_Comm_create_errhandler = false;

bool enable_MPI_Comm_create_group = false;

bool enable_MPI_Comm_create_keyval = false;

bool enable_MPI_Comm_delete_attr = false;

bool enable_MPI_Comm_disconnect = false;

bool enable_MPI_Comm_dup = false;

bool enable_MPI_Comm_dup_with_info = false;

bool enable_MPI_Comm_free = false;

bool enable_MPI_Comm_free_keyval = false;

bool enable_MPI_Comm_get_attr = false;

bool enable_MPI_Comm_get_errhandler = false;

bool enable_MPI_Comm_get_info = false;

bool enable_MPI_Comm_get_name = false;

bool enable_MPI_Comm_get_parent = false;

bool enable_MPI_Comm_group = false;

bool enable_MPI_Comm_idup = false;

bool enable_MPI_Comm_join = false;

bool enable_MPI_Comm_rank = false;

bool enable_MPI_Comm_remote_group = false;

bool enable_MPI_Comm_remote_size = false;

bool enable_MPI_Comm_set_attr = false;

bool enable_MPI_Comm_set_errhandler = false;

bool enable_MPI_Comm_set_info = false;

bool enable_MPI_Comm_set_name = false;

bool enable_MPI_Comm_size = false;

bool enable_MPI_Comm_split = false;

bool enable_MPI_Comm_split_type = false;

bool enable_MPI_Comm_test_inter = false;

bool enable_MPI_Compare_and_swap = false;

bool enable_MPI_Dims_create = false;

bool enable_MPI_Dist_graph_create = false;

bool enable_MPI_Dist_graph_create_adjacent = false;

bool enable_MPI_Dist_graph_neighbors = false;

bool enable_MPI_Dist_graph_neighbors_count = false;

bool enable_MPI_Errhandler_create = false;

bool enable_MPI_Errhandler_free = false;

bool enable_MPI_Errhandler_get = false;

bool enable_MPI_Errhandler_set = false;

bool enable_MPI_Error_class = false;

bool enable_MPI_Error_string = false;

bool enable_MPI_Exscan = false;

bool enable_MPI_Fetch_and_op = false;

bool enable_MPI_File_call_errhandler = false;

bool enable_MPI_File_close = false;

bool enable_MPI_File_create_errhandler = false;

bool enable_MPI_File_delete = false;

bool enable_MPI_File_get_amode = false;

bool enable_MPI_File_get_atomicity = false;

bool enable_MPI_File_get_byte_offset = false;

bool enable_MPI_File_get_errhandler = false;

bool enable_MPI_File_get_group = false;

bool enable_MPI_File_get_info = false;

bool enable_MPI_File_get_position = false;

bool enable_MPI_File_get_position_shared = false;

bool enable_MPI_File_get_size = false;

bool enable_MPI_File_get_type_extent = false;

bool enable_MPI_File_get_view = false;

bool enable_MPI_File_iread = false;

bool enable_MPI_File_iread_at = false;

bool enable_MPI_File_iread_shared = false;

bool enable_MPI_File_iwrite = false;

bool enable_MPI_File_iwrite_at = false;

bool enable_MPI_File_iwrite_shared = false;

bool enable_MPI_File_open = false;

bool enable_MPI_File_preallocate = false;

bool enable_MPI_File_read = false;

bool enable_MPI_File_read_all = false;

bool enable_MPI_File_read_all_begin = false;

bool enable_MPI_File_read_all_end = false;

bool enable_MPI_File_read_at = false;

bool enable_MPI_File_read_at_all = false;

bool enable_MPI_File_read_at_all_begin = false;

bool enable_MPI_File_read_at_all_end = false;

bool enable_MPI_File_read_ordered = false;

bool enable_MPI_File_read_ordered_begin = false;

bool enable_MPI_File_read_ordered_end = false;

bool enable_MPI_File_read_shared = false;

bool enable_MPI_File_seek = false;

bool enable_MPI_File_seek_shared = false;

bool enable_MPI_File_set_atomicity = false;

bool enable_MPI_File_set_errhandler = false;

bool enable_MPI_File_set_info = false;

bool enable_MPI_File_set_size = false;

bool enable_MPI_File_set_view = false;

bool enable_MPI_File_sync = false;

bool enable_MPI_File_write = false;

bool enable_MPI_File_write_all = false;

bool enable_MPI_File_write_all_begin = false;

bool enable_MPI_File_write_all_end = false;

bool enable_MPI_File_write_at = false;

bool enable_MPI_File_write_at_all = false;

bool enable_MPI_File_write_at_all_begin = false;

bool enable_MPI_File_write_at_all_end = false;

bool enable_MPI_File_write_ordered = false;

bool enable_MPI_File_write_ordered_begin = false;

bool enable_MPI_File_write_ordered_end = false;

bool enable_MPI_File_write_shared = false;

bool enable_MPI_Finalize = false;

bool enable_MPI_Finalized = false;

bool enable_MPI_Free_mem = false;

bool enable_MPI_Gather = false;

bool enable_MPI_Gatherv = false;

bool enable_MPI_Get = false;

bool enable_MPI_Get_accumulate = false;

bool enable_MPI_Get_address = false;

bool enable_MPI_Get_count = false;

bool enable_MPI_Get_elements = false;

bool enable_MPI_Get_elements_x = false;

bool enable_MPI_Get_library_version = false;

bool enable_MPI_Get_processor_name = false;

bool enable_MPI_Get_version = false;

bool enable_MPI_Graph_create = false;

bool enable_MPI_Graph_get = false;

bool enable_MPI_Graph_map = false;

bool enable_MPI_Graph_neighbors = false;

bool enable_MPI_Graph_neighbors_count = false;

bool enable_MPI_Graphdims_get = false;

bool enable_MPI_Grequest_complete = false;

bool enable_MPI_Grequest_start = false;

bool enable_MPI_Group_compare = false;

bool enable_MPI_Group_difference = false;

bool enable_MPI_Group_excl = false;

bool enable_MPI_Group_free = false;

bool enable_MPI_Group_incl = false;

bool enable_MPI_Group_intersection = false;

bool enable_MPI_Group_range_excl = false;

bool enable_MPI_Group_range_incl = false;

bool enable_MPI_Group_rank = false;

bool enable_MPI_Group_size = false;

bool enable_MPI_Group_translate_ranks = false;

bool enable_MPI_Group_union = false;

bool enable_MPI_Iallgather = false;

bool enable_MPI_Iallgatherv = false;

bool enable_MPI_Iallreduce = false;

bool enable_MPI_Ialltoall = false;

bool enable_MPI_Ialltoallv = false;

bool enable_MPI_Ialltoallw = false;

bool enable_MPI_Ibarrier = false;

bool enable_MPI_Ibcast = false;

bool enable_MPI_Ibsend = false;

bool enable_MPI_Iexscan = false;

bool enable_MPI_Igather = false;

bool enable_MPI_Igatherv = false;

bool enable_MPI_Improbe = false;

bool enable_MPI_Imrecv = false;

bool enable_MPI_Ineighbor_allgather = false;

bool enable_MPI_Ineighbor_allgatherv = false;

bool enable_MPI_Ineighbor_alltoall = false;

bool enable_MPI_Ineighbor_alltoallv = false;

bool enable_MPI_Ineighbor_alltoallw = false;

bool enable_MPI_Info_create = false;

bool enable_MPI_Info_delete = false;

bool enable_MPI_Info_dup = false;

bool enable_MPI_Info_free = false;

bool enable_MPI_Info_get = false;

bool enable_MPI_Info_get_nkeys = false;

bool enable_MPI_Info_get_nthkey = false;

bool enable_MPI_Info_get_valuelen = false;

bool enable_MPI_Info_set = false;

bool enable_MPI_Init = false;

bool enable_MPI_Init_thread = false;

bool enable_MPI_Initialized = false;

bool enable_MPI_Intercomm_create = false;

bool enable_MPI_Intercomm_merge = false;

bool enable_MPI_Iprobe = false;

bool enable_MPI_Irecv = false;

bool enable_MPI_Ireduce = false;

bool enable_MPI_Ireduce_scatter = false;

bool enable_MPI_Ireduce_scatter_block = false;

bool enable_MPI_Irsend = false;

bool enable_MPI_Is_thread_main = false;

bool enable_MPI_Iscan = false;

bool enable_MPI_Iscatter = false;

bool enable_MPI_Iscatterv = false;

bool enable_MPI_Isend = false;

bool enable_MPI_Issend = false;

bool enable_MPI_Keyval_create = false;

bool enable_MPI_Keyval_free = false;

bool enable_MPI_Lookup_name = false;

bool enable_MPI_Mprobe = false;

bool enable_MPI_Mrecv = false;

bool enable_MPI_Neighbor_allgather = false;

bool enable_MPI_Neighbor_allgatherv = false;

bool enable_MPI_Neighbor_alltoall = false;

bool enable_MPI_Neighbor_alltoallv = false;

bool enable_MPI_Neighbor_alltoallw = false;

bool enable_MPI_Op_commutative = false;

bool enable_MPI_Op_create = false;

bool enable_MPI_Op_free = false;

bool enable_MPI_Open_port = false;

bool enable_MPI_Pack = false;

bool enable_MPI_Pack_external = false;

bool enable_MPI_Pack_external_size = false;

bool enable_MPI_Pack_size = false;

bool enable_MPI_Pcontrol = false;

bool enable_MPI_Probe = false;

bool enable_MPI_Publish_name = false;

bool enable_MPI_Put = false;

bool enable_MPI_Query_thread = false;

bool enable_MPI_Raccumulate = false;

bool enable_MPI_Recv = false;

bool enable_MPI_Recv_init = false;

bool enable_MPI_Reduce = false;

bool enable_MPI_Reduce_local = false;

bool enable_MPI_Reduce_scatter = false;

bool enable_MPI_Reduce_scatter_block = false;

bool enable_MPI_Register_datarep = false;

bool enable_MPI_Request_free = false;

bool enable_MPI_Request_get_status = false;

bool enable_MPI_Rget = false;

bool enable_MPI_Rget_accumulate = false;

bool enable_MPI_Rput = false;

bool enable_MPI_Rsend = false;

bool enable_MPI_Rsend_init = false;

bool enable_MPI_Scan = false;

bool enable_MPI_Scatter = false;

bool enable_MPI_Scatterv = false;

bool enable_MPI_Send = false;

bool enable_MPI_Send_init = false;

bool enable_MPI_Sendrecv = false;

bool enable_MPI_Sendrecv_replace = false;

bool enable_MPI_Ssend = false;

bool enable_MPI_Ssend_init = false;

bool enable_MPI_Start = false;

bool enable_MPI_Startall = false;

bool enable_MPI_Status_set_cancelled = false;

bool enable_MPI_Status_set_elements = false;

bool enable_MPI_Status_set_elements_x = false;

bool enable_MPI_Test = false;

bool enable_MPI_Test_cancelled = false;

bool enable_MPI_Testall = false;

bool enable_MPI_Testany = false;

bool enable_MPI_Testsome = false;

bool enable_MPI_Topo_test = false;

bool enable_MPI_Type_commit = false;

bool enable_MPI_Type_contiguous = false;

bool enable_MPI_Type_create_darray = false;

bool enable_MPI_Type_create_f90_complex = false;

bool enable_MPI_Type_create_f90_integer = false;

bool enable_MPI_Type_create_f90_real = false;

bool enable_MPI_Type_create_hindexed = false;

bool enable_MPI_Type_create_hindexed_block = false;

bool enable_MPI_Type_create_hvector = false;

bool enable_MPI_Type_create_indexed_block = false;

bool enable_MPI_Type_create_keyval = false;

bool enable_MPI_Type_create_resized = false;

bool enable_MPI_Type_create_struct = false;

bool enable_MPI_Type_create_subarray = false;

bool enable_MPI_Type_delete_attr = false;

bool enable_MPI_Type_dup = false;

bool enable_MPI_Type_extent = false;

bool enable_MPI_Type_free = false;

bool enable_MPI_Type_free_keyval = false;

bool enable_MPI_Type_get_attr = false;

bool enable_MPI_Type_get_contents = false;

bool enable_MPI_Type_get_envelope = false;

bool enable_MPI_Type_get_extent = false;

bool enable_MPI_Type_get_extent_x = false;

bool enable_MPI_Type_get_name = false;

bool enable_MPI_Type_get_true_extent = false;

bool enable_MPI_Type_get_true_extent_x = false;

bool enable_MPI_Type_hindexed = false;

bool enable_MPI_Type_hvector = false;

bool enable_MPI_Type_indexed = false;

bool enable_MPI_Type_lb = false;

bool enable_MPI_Type_match_size = false;

bool enable_MPI_Type_set_attr = false;

bool enable_MPI_Type_set_name = false;

bool enable_MPI_Type_size = false;

bool enable_MPI_Type_size_x = false;

bool enable_MPI_Type_struct = false;

bool enable_MPI_Type_ub = false;

bool enable_MPI_Type_vector = false;

bool enable_MPI_Unpack = false;

bool enable_MPI_Unpack_external = false;

bool enable_MPI_Unpublish_name = false;

bool enable_MPI_Wait = false;

bool enable_MPI_Waitall = false;

bool enable_MPI_Waitany = false;

bool enable_MPI_Waitsome = false;

bool enable_MPI_Win_allocate = false;

bool enable_MPI_Win_allocate_shared = false;

bool enable_MPI_Win_attach = false;

bool enable_MPI_Win_call_errhandler = false;

bool enable_MPI_Win_complete = false;

bool enable_MPI_Win_create = false;

bool enable_MPI_Win_create_dynamic = false;

bool enable_MPI_Win_create_errhandler = false;

bool enable_MPI_Win_create_keyval = false;

bool enable_MPI_Win_delete_attr = false;

bool enable_MPI_Win_detach = false;

bool enable_MPI_Win_fence = false;

bool enable_MPI_Win_flush = false;

bool enable_MPI_Win_flush_all = false;

bool enable_MPI_Win_flush_local = false;

bool enable_MPI_Win_flush_local_all = false;

bool enable_MPI_Win_free = false;

bool enable_MPI_Win_free_keyval = false;

bool enable_MPI_Win_get_attr = false;

bool enable_MPI_Win_get_errhandler = false;

bool enable_MPI_Win_get_group = false;

bool enable_MPI_Win_get_info = false;

bool enable_MPI_Win_get_name = false;

bool enable_MPI_Win_lock = false;

bool enable_MPI_Win_lock_all = false;

bool enable_MPI_Win_post = false;

bool enable_MPI_Win_set_attr = false;

bool enable_MPI_Win_set_errhandler = false;

bool enable_MPI_Win_set_info = false;

bool enable_MPI_Win_set_name = false;

bool enable_MPI_Win_shared_query = false;

bool enable_MPI_Win_start = false;

bool enable_MPI_Win_sync = false;

bool enable_MPI_Win_test = false;

bool enable_MPI_Win_unlock = false;

bool enable_MPI_Win_unlock_all = false;

bool enable_MPI_Win_wait = false;

bool enable_MPI_Wtick = false;

bool enable_MPI_Wtime = false;


void setup_filter(const std::string& whitelist_string, const std::string& blacklist_string) {
    std::vector<std::string> whitelist =
        StringConverter(whitelist_string).to_stringlist(",:");
    std::vector<std::string> blacklist =
        StringConverter(blacklist_string).to_stringlist(",:");

    bool have_whitelist = whitelist.size() > 0;
    bool have_blacklist = blacklist.size() > 0;

    if (!have_whitelist && !have_blacklist)
        return;

    bool enable_all = false;

    if (have_whitelist && whitelist.front() == "all") {
        enable_all = true;
        whitelist.erase(whitelist.begin());
    }

    const struct fntable_elem {
        const char* name;
        bool*       enableptr; 
    } table[] = {
        
        { "MPI_Abort", &enable_MPI_Abort },
        
        { "MPI_Accumulate", &enable_MPI_Accumulate },
        
        { "MPI_Add_error_class", &enable_MPI_Add_error_class },
        
        { "MPI_Add_error_code", &enable_MPI_Add_error_code },
        
        { "MPI_Add_error_string", &enable_MPI_Add_error_string },
        
        { "MPI_Address", &enable_MPI_Address },
        
        { "MPI_Allgather", &enable_MPI_Allgather },
        
        { "MPI_Allgatherv", &enable_MPI_Allgatherv },
        
        { "MPI_Alloc_mem", &enable_MPI_Alloc_mem },
        
        { "MPI_Allreduce", &enable_MPI_Allreduce },
        
        { "MPI_Alltoall", &enable_MPI_Alltoall },
        
        { "MPI_Alltoallv", &enable_MPI_Alltoallv },
        
        { "MPI_Alltoallw", &enable_MPI_Alltoallw },
        
        { "MPI_Attr_delete", &enable_MPI_Attr_delete },
        
        { "MPI_Attr_get", &enable_MPI_Attr_get },
        
        { "MPI_Attr_put", &enable_MPI_Attr_put },
        
        { "MPI_Barrier", &enable_MPI_Barrier },
        
        { "MPI_Bcast", &enable_MPI_Bcast },
        
        { "MPI_Bsend", &enable_MPI_Bsend },
        
        { "MPI_Bsend_init", &enable_MPI_Bsend_init },
        
        { "MPI_Buffer_attach", &enable_MPI_Buffer_attach },
        
        { "MPI_Buffer_detach", &enable_MPI_Buffer_detach },
        
        { "MPI_Cancel", &enable_MPI_Cancel },
        
        { "MPI_Cart_coords", &enable_MPI_Cart_coords },
        
        { "MPI_Cart_create", &enable_MPI_Cart_create },
        
        { "MPI_Cart_get", &enable_MPI_Cart_get },
        
        { "MPI_Cart_map", &enable_MPI_Cart_map },
        
        { "MPI_Cart_rank", &enable_MPI_Cart_rank },
        
        { "MPI_Cart_shift", &enable_MPI_Cart_shift },
        
        { "MPI_Cart_sub", &enable_MPI_Cart_sub },
        
        { "MPI_Cartdim_get", &enable_MPI_Cartdim_get },
        
        { "MPI_Close_port", &enable_MPI_Close_port },
        
        { "MPI_Comm_accept", &enable_MPI_Comm_accept },
        
        { "MPI_Comm_call_errhandler", &enable_MPI_Comm_call_errhandler },
        
        { "MPI_Comm_compare", &enable_MPI_Comm_compare },
        
        { "MPI_Comm_connect", &enable_MPI_Comm_connect },
        
        { "MPI_Comm_create", &enable_MPI_Comm_create },
        
        { "MPI_Comm_create_errhandler", &enable_MPI_Comm_create_errhandler },
        
        { "MPI_Comm_create_group", &enable_MPI_Comm_create_group },
        
        { "MPI_Comm_create_keyval", &enable_MPI_Comm_create_keyval },
        
        { "MPI_Comm_delete_attr", &enable_MPI_Comm_delete_attr },
        
        { "MPI_Comm_disconnect", &enable_MPI_Comm_disconnect },
        
        { "MPI_Comm_dup", &enable_MPI_Comm_dup },
        
        { "MPI_Comm_dup_with_info", &enable_MPI_Comm_dup_with_info },
        
        { "MPI_Comm_free", &enable_MPI_Comm_free },
        
        { "MPI_Comm_free_keyval", &enable_MPI_Comm_free_keyval },
        
        { "MPI_Comm_get_attr", &enable_MPI_Comm_get_attr },
        
        { "MPI_Comm_get_errhandler", &enable_MPI_Comm_get_errhandler },
        
        { "MPI_Comm_get_info", &enable_MPI_Comm_get_info },
        
        { "MPI_Comm_get_name", &enable_MPI_Comm_get_name },
        
        { "MPI_Comm_get_parent", &enable_MPI_Comm_get_parent },
        
        { "MPI_Comm_group", &enable_MPI_Comm_group },
        
        { "MPI_Comm_idup", &enable_MPI_Comm_idup },
        
        { "MPI_Comm_join", &enable_MPI_Comm_join },
        
        { "MPI_Comm_rank", &enable_MPI_Comm_rank },
        
        { "MPI_Comm_remote_group", &enable_MPI_Comm_remote_group },
        
        { "MPI_Comm_remote_size", &enable_MPI_Comm_remote_size },
        
        { "MPI_Comm_set_attr", &enable_MPI_Comm_set_attr },
        
        { "MPI_Comm_set_errhandler", &enable_MPI_Comm_set_errhandler },
        
        { "MPI_Comm_set_info", &enable_MPI_Comm_set_info },
        
        { "MPI_Comm_set_name", &enable_MPI_Comm_set_name },
        
        { "MPI_Comm_size", &enable_MPI_Comm_size },
        
        { "MPI_Comm_split", &enable_MPI_Comm_split },
        
        { "MPI_Comm_split_type", &enable_MPI_Comm_split_type },
        
        { "MPI_Comm_test_inter", &enable_MPI_Comm_test_inter },
        
        { "MPI_Compare_and_swap", &enable_MPI_Compare_and_swap },
        
        { "MPI_Dims_create", &enable_MPI_Dims_create },
        
        { "MPI_Dist_graph_create", &enable_MPI_Dist_graph_create },
        
        { "MPI_Dist_graph_create_adjacent", &enable_MPI_Dist_graph_create_adjacent },
        
        { "MPI_Dist_graph_neighbors", &enable_MPI_Dist_graph_neighbors },
        
        { "MPI_Dist_graph_neighbors_count", &enable_MPI_Dist_graph_neighbors_count },
        
        { "MPI_Errhandler_create", &enable_MPI_Errhandler_create },
        
        { "MPI_Errhandler_free", &enable_MPI_Errhandler_free },
        
        { "MPI_Errhandler_get", &enable_MPI_Errhandler_get },
        
        { "MPI_Errhandler_set", &enable_MPI_Errhandler_set },
        
        { "MPI_Error_class", &enable_MPI_Error_class },
        
        { "MPI_Error_string", &enable_MPI_Error_string },
        
        { "MPI_Exscan", &enable_MPI_Exscan },
        
        { "MPI_Fetch_and_op", &enable_MPI_Fetch_and_op },
        
        { "MPI_File_call_errhandler", &enable_MPI_File_call_errhandler },
        
        { "MPI_File_close", &enable_MPI_File_close },
        
        { "MPI_File_create_errhandler", &enable_MPI_File_create_errhandler },
        
        { "MPI_File_delete", &enable_MPI_File_delete },
        
        { "MPI_File_get_amode", &enable_MPI_File_get_amode },
        
        { "MPI_File_get_atomicity", &enable_MPI_File_get_atomicity },
        
        { "MPI_File_get_byte_offset", &enable_MPI_File_get_byte_offset },
        
        { "MPI_File_get_errhandler", &enable_MPI_File_get_errhandler },
        
        { "MPI_File_get_group", &enable_MPI_File_get_group },
        
        { "MPI_File_get_info", &enable_MPI_File_get_info },
        
        { "MPI_File_get_position", &enable_MPI_File_get_position },
        
        { "MPI_File_get_position_shared", &enable_MPI_File_get_position_shared },
        
        { "MPI_File_get_size", &enable_MPI_File_get_size },
        
        { "MPI_File_get_type_extent", &enable_MPI_File_get_type_extent },
        
        { "MPI_File_get_view", &enable_MPI_File_get_view },
        
        { "MPI_File_iread", &enable_MPI_File_iread },
        
        { "MPI_File_iread_at", &enable_MPI_File_iread_at },
        
        { "MPI_File_iread_shared", &enable_MPI_File_iread_shared },
        
        { "MPI_File_iwrite", &enable_MPI_File_iwrite },
        
        { "MPI_File_iwrite_at", &enable_MPI_File_iwrite_at },
        
        { "MPI_File_iwrite_shared", &enable_MPI_File_iwrite_shared },
        
        { "MPI_File_open", &enable_MPI_File_open },
        
        { "MPI_File_preallocate", &enable_MPI_File_preallocate },
        
        { "MPI_File_read", &enable_MPI_File_read },
        
        { "MPI_File_read_all", &enable_MPI_File_read_all },
        
        { "MPI_File_read_all_begin", &enable_MPI_File_read_all_begin },
        
        { "MPI_File_read_all_end", &enable_MPI_File_read_all_end },
        
        { "MPI_File_read_at", &enable_MPI_File_read_at },
        
        { "MPI_File_read_at_all", &enable_MPI_File_read_at_all },
        
        { "MPI_File_read_at_all_begin", &enable_MPI_File_read_at_all_begin },
        
        { "MPI_File_read_at_all_end", &enable_MPI_File_read_at_all_end },
        
        { "MPI_File_read_ordered", &enable_MPI_File_read_ordered },
        
        { "MPI_File_read_ordered_begin", &enable_MPI_File_read_ordered_begin },
        
        { "MPI_File_read_ordered_end", &enable_MPI_File_read_ordered_end },
        
        { "MPI_File_read_shared", &enable_MPI_File_read_shared },
        
        { "MPI_File_seek", &enable_MPI_File_seek },
        
        { "MPI_File_seek_shared", &enable_MPI_File_seek_shared },
        
        { "MPI_File_set_atomicity", &enable_MPI_File_set_atomicity },
        
        { "MPI_File_set_errhandler", &enable_MPI_File_set_errhandler },
        
        { "MPI_File_set_info", &enable_MPI_File_set_info },
        
        { "MPI_File_set_size", &enable_MPI_File_set_size },
        
        { "MPI_File_set_view", &enable_MPI_File_set_view },
        
        { "MPI_File_sync", &enable_MPI_File_sync },
        
        { "MPI_File_write", &enable_MPI_File_write },
        
        { "MPI_File_write_all", &enable_MPI_File_write_all },
        
        { "MPI_File_write_all_begin", &enable_MPI_File_write_all_begin },
        
        { "MPI_File_write_all_end", &enable_MPI_File_write_all_end },
        
        { "MPI_File_write_at", &enable_MPI_File_write_at },
        
        { "MPI_File_write_at_all", &enable_MPI_File_write_at_all },
        
        { "MPI_File_write_at_all_begin", &enable_MPI_File_write_at_all_begin },
        
        { "MPI_File_write_at_all_end", &enable_MPI_File_write_at_all_end },
        
        { "MPI_File_write_ordered", &enable_MPI_File_write_ordered },
        
        { "MPI_File_write_ordered_begin", &enable_MPI_File_write_ordered_begin },
        
        { "MPI_File_write_ordered_end", &enable_MPI_File_write_ordered_end },
        
        { "MPI_File_write_shared", &enable_MPI_File_write_shared },
        
        { "MPI_Finalize", &enable_MPI_Finalize },
        
        { "MPI_Finalized", &enable_MPI_Finalized },
        
        { "MPI_Free_mem", &enable_MPI_Free_mem },
        
        { "MPI_Gather", &enable_MPI_Gather },
        
        { "MPI_Gatherv", &enable_MPI_Gatherv },
        
        { "MPI_Get", &enable_MPI_Get },
        
        { "MPI_Get_accumulate", &enable_MPI_Get_accumulate },
        
        { "MPI_Get_address", &enable_MPI_Get_address },
        
        { "MPI_Get_count", &enable_MPI_Get_count },
        
        { "MPI_Get_elements", &enable_MPI_Get_elements },
        
        { "MPI_Get_elements_x", &enable_MPI_Get_elements_x },
        
        { "MPI_Get_library_version", &enable_MPI_Get_library_version },
        
        { "MPI_Get_processor_name", &enable_MPI_Get_processor_name },
        
        { "MPI_Get_version", &enable_MPI_Get_version },
        
        { "MPI_Graph_create", &enable_MPI_Graph_create },
        
        { "MPI_Graph_get", &enable_MPI_Graph_get },
        
        { "MPI_Graph_map", &enable_MPI_Graph_map },
        
        { "MPI_Graph_neighbors", &enable_MPI_Graph_neighbors },
        
        { "MPI_Graph_neighbors_count", &enable_MPI_Graph_neighbors_count },
        
        { "MPI_Graphdims_get", &enable_MPI_Graphdims_get },
        
        { "MPI_Grequest_complete", &enable_MPI_Grequest_complete },
        
        { "MPI_Grequest_start", &enable_MPI_Grequest_start },
        
        { "MPI_Group_compare", &enable_MPI_Group_compare },
        
        { "MPI_Group_difference", &enable_MPI_Group_difference },
        
        { "MPI_Group_excl", &enable_MPI_Group_excl },
        
        { "MPI_Group_free", &enable_MPI_Group_free },
        
        { "MPI_Group_incl", &enable_MPI_Group_incl },
        
        { "MPI_Group_intersection", &enable_MPI_Group_intersection },
        
        { "MPI_Group_range_excl", &enable_MPI_Group_range_excl },
        
        { "MPI_Group_range_incl", &enable_MPI_Group_range_incl },
        
        { "MPI_Group_rank", &enable_MPI_Group_rank },
        
        { "MPI_Group_size", &enable_MPI_Group_size },
        
        { "MPI_Group_translate_ranks", &enable_MPI_Group_translate_ranks },
        
        { "MPI_Group_union", &enable_MPI_Group_union },
        
        { "MPI_Iallgather", &enable_MPI_Iallgather },
        
        { "MPI_Iallgatherv", &enable_MPI_Iallgatherv },
        
        { "MPI_Iallreduce", &enable_MPI_Iallreduce },
        
        { "MPI_Ialltoall", &enable_MPI_Ialltoall },
        
        { "MPI_Ialltoallv", &enable_MPI_Ialltoallv },
        
        { "MPI_Ialltoallw", &enable_MPI_Ialltoallw },
        
        { "MPI_Ibarrier", &enable_MPI_Ibarrier },
        
        { "MPI_Ibcast", &enable_MPI_Ibcast },
        
        { "MPI_Ibsend", &enable_MPI_Ibsend },
        
        { "MPI_Iexscan", &enable_MPI_Iexscan },
        
        { "MPI_Igather", &enable_MPI_Igather },
        
        { "MPI_Igatherv", &enable_MPI_Igatherv },
        
        { "MPI_Improbe", &enable_MPI_Improbe },
        
        { "MPI_Imrecv", &enable_MPI_Imrecv },
        
        { "MPI_Ineighbor_allgather", &enable_MPI_Ineighbor_allgather },
        
        { "MPI_Ineighbor_allgatherv", &enable_MPI_Ineighbor_allgatherv },
        
        { "MPI_Ineighbor_alltoall", &enable_MPI_Ineighbor_alltoall },
        
        { "MPI_Ineighbor_alltoallv", &enable_MPI_Ineighbor_alltoallv },
        
        { "MPI_Ineighbor_alltoallw", &enable_MPI_Ineighbor_alltoallw },
        
        { "MPI_Info_create", &enable_MPI_Info_create },
        
        { "MPI_Info_delete", &enable_MPI_Info_delete },
        
        { "MPI_Info_dup", &enable_MPI_Info_dup },
        
        { "MPI_Info_free", &enable_MPI_Info_free },
        
        { "MPI_Info_get", &enable_MPI_Info_get },
        
        { "MPI_Info_get_nkeys", &enable_MPI_Info_get_nkeys },
        
        { "MPI_Info_get_nthkey", &enable_MPI_Info_get_nthkey },
        
        { "MPI_Info_get_valuelen", &enable_MPI_Info_get_valuelen },
        
        { "MPI_Info_set", &enable_MPI_Info_set },
        
        { "MPI_Init", &enable_MPI_Init },
        
        { "MPI_Init_thread", &enable_MPI_Init_thread },
        
        { "MPI_Initialized", &enable_MPI_Initialized },
        
        { "MPI_Intercomm_create", &enable_MPI_Intercomm_create },
        
        { "MPI_Intercomm_merge", &enable_MPI_Intercomm_merge },
        
        { "MPI_Iprobe", &enable_MPI_Iprobe },
        
        { "MPI_Irecv", &enable_MPI_Irecv },
        
        { "MPI_Ireduce", &enable_MPI_Ireduce },
        
        { "MPI_Ireduce_scatter", &enable_MPI_Ireduce_scatter },
        
        { "MPI_Ireduce_scatter_block", &enable_MPI_Ireduce_scatter_block },
        
        { "MPI_Irsend", &enable_MPI_Irsend },
        
        { "MPI_Is_thread_main", &enable_MPI_Is_thread_main },
        
        { "MPI_Iscan", &enable_MPI_Iscan },
        
        { "MPI_Iscatter", &enable_MPI_Iscatter },
        
        { "MPI_Iscatterv", &enable_MPI_Iscatterv },
        
        { "MPI_Isend", &enable_MPI_Isend },
        
        { "MPI_Issend", &enable_MPI_Issend },
        
        { "MPI_Keyval_create", &enable_MPI_Keyval_create },
        
        { "MPI_Keyval_free", &enable_MPI_Keyval_free },
        
        { "MPI_Lookup_name", &enable_MPI_Lookup_name },
        
        { "MPI_Mprobe", &enable_MPI_Mprobe },
        
        { "MPI_Mrecv", &enable_MPI_Mrecv },
        
        { "MPI_Neighbor_allgather", &enable_MPI_Neighbor_allgather },
        
        { "MPI_Neighbor_allgatherv", &enable_MPI_Neighbor_allgatherv },
        
        { "MPI_Neighbor_alltoall", &enable_MPI_Neighbor_alltoall },
        
        { "MPI_Neighbor_alltoallv", &enable_MPI_Neighbor_alltoallv },
        
        { "MPI_Neighbor_alltoallw", &enable_MPI_Neighbor_alltoallw },
        
        { "MPI_Op_commutative", &enable_MPI_Op_commutative },
        
        { "MPI_Op_create", &enable_MPI_Op_create },
        
        { "MPI_Op_free", &enable_MPI_Op_free },
        
        { "MPI_Open_port", &enable_MPI_Open_port },
        
        { "MPI_Pack", &enable_MPI_Pack },
        
        { "MPI_Pack_external", &enable_MPI_Pack_external },
        
        { "MPI_Pack_external_size", &enable_MPI_Pack_external_size },
        
        { "MPI_Pack_size", &enable_MPI_Pack_size },
        
        { "MPI_Pcontrol", &enable_MPI_Pcontrol },
        
        { "MPI_Probe", &enable_MPI_Probe },
        
        { "MPI_Publish_name", &enable_MPI_Publish_name },
        
        { "MPI_Put", &enable_MPI_Put },
        
        { "MPI_Query_thread", &enable_MPI_Query_thread },
        
        { "MPI_Raccumulate", &enable_MPI_Raccumulate },
        
        { "MPI_Recv", &enable_MPI_Recv },
        
        { "MPI_Recv_init", &enable_MPI_Recv_init },
        
        { "MPI_Reduce", &enable_MPI_Reduce },
        
        { "MPI_Reduce_local", &enable_MPI_Reduce_local },
        
        { "MPI_Reduce_scatter", &enable_MPI_Reduce_scatter },
        
        { "MPI_Reduce_scatter_block", &enable_MPI_Reduce_scatter_block },
        
        { "MPI_Register_datarep", &enable_MPI_Register_datarep },
        
        { "MPI_Request_free", &enable_MPI_Request_free },
        
        { "MPI_Request_get_status", &enable_MPI_Request_get_status },
        
        { "MPI_Rget", &enable_MPI_Rget },
        
        { "MPI_Rget_accumulate", &enable_MPI_Rget_accumulate },
        
        { "MPI_Rput", &enable_MPI_Rput },
        
        { "MPI_Rsend", &enable_MPI_Rsend },
        
        { "MPI_Rsend_init", &enable_MPI_Rsend_init },
        
        { "MPI_Scan", &enable_MPI_Scan },
        
        { "MPI_Scatter", &enable_MPI_Scatter },
        
        { "MPI_Scatterv", &enable_MPI_Scatterv },
        
        { "MPI_Send", &enable_MPI_Send },
        
        { "MPI_Send_init", &enable_MPI_Send_init },
        
        { "MPI_Sendrecv", &enable_MPI_Sendrecv },
        
        { "MPI_Sendrecv_replace", &enable_MPI_Sendrecv_replace },
        
        { "MPI_Ssend", &enable_MPI_Ssend },
        
        { "MPI_Ssend_init", &enable_MPI_Ssend_init },
        
        { "MPI_Start", &enable_MPI_Start },
        
        { "MPI_Startall", &enable_MPI_Startall },
        
        { "MPI_Status_set_cancelled", &enable_MPI_Status_set_cancelled },
        
        { "MPI_Status_set_elements", &enable_MPI_Status_set_elements },
        
        { "MPI_Status_set_elements_x", &enable_MPI_Status_set_elements_x },
        
        { "MPI_Test", &enable_MPI_Test },
        
        { "MPI_Test_cancelled", &enable_MPI_Test_cancelled },
        
        { "MPI_Testall", &enable_MPI_Testall },
        
        { "MPI_Testany", &enable_MPI_Testany },
        
        { "MPI_Testsome", &enable_MPI_Testsome },
        
        { "MPI_Topo_test", &enable_MPI_Topo_test },
        
        { "MPI_Type_commit", &enable_MPI_Type_commit },
        
        { "MPI_Type_contiguous", &enable_MPI_Type_contiguous },
        
        { "MPI_Type_create_darray", &enable_MPI_Type_create_darray },
        
        { "MPI_Type_create_f90_complex", &enable_MPI_Type_create_f90_complex },
        
        { "MPI_Type_create_f90_integer", &enable_MPI_Type_create_f90_integer },
        
        { "MPI_Type_create_f90_real", &enable_MPI_Type_create_f90_real },
        
        { "MPI_Type_create_hindexed", &enable_MPI_Type_create_hindexed },
        
        { "MPI_Type_create_hindexed_block", &enable_MPI_Type_create_hindexed_block },
        
        { "MPI_Type_create_hvector", &enable_MPI_Type_create_hvector },
        
        { "MPI_Type_create_indexed_block", &enable_MPI_Type_create_indexed_block },
        
        { "MPI_Type_create_keyval", &enable_MPI_Type_create_keyval },
        
        { "MPI_Type_create_resized", &enable_MPI_Type_create_resized },
        
        { "MPI_Type_create_struct", &enable_MPI_Type_create_struct },
        
        { "MPI_Type_create_subarray", &enable_MPI_Type_create_subarray },
        
        { "MPI_Type_delete_attr", &enable_MPI_Type_delete_attr },
        
        { "MPI_Type_dup", &enable_MPI_Type_dup },
        
        { "MPI_Type_extent", &enable_MPI_Type_extent },
        
        { "MPI_Type_free", &enable_MPI_Type_free },
        
        { "MPI_Type_free_keyval", &enable_MPI_Type_free_keyval },
        
        { "MPI_Type_get_attr", &enable_MPI_Type_get_attr },
        
        { "MPI_Type_get_contents", &enable_MPI_Type_get_contents },
        
        { "MPI_Type_get_envelope", &enable_MPI_Type_get_envelope },
        
        { "MPI_Type_get_extent", &enable_MPI_Type_get_extent },
        
        { "MPI_Type_get_extent_x", &enable_MPI_Type_get_extent_x },
        
        { "MPI_Type_get_name", &enable_MPI_Type_get_name },
        
        { "MPI_Type_get_true_extent", &enable_MPI_Type_get_true_extent },
        
        { "MPI_Type_get_true_extent_x", &enable_MPI_Type_get_true_extent_x },
        
        { "MPI_Type_hindexed", &enable_MPI_Type_hindexed },
        
        { "MPI_Type_hvector", &enable_MPI_Type_hvector },
        
        { "MPI_Type_indexed", &enable_MPI_Type_indexed },
        
        { "MPI_Type_lb", &enable_MPI_Type_lb },
        
        { "MPI_Type_match_size", &enable_MPI_Type_match_size },
        
        { "MPI_Type_set_attr", &enable_MPI_Type_set_attr },
        
        { "MPI_Type_set_name", &enable_MPI_Type_set_name },
        
        { "MPI_Type_size", &enable_MPI_Type_size },
        
        { "MPI_Type_size_x", &enable_MPI_Type_size_x },
        
        { "MPI_Type_struct", &enable_MPI_Type_struct },
        
        { "MPI_Type_ub", &enable_MPI_Type_ub },
        
        { "MPI_Type_vector", &enable_MPI_Type_vector },
        
        { "MPI_Unpack", &enable_MPI_Unpack },
        
        { "MPI_Unpack_external", &enable_MPI_Unpack_external },
        
        { "MPI_Unpublish_name", &enable_MPI_Unpublish_name },
        
        { "MPI_Wait", &enable_MPI_Wait },
        
        { "MPI_Waitall", &enable_MPI_Waitall },
        
        { "MPI_Waitany", &enable_MPI_Waitany },
        
        { "MPI_Waitsome", &enable_MPI_Waitsome },
        
        { "MPI_Win_allocate", &enable_MPI_Win_allocate },
        
        { "MPI_Win_allocate_shared", &enable_MPI_Win_allocate_shared },
        
        { "MPI_Win_attach", &enable_MPI_Win_attach },
        
        { "MPI_Win_call_errhandler", &enable_MPI_Win_call_errhandler },
        
        { "MPI_Win_complete", &enable_MPI_Win_complete },
        
        { "MPI_Win_create", &enable_MPI_Win_create },
        
        { "MPI_Win_create_dynamic", &enable_MPI_Win_create_dynamic },
        
        { "MPI_Win_create_errhandler", &enable_MPI_Win_create_errhandler },
        
        { "MPI_Win_create_keyval", &enable_MPI_Win_create_keyval },
        
        { "MPI_Win_delete_attr", &enable_MPI_Win_delete_attr },
        
        { "MPI_Win_detach", &enable_MPI_Win_detach },
        
        { "MPI_Win_fence", &enable_MPI_Win_fence },
        
        { "MPI_Win_flush", &enable_MPI_Win_flush },
        
        { "MPI_Win_flush_all", &enable_MPI_Win_flush_all },
        
        { "MPI_Win_flush_local", &enable_MPI_Win_flush_local },
        
        { "MPI_Win_flush_local_all", &enable_MPI_Win_flush_local_all },
        
        { "MPI_Win_free", &enable_MPI_Win_free },
        
        { "MPI_Win_free_keyval", &enable_MPI_Win_free_keyval },
        
        { "MPI_Win_get_attr", &enable_MPI_Win_get_attr },
        
        { "MPI_Win_get_errhandler", &enable_MPI_Win_get_errhandler },
        
        { "MPI_Win_get_group", &enable_MPI_Win_get_group },
        
        { "MPI_Win_get_info", &enable_MPI_Win_get_info },
        
        { "MPI_Win_get_name", &enable_MPI_Win_get_name },
        
        { "MPI_Win_lock", &enable_MPI_Win_lock },
        
        { "MPI_Win_lock_all", &enable_MPI_Win_lock_all },
        
        { "MPI_Win_post", &enable_MPI_Win_post },
        
        { "MPI_Win_set_attr", &enable_MPI_Win_set_attr },
        
        { "MPI_Win_set_errhandler", &enable_MPI_Win_set_errhandler },
        
        { "MPI_Win_set_info", &enable_MPI_Win_set_info },
        
        { "MPI_Win_set_name", &enable_MPI_Win_set_name },
        
        { "MPI_Win_shared_query", &enable_MPI_Win_shared_query },
        
        { "MPI_Win_start", &enable_MPI_Win_start },
        
        { "MPI_Win_sync", &enable_MPI_Win_sync },
        
        { "MPI_Win_test", &enable_MPI_Win_test },
        
        { "MPI_Win_unlock", &enable_MPI_Win_unlock },
        
        { "MPI_Win_unlock_all", &enable_MPI_Win_unlock_all },
        
        { "MPI_Win_wait", &enable_MPI_Win_wait },
        
        { "MPI_Wtick", &enable_MPI_Wtick },
        
        { "MPI_Wtime", &enable_MPI_Wtime },
        
        { 0, 0 }
    };

    for (const fntable_elem* e = table; e->name && e->enableptr; ++e) {
        if (enable_all)
            *(e->enableptr) = true;

        std::string fnstr(e->name);

        if (have_whitelist) {
            vector<string>::iterator it = std::find(whitelist.begin(), whitelist.end(), fnstr);

            if (it != whitelist.end()) {
                *(e->enableptr) = true;
                whitelist.erase(it);
            }
        }
        if (have_blacklist) {
            vector<string>::iterator it = std::find(blacklist.begin(), blacklist.end(), fnstr);

            if (it != blacklist.end()) {
                blacklist.erase(it);
                *(e->enableptr) = false;
            } else if (!have_whitelist) {
                *(e->enableptr) = true;
            }
        }
    }

    for (vector<string>::const_iterator it = whitelist.begin(); it != whitelist.end(); ++it)
        Log(1).stream() << "Unknown MPI function " << *it << " in MPI function whitelist" << endl;
    for (vector<string>::const_iterator it = blacklist.begin(); it != blacklist.end(); ++it)
        Log(1).stream() << "Unknown MPI function " << *it << " in MPI function blacklist" << endl;
}

void
mpi_init_cb(Caliper* c)
{
    int rank = -1, size = -1;

    PMPI_Comm_rank(MPI_COMM_WORLD, &rank);
    PMPI_Comm_size(MPI_COMM_WORLD, &size);

    c->set(mpirank_attr, Variant(rank));
    c->set(mpisize_attr, Variant(size));

    ::tracing.init_mpi(c);
}

void
post_init_cb(Caliper* c)
{
    int initialized = 0;
    int finalized   = 0;

    PMPI_Initialized(&initialized);
    PMPI_Finalized(&finalized);

    if (initialized && !finalized)
        MpiEvents::events.mpi_init_evt(c);
}

} // namespace [anonymous]


//
// --- Point-to-Point
//

/* ================== C Wrappers for MPI_Send ================== */
_EXTERN_C_ int PMPI_Send(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm);
_EXTERN_C_ int MPI_Send(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Send) {
#endif
        Caliper c;

        if (enable_msg_tracing)
            ::tracing.push_call_id(&c);
        
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Send", strlen("MPI_Send")));        

        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Send(buf, count, datatype, dest, tag, comm);
WRAP_MPI_CALL_POSTFIX

        if (enable_msg_tracing)
            ::tracing.handle_send(&c, count, datatype, dest, tag, comm);

        c.end(mpifn_attr);

        if (enable_msg_tracing)
            ::tracing.pop_call_id(&c);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Send(buf, count, datatype, dest, tag, comm);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Bsend ================== */
_EXTERN_C_ int PMPI_Bsend(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm);
_EXTERN_C_ int MPI_Bsend(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Bsend) {
#endif
        Caliper c;

        if (enable_msg_tracing)
            ::tracing.push_call_id(&c);
        
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Bsend", strlen("MPI_Bsend")));        

        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Bsend(buf, count, datatype, dest, tag, comm);
WRAP_MPI_CALL_POSTFIX

        if (enable_msg_tracing)
            ::tracing.handle_send(&c, count, datatype, dest, tag, comm);

        c.end(mpifn_attr);

        if (enable_msg_tracing)
            ::tracing.pop_call_id(&c);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Bsend(buf, count, datatype, dest, tag, comm);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Rsend ================== */
_EXTERN_C_ int PMPI_Rsend(const void *ibuf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm);
_EXTERN_C_ int MPI_Rsend(const void *ibuf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Rsend) {
#endif
        Caliper c;

        if (enable_msg_tracing)
            ::tracing.push_call_id(&c);
        
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Rsend", strlen("MPI_Rsend")));        

        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Rsend(ibuf, count, datatype, dest, tag, comm);
WRAP_MPI_CALL_POSTFIX

        if (enable_msg_tracing)
            ::tracing.handle_send(&c, count, datatype, dest, tag, comm);

        c.end(mpifn_attr);

        if (enable_msg_tracing)
            ::tracing.pop_call_id(&c);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Rsend(ibuf, count, datatype, dest, tag, comm);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Ssend ================== */
_EXTERN_C_ int PMPI_Ssend(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm);
_EXTERN_C_ int MPI_Ssend(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Ssend) {
#endif
        Caliper c;

        if (enable_msg_tracing)
            ::tracing.push_call_id(&c);
        
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Ssend", strlen("MPI_Ssend")));        

        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Ssend(buf, count, datatype, dest, tag, comm);
WRAP_MPI_CALL_POSTFIX

        if (enable_msg_tracing)
            ::tracing.handle_send(&c, count, datatype, dest, tag, comm);

        c.end(mpifn_attr);

        if (enable_msg_tracing)
            ::tracing.pop_call_id(&c);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Ssend(buf, count, datatype, dest, tag, comm);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Isend ================== */
_EXTERN_C_ int PMPI_Isend(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Isend(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Isend) {
#endif
        Caliper c;

        if (enable_msg_tracing)
            ::tracing.push_call_id(&c);
        
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Isend", strlen("MPI_Isend")));        

        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Isend(buf, count, datatype, dest, tag, comm, request);
WRAP_MPI_CALL_POSTFIX

        if (enable_msg_tracing)
            ::tracing.handle_send(&c, count, datatype, dest, tag, comm);

        c.end(mpifn_attr);

        if (enable_msg_tracing)
            ::tracing.pop_call_id(&c);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Isend(buf, count, datatype, dest, tag, comm, request);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Ibsend ================== */
_EXTERN_C_ int PMPI_Ibsend(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Ibsend(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Ibsend) {
#endif
        Caliper c;

        if (enable_msg_tracing)
            ::tracing.push_call_id(&c);
        
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Ibsend", strlen("MPI_Ibsend")));        

        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Ibsend(buf, count, datatype, dest, tag, comm, request);
WRAP_MPI_CALL_POSTFIX

        if (enable_msg_tracing)
            ::tracing.handle_send(&c, count, datatype, dest, tag, comm);

        c.end(mpifn_attr);

        if (enable_msg_tracing)
            ::tracing.pop_call_id(&c);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Ibsend(buf, count, datatype, dest, tag, comm, request);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Irsend ================== */
_EXTERN_C_ int PMPI_Irsend(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Irsend(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Irsend) {
#endif
        Caliper c;

        if (enable_msg_tracing)
            ::tracing.push_call_id(&c);
        
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Irsend", strlen("MPI_Irsend")));        

        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Irsend(buf, count, datatype, dest, tag, comm, request);
WRAP_MPI_CALL_POSTFIX

        if (enable_msg_tracing)
            ::tracing.handle_send(&c, count, datatype, dest, tag, comm);

        c.end(mpifn_attr);

        if (enable_msg_tracing)
            ::tracing.pop_call_id(&c);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Irsend(buf, count, datatype, dest, tag, comm, request);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Issend ================== */
_EXTERN_C_ int PMPI_Issend(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Issend(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Issend) {
#endif
        Caliper c;

        if (enable_msg_tracing)
            ::tracing.push_call_id(&c);
        
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Issend", strlen("MPI_Issend")));        

        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Issend(buf, count, datatype, dest, tag, comm, request);
WRAP_MPI_CALL_POSTFIX

        if (enable_msg_tracing)
            ::tracing.handle_send(&c, count, datatype, dest, tag, comm);

        c.end(mpifn_attr);

        if (enable_msg_tracing)
            ::tracing.pop_call_id(&c);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Issend(buf, count, datatype, dest, tag, comm, request);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}



/* ================== C Wrappers for MPI_Send_init ================== */
_EXTERN_C_ int PMPI_Send_init(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Send_init(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Send_init) {
#endif
        Caliper c;

        if (enable_msg_tracing)
            ::tracing.push_call_id(&c);
        
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Send_init", strlen("MPI_Send_init")));        

        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Send_init(buf, count, datatype, dest, tag, comm, request);
WRAP_MPI_CALL_POSTFIX

        if (enable_msg_tracing)
            ::tracing.handle_send_init(&c, count, datatype, dest, tag, comm, request);

        c.end(mpifn_attr);

        if (enable_msg_tracing)
            ::tracing.pop_call_id(&c);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Send_init(buf, count, datatype, dest, tag, comm, request);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Bsend_init ================== */
_EXTERN_C_ int PMPI_Bsend_init(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Bsend_init(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Bsend_init) {
#endif
        Caliper c;

        if (enable_msg_tracing)
            ::tracing.push_call_id(&c);
        
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Bsend_init", strlen("MPI_Bsend_init")));        

        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Bsend_init(buf, count, datatype, dest, tag, comm, request);
WRAP_MPI_CALL_POSTFIX

        if (enable_msg_tracing)
            ::tracing.handle_send_init(&c, count, datatype, dest, tag, comm, request);

        c.end(mpifn_attr);

        if (enable_msg_tracing)
            ::tracing.pop_call_id(&c);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Bsend_init(buf, count, datatype, dest, tag, comm, request);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Rsend_init ================== */
_EXTERN_C_ int PMPI_Rsend_init(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Rsend_init(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Rsend_init) {
#endif
        Caliper c;

        if (enable_msg_tracing)
            ::tracing.push_call_id(&c);
        
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Rsend_init", strlen("MPI_Rsend_init")));        

        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Rsend_init(buf, count, datatype, dest, tag, comm, request);
WRAP_MPI_CALL_POSTFIX

        if (enable_msg_tracing)
            ::tracing.handle_send_init(&c, count, datatype, dest, tag, comm, request);

        c.end(mpifn_attr);

        if (enable_msg_tracing)
            ::tracing.pop_call_id(&c);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Rsend_init(buf, count, datatype, dest, tag, comm, request);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Ssend_init ================== */
_EXTERN_C_ int PMPI_Ssend_init(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Ssend_init(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Ssend_init) {
#endif
        Caliper c;

        if (enable_msg_tracing)
            ::tracing.push_call_id(&c);
        
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Ssend_init", strlen("MPI_Ssend_init")));        

        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Ssend_init(buf, count, datatype, dest, tag, comm, request);
WRAP_MPI_CALL_POSTFIX

        if (enable_msg_tracing)
            ::tracing.handle_send_init(&c, count, datatype, dest, tag, comm, request);

        c.end(mpifn_attr);

        if (enable_msg_tracing)
            ::tracing.pop_call_id(&c);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Ssend_init(buf, count, datatype, dest, tag, comm, request);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}



/* ================== C Wrappers for MPI_Recv ================== */
_EXTERN_C_ int PMPI_Recv(void *buf, int count, MPI_Datatype datatype, int source, int tag, MPI_Comm comm, MPI_Status *status);
_EXTERN_C_ int MPI_Recv(void *buf, int count, MPI_Datatype datatype, int source, int tag, MPI_Comm comm, MPI_Status *status) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Recv) {
#endif
        Caliper c;

        if (enable_msg_tracing)
            ::tracing.push_call_id(&c);
        
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Recv", strlen("MPI_Recv")));

        MPI_Status tmp_status;
        
        if (status == MPI_STATUS_IGNORE)
            status = &tmp_status;

        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Recv(buf, count, datatype, source, tag, comm, status);
WRAP_MPI_CALL_POSTFIX

        if (enable_msg_tracing)
            ::tracing.handle_recv(&c, count, datatype, source, tag, comm, status);
        
        c.end(mpifn_attr);

        if (enable_msg_tracing)
            ::tracing.pop_call_id(&c);        
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Recv(buf, count, datatype, source, tag, comm, status);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}



/* ================== C Wrappers for MPI_Sendrecv ================== */
_EXTERN_C_ int PMPI_Sendrecv(const void *sendbuf, int sendcount, MPI_Datatype sendtype, int dest, int sendtag, void *recvbuf, int recvcount, MPI_Datatype recvtype, int source, int recvtag, MPI_Comm comm, MPI_Status *status);
_EXTERN_C_ int MPI_Sendrecv(const void *sendbuf, int sendcount, MPI_Datatype sendtype, int dest, int sendtag, void *recvbuf, int recvcount, MPI_Datatype recvtype, int source, int recvtag, MPI_Comm comm, MPI_Status *status) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Sendrecv) {
#endif
        Caliper c;

        if (enable_msg_tracing)
            ::tracing.push_call_id(&c);
        
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Sendrecv", strlen("MPI_Sendrecv")));

        MPI_Status tmp_status;
        
        if (status == MPI_STATUS_IGNORE)
            status = &tmp_status;

        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Sendrecv(sendbuf, sendcount, sendtype, dest, sendtag, recvbuf, recvcount, recvtype, source, recvtag, comm, status);
WRAP_MPI_CALL_POSTFIX

        if (enable_msg_tracing) {
            ::tracing.handle_send(&c, sendcount, sendtype, dest, sendtag, comm);
            ::tracing.handle_recv(&c, recvcount, recvtype, source, recvtag, comm, status);
        }
        
        c.end(mpifn_attr);

        if (enable_msg_tracing)
            ::tracing.pop_call_id(&c);        
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Sendrecv(sendbuf, sendcount, sendtype, dest, sendtag, recvbuf, recvcount, recvtype, source, recvtag, comm, status);
WRAP_MPI_CALL_POSTFIX
    }
#endif    
}    return _wrap_py_return_val;
}



/* ================== C Wrappers for MPI_Sendrecv_replace ================== */
_EXTERN_C_ int PMPI_Sendrecv_replace(void *buf, int count, MPI_Datatype datatype, int dest, int sendtag, int source, int recvtag, MPI_Comm comm, MPI_Status *status);
_EXTERN_C_ int MPI_Sendrecv_replace(void *buf, int count, MPI_Datatype datatype, int dest, int sendtag, int source, int recvtag, MPI_Comm comm, MPI_Status *status) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Sendrecv_replace) {
#endif
        Caliper c;

        if (enable_msg_tracing)
            ::tracing.push_call_id(&c);
        
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Sendrecv_replace", strlen("MPI_Sendrecv_replace")));

        MPI_Status tmp_status;
        
        if (status == MPI_STATUS_IGNORE)
            status = &tmp_status;

        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Sendrecv_replace(buf, count, datatype, dest, sendtag, source, recvtag, comm, status);
WRAP_MPI_CALL_POSTFIX

        if (enable_msg_tracing) {
            ::tracing.handle_send(&c, count, datatype, dest, sendtag, comm);
            ::tracing.handle_recv(&c, count, datatype, source, recvtag, comm, status);
        }
        
        c.end(mpifn_attr);

        if (enable_msg_tracing)
            ::tracing.pop_call_id(&c);        
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Sendrecv_replace(buf, count, datatype, dest, sendtag, source, recvtag, comm, status);
WRAP_MPI_CALL_POSTFIX
    }
#endif    
}    return _wrap_py_return_val;
}



/* ================== C Wrappers for MPI_Irecv ================== */
_EXTERN_C_ int PMPI_Irecv(void *buf, int count, MPI_Datatype datatype, int source, int tag, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Irecv(void *buf, int count, MPI_Datatype datatype, int source, int tag, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Irecv) {
#endif
        Caliper c;

        if (enable_msg_tracing)
            ::tracing.push_call_id(&c);
        
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Irecv", strlen("MPI_Irecv")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Irecv(buf, count, datatype, source, tag, comm, request);
WRAP_MPI_CALL_POSTFIX
        
        if (enable_msg_tracing)
            ::tracing.handle_irecv(&c, count, datatype, source, tag, comm, request);

        c.end(mpifn_attr);

        if (enable_msg_tracing)
            ::tracing.pop_call_id(&c);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Irecv(buf, count, datatype, source, tag, comm, request);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}



/* ================== C Wrappers for MPI_Recv_init ================== */
_EXTERN_C_ int PMPI_Recv_init(void *buf, int count, MPI_Datatype datatype, int source, int tag, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Recv_init(void *buf, int count, MPI_Datatype datatype, int source, int tag, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Recv_init) {
#endif
        Caliper c;

        if (enable_msg_tracing)
            ::tracing.push_call_id(&c);
        
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Recv_init", strlen("MPI_Recv_init")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Recv_init(buf, count, datatype, source, tag, comm, request);
WRAP_MPI_CALL_POSTFIX
        
        if (enable_msg_tracing)
            ::tracing.handle_recv_init(&c, count, datatype, source, tag, comm, request);

        c.end(mpifn_attr);

        if (enable_msg_tracing)
            ::tracing.pop_call_id(&c);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Recv_init(buf, count, datatype, source, tag, comm, request);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}



/* ================== C Wrappers for MPI_Start ================== */
_EXTERN_C_ int PMPI_Start(MPI_Request *request);
_EXTERN_C_ int MPI_Start(MPI_Request *request) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Start) {
#endif
        Caliper c;

        if (enable_msg_tracing)
            ::tracing.push_call_id(&c);

        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Start", strlen("MPI_Start")));
        
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Start(request);
WRAP_MPI_CALL_POSTFIX
        
        if (enable_msg_tracing)
            ::tracing.handle_start(&c, 1, request);

        c.end(mpifn_attr);
        
        if (enable_msg_tracing)
            ::tracing.pop_call_id(&c);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Start(request);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}



/* ================== C Wrappers for MPI_Startall ================== */
_EXTERN_C_ int PMPI_Startall(int count, MPI_Request array_of_requests[]);
_EXTERN_C_ int MPI_Startall(int count, MPI_Request array_of_requests[]) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Startall) {
#endif
        Caliper c;

        if (enable_msg_tracing)
            ::tracing.push_call_id(&c);

        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Startall", strlen("MPI_Startall")));
        
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Startall(count, array_of_requests);
WRAP_MPI_CALL_POSTFIX
        
        if (enable_msg_tracing)
            ::tracing.handle_start(&c, count, array_of_requests);

        c.end(mpifn_attr);
        
        if (enable_msg_tracing)
            ::tracing.pop_call_id(&c);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Startall(count, array_of_requests);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}



/* ================== C Wrappers for MPI_Wait ================== */
_EXTERN_C_ int PMPI_Wait(MPI_Request *request, MPI_Status *status);
_EXTERN_C_ int MPI_Wait(MPI_Request *request, MPI_Status *status) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Wait) {
#endif
        Caliper c;

        if (enable_msg_tracing)
            ::tracing.push_call_id(&c);
        
        MPI_Request tmp_req = *request;
        MPI_Status  tmp_status;

        if (status == MPI_STATUS_IGNORE)
            status = &tmp_status;

        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Wait", strlen("MPI_Wait")));
        
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Wait(request, status);
WRAP_MPI_CALL_POSTFIX
        
        if (enable_msg_tracing)
            ::tracing.handle_completion(&c, 1, &tmp_req, status);

        c.end(mpifn_attr);
        
        if (enable_msg_tracing)
            ::tracing.pop_call_id(&c);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Wait(request, status);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}



/* ================== C Wrappers for MPI_Waitall ================== */
_EXTERN_C_ int PMPI_Waitall(int count, MPI_Request array_of_requests[], MPI_Status *array_of_statuses);
_EXTERN_C_ int MPI_Waitall(int count, MPI_Request array_of_requests[], MPI_Status *array_of_statuses) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Waitall) {
#endif
        Caliper c;

        int nreq = count;
        
        MPI_Request* tmp_req      = nullptr;
        MPI_Status*  tmp_statuses = nullptr;

        if (enable_msg_tracing) {
            ::tracing.push_call_id(&c);

            tmp_req      = new MPI_Request[nreq];
            tmp_statuses = new MPI_Status[nreq];

            std::copy_n(array_of_requests, nreq, tmp_req);

            if (array_of_statuses == MPI_STATUSES_IGNORE)
                array_of_statuses = tmp_statuses;
        }

        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Waitall", strlen("MPI_Waitall")));
        
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Waitall(count, array_of_requests, array_of_statuses);
WRAP_MPI_CALL_POSTFIX
        
        if (enable_msg_tracing)
            ::tracing.handle_completion(&c, nreq, tmp_req, array_of_statuses);

        c.end(mpifn_attr);
        
        if (enable_msg_tracing) {
            delete[] tmp_statuses;
            delete[] tmp_req;

            ::tracing.pop_call_id(&c);
        }
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Waitall(count, array_of_requests, array_of_statuses);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}



/* ================== C Wrappers for MPI_Waitany ================== */
_EXTERN_C_ int PMPI_Waitany(int count, MPI_Request array_of_requests[], int *index, MPI_Status *status);
_EXTERN_C_ int MPI_Waitany(int count, MPI_Request array_of_requests[], int *index, MPI_Status *status) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Waitany) {
#endif
        Caliper c;

        int nreq = count;
        
        MPI_Request* tmp_req = nullptr;
        MPI_Status   tmp_status;

        if (enable_msg_tracing) {
            ::tracing.push_call_id(&c);

            tmp_req = new MPI_Request[nreq];
            std::copy_n(array_of_requests, nreq, tmp_req);

            if (status == MPI_STATUS_IGNORE)
                status = &tmp_status;
        }

        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Waitany", strlen("MPI_Waitany")));
        
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Waitany(count, array_of_requests, index, status);
WRAP_MPI_CALL_POSTFIX
        
        if (enable_msg_tracing && nreq > 0)
            ::tracing.handle_completion(&c, 1, tmp_req+(*index), status);

        c.end(mpifn_attr);
        
        if (enable_msg_tracing) {
            delete[] tmp_req;

            ::tracing.pop_call_id(&c);
        }
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Waitany(count, array_of_requests, index, status);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}



/* ================== C Wrappers for MPI_Testsome ================== */
_EXTERN_C_ int PMPI_Testsome(int incount, MPI_Request array_of_requests[], int *outcount, int array_of_indices[], MPI_Status array_of_statuses[]);
_EXTERN_C_ int MPI_Testsome(int incount, MPI_Request array_of_requests[], int *outcount, int array_of_indices[], MPI_Status array_of_statuses[]) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Testsome) {
#endif
        Caliper c;

        int nreq = incount;
        
        MPI_Request* tmp_req = nullptr;
        MPI_Status*  tmp_statuses;

        if (enable_msg_tracing) {
            ::tracing.push_call_id(&c);

            tmp_req      = new MPI_Request[nreq];
            tmp_statuses = new MPI_Status[nreq];
                
            std::copy_n(array_of_requests, nreq, tmp_req);

            if (array_of_statuses == MPI_STATUSES_IGNORE)
                array_of_statuses = tmp_statuses;
        }

        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Testsome", strlen("MPI_Testsome")));
        
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Testsome(incount, array_of_requests, outcount, array_of_indices, array_of_statuses);
WRAP_MPI_CALL_POSTFIX
        
        if (enable_msg_tracing > 0)
            for (int i = 0; i < *outcount; ++i)
                ::tracing.handle_completion(&c, 1, tmp_req+array_of_indices[i], array_of_statuses+array_of_indices[i]);

        c.end(mpifn_attr);
        
        if (enable_msg_tracing) {
            delete[] tmp_statuses;
            delete[] tmp_req;

            ::tracing.pop_call_id(&c);
        }
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Testsome(incount, array_of_requests, outcount, array_of_indices, array_of_statuses);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Waitsome ================== */
_EXTERN_C_ int PMPI_Waitsome(int incount, MPI_Request array_of_requests[], int *outcount, int array_of_indices[], MPI_Status array_of_statuses[]);
_EXTERN_C_ int MPI_Waitsome(int incount, MPI_Request array_of_requests[], int *outcount, int array_of_indices[], MPI_Status array_of_statuses[]) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Waitsome) {
#endif
        Caliper c;

        int nreq = incount;
        
        MPI_Request* tmp_req = nullptr;
        MPI_Status*  tmp_statuses;

        if (enable_msg_tracing) {
            ::tracing.push_call_id(&c);

            tmp_req      = new MPI_Request[nreq];
            tmp_statuses = new MPI_Status[nreq];
                
            std::copy_n(array_of_requests, nreq, tmp_req);

            if (array_of_statuses == MPI_STATUSES_IGNORE)
                array_of_statuses = tmp_statuses;
        }

        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Waitsome", strlen("MPI_Waitsome")));
        
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Waitsome(incount, array_of_requests, outcount, array_of_indices, array_of_statuses);
WRAP_MPI_CALL_POSTFIX
        
        if (enable_msg_tracing > 0)
            for (int i = 0; i < *outcount; ++i)
                ::tracing.handle_completion(&c, 1, tmp_req+array_of_indices[i], array_of_statuses+array_of_indices[i]);

        c.end(mpifn_attr);
        
        if (enable_msg_tracing) {
            delete[] tmp_statuses;
            delete[] tmp_req;

            ::tracing.pop_call_id(&c);
        }
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Waitsome(incount, array_of_requests, outcount, array_of_indices, array_of_statuses);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}



/* ================== C Wrappers for MPI_Test ================== */
_EXTERN_C_ int PMPI_Test(MPI_Request *request, int *flag, MPI_Status *status);
_EXTERN_C_ int MPI_Test(MPI_Request *request, int *flag, MPI_Status *status) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Test) {
#endif
        Caliper c;

        if (enable_msg_tracing)
            ::tracing.push_call_id(&c);
        
        MPI_Request tmp_req = *request;
        MPI_Status  tmp_status;

        if (status == MPI_STATUS_IGNORE)
            status = &tmp_status;

        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Test", strlen("MPI_Test")));
        
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Test(request, flag, status);
WRAP_MPI_CALL_POSTFIX
        
        if (enable_msg_tracing && *flag)
            ::tracing.handle_completion(&c, 1, &tmp_req, status);

        c.end(mpifn_attr);
        
        if (enable_msg_tracing)
            ::tracing.pop_call_id(&c);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Test(request, flag, status);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}



/* ================== C Wrappers for MPI_Testall ================== */
_EXTERN_C_ int PMPI_Testall(int count, MPI_Request array_of_requests[], int *flag, MPI_Status array_of_statuses[]);
_EXTERN_C_ int MPI_Testall(int count, MPI_Request array_of_requests[], int *flag, MPI_Status array_of_statuses[]) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Testall) {
#endif
        Caliper c;

        int nreq = count;
        
        MPI_Request* tmp_req      = nullptr;
        MPI_Status*  tmp_statuses = nullptr;

        if (enable_msg_tracing) {
            ::tracing.push_call_id(&c);

            tmp_req      = new MPI_Request[nreq];
            tmp_statuses = new MPI_Status[nreq];

            std::copy_n(array_of_requests, nreq, tmp_req);

            if (array_of_statuses == MPI_STATUSES_IGNORE)
                array_of_statuses = tmp_statuses;
        }

        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Testall", strlen("MPI_Testall")));
        
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Testall(count, array_of_requests, flag, array_of_statuses);
WRAP_MPI_CALL_POSTFIX
        
        if (enable_msg_tracing && *flag)
            ::tracing.handle_completion(&c, nreq, tmp_req, array_of_statuses);

        c.end(mpifn_attr);
        
        if (enable_msg_tracing) {
            delete[] tmp_statuses;
            delete[] tmp_req;

            ::tracing.pop_call_id(&c);
        }
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Testall(count, array_of_requests, flag, array_of_statuses);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}



/* ================== C Wrappers for MPI_Testany ================== */
_EXTERN_C_ int PMPI_Testany(int count, MPI_Request array_of_requests[], int *index, int *flag, MPI_Status *status);
_EXTERN_C_ int MPI_Testany(int count, MPI_Request array_of_requests[], int *index, int *flag, MPI_Status *status) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Testany) {
#endif
        Caliper c;

        int nreq = count;
        
        MPI_Request* tmp_req = nullptr;
        MPI_Status   tmp_status;

        if (enable_msg_tracing) {
            ::tracing.push_call_id(&c);

            tmp_req = new MPI_Request[nreq];
            std::copy_n(array_of_requests, nreq, tmp_req);

            if (status == MPI_STATUS_IGNORE)
                status = &tmp_status;
        }

        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Testany", strlen("MPI_Testany")));
        
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Testany(count, array_of_requests, index, flag, status);
WRAP_MPI_CALL_POSTFIX
        
        if (enable_msg_tracing && *flag)
            ::tracing.handle_completion(&c, 1, tmp_req+(*index), status);

        c.end(mpifn_attr);
        
        if (enable_msg_tracing) {
            delete[] tmp_req;

            ::tracing.pop_call_id(&c);
        }
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Testany(count, array_of_requests, index, flag, status);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}



/* ================== C Wrappers for MPI_Request_free ================== */
_EXTERN_C_ int PMPI_Request_free(MPI_Request *request);
_EXTERN_C_ int MPI_Request_free(MPI_Request *request) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Request_free) {
#endif
        Caliper c;

        if (enable_msg_tracing)
            ::tracing.push_call_id(&c);

        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Request_free", strlen("MPI_Request_free")));
        
        if (enable_msg_tracing)
            ::tracing.request_free(&c, request);

        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Request_free(request);
WRAP_MPI_CALL_POSTFIX
        
        c.end(mpifn_attr);
        
        if (enable_msg_tracing)
            ::tracing.pop_call_id(&c);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Request_free(request);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}



//
// --- Collectives
//

/* ================== C Wrappers for MPI_Barrier ================== */
_EXTERN_C_ int PMPI_Barrier(MPI_Comm comm);
_EXTERN_C_ int MPI_Barrier(MPI_Comm comm) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Barrier) {
#endif
        Caliper c;

        if (enable_msg_tracing)
            ::tracing.push_call_id(&c);

        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Barrier", strlen("MPI_Barrier")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Barrier(comm);
WRAP_MPI_CALL_POSTFIX

        if (enable_msg_tracing)
            ::tracing.handle_barrier(&c, comm);

        c.end(mpifn_attr);
        
        if (enable_msg_tracing)
            ::tracing.pop_call_id(&c);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Barrier(comm);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}



/* ================== C Wrappers for MPI_Bcast ================== */
_EXTERN_C_ int PMPI_Bcast(void *buffer, int count, MPI_Datatype datatype, int root, MPI_Comm comm);
_EXTERN_C_ int MPI_Bcast(void *buffer, int count, MPI_Datatype datatype, int root, MPI_Comm comm) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Bcast) {
#endif
        Caliper c;

        if (enable_msg_tracing)
            ::tracing.push_call_id(&c);

        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Bcast", strlen("MPI_Bcast")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Bcast(buffer, count, datatype, root, comm);
WRAP_MPI_CALL_POSTFIX

        if (enable_msg_tracing)
            ::tracing.handle_12n(&c, count, datatype, root, comm);
        
        c.end(mpifn_attr);
        
        if (enable_msg_tracing)
            ::tracing.pop_call_id(&c);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Bcast(buffer, count, datatype, root, comm);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}



/* ================== C Wrappers for MPI_Scatter ================== */
_EXTERN_C_ int PMPI_Scatter(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm);
_EXTERN_C_ int MPI_Scatter(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Scatter) {
#endif
        Caliper c;

        if (enable_msg_tracing)
            ::tracing.push_call_id(&c);

        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Scatter", strlen("MPI_Scatter")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Scatter(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm);
WRAP_MPI_CALL_POSTFIX

        if (enable_msg_tracing)
            ::tracing.handle_12n(&c, sendcount, sendtype, root, comm);
        
        c.end(mpifn_attr);
        
        if (enable_msg_tracing)
            ::tracing.pop_call_id(&c);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Scatter(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}



/* ================== C Wrappers for MPI_Scatterv ================== */
_EXTERN_C_ int PMPI_Scatterv(const void *sendbuf, const int sendcounts[], const int displs[], MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm);
_EXTERN_C_ int MPI_Scatterv(const void *sendbuf, const int sendcounts[], const int displs[], MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Scatterv) {
#endif
        Caliper c;

        if (enable_msg_tracing)
            ::tracing.push_call_id(&c);

        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Scatterv", strlen("MPI_Scatterv")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Scatterv(sendbuf, sendcounts, displs, sendtype, recvbuf, recvcount, recvtype, root, comm);
WRAP_MPI_CALL_POSTFIX

        if (enable_msg_tracing) {
            int tmp_commsize = 0;
            PMPI_Comm_size(comm, &tmp_commsize);
            int total_count  = std::accumulate(displs, displs+tmp_commsize, 0);
            
            ::tracing.handle_12n(&c, total_count, sendtype, root, comm);
        }
        
        c.end(mpifn_attr);
        
        if (enable_msg_tracing)
            ::tracing.pop_call_id(&c);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Scatterv(sendbuf, sendcounts, displs, sendtype, recvbuf, recvcount, recvtype, root, comm);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}



/* ================== C Wrappers for MPI_Gather ================== */
_EXTERN_C_ int PMPI_Gather(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm);
_EXTERN_C_ int MPI_Gather(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Gather) {
#endif
        Caliper c;

        if (enable_msg_tracing)
            ::tracing.push_call_id(&c);

        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Gather", strlen("MPI_Gather")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Gather(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm);
WRAP_MPI_CALL_POSTFIX

        if (enable_msg_tracing)
            ::tracing.handle_n21(&c, sendcount, sendtype, root, comm);
        
        c.end(mpifn_attr);
        
        if (enable_msg_tracing)
            ::tracing.pop_call_id(&c);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Gather(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}



/* ================== C Wrappers for MPI_Gatherv ================== */
_EXTERN_C_ int PMPI_Gatherv(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, int root, MPI_Comm comm);
_EXTERN_C_ int MPI_Gatherv(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, int root, MPI_Comm comm) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Gatherv) {
#endif
        Caliper c;

        if (enable_msg_tracing)
            ::tracing.push_call_id(&c);

        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Gatherv", strlen("MPI_Gatherv")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Gatherv(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, root, comm);
WRAP_MPI_CALL_POSTFIX

        if (enable_msg_tracing)
            ::tracing.handle_n21(&c, sendcount, sendtype, root, comm);
        
        c.end(mpifn_attr);
        
        if (enable_msg_tracing)
            ::tracing.pop_call_id(&c);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Gatherv(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, root, comm);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}



/* ================== C Wrappers for MPI_Reduce ================== */
_EXTERN_C_ int PMPI_Reduce(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, int root, MPI_Comm comm);
_EXTERN_C_ int MPI_Reduce(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, int root, MPI_Comm comm) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Reduce) {
#endif
        Caliper c;

        if (enable_msg_tracing)
            ::tracing.push_call_id(&c);

        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Reduce", strlen("MPI_Reduce")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Reduce(sendbuf, recvbuf, count, datatype, op, root, comm);
WRAP_MPI_CALL_POSTFIX

        if (enable_msg_tracing)
            ::tracing.handle_n21(&c, count, datatype, root, comm);
        
        c.end(mpifn_attr);
        
        if (enable_msg_tracing)
            ::tracing.pop_call_id(&c);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Reduce(sendbuf, recvbuf, count, datatype, op, root, comm);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}



/* ================== C Wrappers for MPI_Scan ================== */
_EXTERN_C_ int PMPI_Scan(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm);
_EXTERN_C_ int MPI_Scan(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Scan) {
#endif
        Caliper c;

        if (enable_msg_tracing)
            ::tracing.push_call_id(&c);

        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Scan", strlen("MPI_Scan")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Scan(sendbuf, recvbuf, count, datatype, op, comm);
WRAP_MPI_CALL_POSTFIX

        if (enable_msg_tracing)
            ::tracing.handle_n2n(&c, count, datatype, comm);
        
        c.end(mpifn_attr);
        
        if (enable_msg_tracing)
            ::tracing.pop_call_id(&c);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Scan(sendbuf, recvbuf, count, datatype, op, comm);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Exscan ================== */
_EXTERN_C_ int PMPI_Exscan(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm);
_EXTERN_C_ int MPI_Exscan(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Exscan) {
#endif
        Caliper c;

        if (enable_msg_tracing)
            ::tracing.push_call_id(&c);

        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Exscan", strlen("MPI_Exscan")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Exscan(sendbuf, recvbuf, count, datatype, op, comm);
WRAP_MPI_CALL_POSTFIX

        if (enable_msg_tracing)
            ::tracing.handle_n2n(&c, count, datatype, comm);
        
        c.end(mpifn_attr);
        
        if (enable_msg_tracing)
            ::tracing.pop_call_id(&c);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Exscan(sendbuf, recvbuf, count, datatype, op, comm);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}



/* ================== C Wrappers for MPI_Reduce_scatter ================== */
_EXTERN_C_ int PMPI_Reduce_scatter(const void *sendbuf, void *recvbuf, const int recvcounts[], MPI_Datatype datatype, MPI_Op op, MPI_Comm comm);
_EXTERN_C_ int MPI_Reduce_scatter(const void *sendbuf, void *recvbuf, const int recvcounts[], MPI_Datatype datatype, MPI_Op op, MPI_Comm comm) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Reduce_scatter) {
#endif
        Caliper c;

        if (enable_msg_tracing)
            ::tracing.push_call_id(&c);

        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Reduce_scatter", strlen("MPI_Reduce_scatter")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Reduce_scatter(sendbuf, recvbuf, recvcounts, datatype, op, comm);
WRAP_MPI_CALL_POSTFIX

        if (enable_msg_tracing) {
            int tmp_rank = 0;
            PMPI_Comm_rank(comm, &tmp_rank);

            ::tracing.handle_n2n(&c, recvcounts[tmp_rank], datatype, comm);
        }
        
        c.end(mpifn_attr);
        
        if (enable_msg_tracing)
            ::tracing.pop_call_id(&c);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Reduce_scatter(sendbuf, recvbuf, recvcounts, datatype, op, comm);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}



/* ================== C Wrappers for MPI_Allreduce ================== */
_EXTERN_C_ int PMPI_Allreduce(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm);
_EXTERN_C_ int MPI_Allreduce(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Allreduce) {
#endif
        Caliper c;

        if (enable_msg_tracing)
            ::tracing.push_call_id(&c);

        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Allreduce", strlen("MPI_Allreduce")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Allreduce(sendbuf, recvbuf, count, datatype, op, comm);
WRAP_MPI_CALL_POSTFIX

        if (enable_msg_tracing)
            ::tracing.handle_n2n(&c, count, datatype, comm);
        
        c.end(mpifn_attr);
        
        if (enable_msg_tracing)
            ::tracing.pop_call_id(&c);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Allreduce(sendbuf, recvbuf, count, datatype, op, comm);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}



/* ================== C Wrappers for MPI_Allgather ================== */
_EXTERN_C_ int PMPI_Allgather(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm);
_EXTERN_C_ int MPI_Allgather(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Allgather) {
#endif
        Caliper c;

        if (enable_msg_tracing)
            ::tracing.push_call_id(&c);

        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Allgather", strlen("MPI_Allgather")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Allgather(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm);
WRAP_MPI_CALL_POSTFIX

        if (enable_msg_tracing)
            ::tracing.handle_n2n(&c, sendcount, sendtype, comm);
        
        c.end(mpifn_attr);
        
        if (enable_msg_tracing)
            ::tracing.pop_call_id(&c);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Allgather(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}



/* ================== C Wrappers for MPI_Allgatherv ================== */
_EXTERN_C_ int PMPI_Allgatherv(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, MPI_Comm comm);
_EXTERN_C_ int MPI_Allgatherv(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, MPI_Comm comm) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Allgatherv) {
#endif
        Caliper c;

        if (enable_msg_tracing)
            ::tracing.push_call_id(&c);

        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Allgatherv", strlen("MPI_Allgatherv")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Allgatherv(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, comm);
WRAP_MPI_CALL_POSTFIX

        if (enable_msg_tracing)
            ::tracing.handle_n2n(&c, sendcount, sendtype, comm);
        
        c.end(mpifn_attr);
        
        if (enable_msg_tracing)
            ::tracing.pop_call_id(&c);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Allgatherv(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, comm);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}



/* ================== C Wrappers for MPI_Alltoall ================== */
_EXTERN_C_ int PMPI_Alltoall(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm);
_EXTERN_C_ int MPI_Alltoall(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Alltoall) {
#endif
        Caliper c;

        if (enable_msg_tracing)
            ::tracing.push_call_id(&c);

        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Alltoall", strlen("MPI_Alltoall")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Alltoall(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm);
WRAP_MPI_CALL_POSTFIX
        
        if (enable_msg_tracing) {
            int tmp_commsize = 0;
            PMPI_Comm_size(comm, &tmp_commsize);

            ::tracing.handle_n2n(&c, tmp_commsize * sendcount, sendtype, comm);
        }
        
        c.end(mpifn_attr);
        
        if (enable_msg_tracing)
            ::tracing.pop_call_id(&c);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Alltoall(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}



/* ================== C Wrappers for MPI_Alltoallv ================== */
_EXTERN_C_ int PMPI_Alltoallv(const void *sendbuf, const int sendcounts[], const int sdispls[], MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int rdispls[], MPI_Datatype recvtype, MPI_Comm comm);
_EXTERN_C_ int MPI_Alltoallv(const void *sendbuf, const int sendcounts[], const int sdispls[], MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int rdispls[], MPI_Datatype recvtype, MPI_Comm comm) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Alltoallv) {
#endif
        Caliper c;

        if (enable_msg_tracing)
            ::tracing.push_call_id(&c);

        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Alltoallv", strlen("MPI_Alltoallv")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Alltoallv(sendbuf, sendcounts, sdispls, sendtype, recvbuf, recvcounts, rdispls, recvtype, comm);
WRAP_MPI_CALL_POSTFIX
        
        if (enable_msg_tracing) {
            int tmp_commsize = 0;
            PMPI_Comm_size(comm, &tmp_commsize);
            int total_count  = std::accumulate(sendcounts, sendcounts+tmp_commsize, 0);

            ::tracing.handle_n2n(&c, total_count, sendtype, comm);
        }
        
        c.end(mpifn_attr);
        
        if (enable_msg_tracing)
            ::tracing.pop_call_id(&c);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Alltoallv(sendbuf, sendcounts, sdispls, sendtype, recvbuf, recvcounts, rdispls, recvtype, comm);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}



//
// --- Init/finalize
//

/* ================== C Wrappers for MPI_Init ================== */
_EXTERN_C_ int PMPI_Init(int *argc, char ***argv);
_EXTERN_C_ int MPI_Init(int *argc, char ***argv) { 
    int _wrap_py_return_val = 0;
{
    WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Init(argc, argv);
WRAP_MPI_CALL_POSTFIX

    bool run_init_evts = Caliper::is_initialized();
    
    Caliper c;

    //   Run mpi init events here if Caliper was initialized before MPI_Init
    // Otherwise they will run via the Caliper initialization above.
    if (run_init_evts)
        MpiEvents::events.mpi_init_evt(&c);
    
    // cheat a bit: put begin/ends around a barrier here
    
    if (enable_msg_tracing)
        ::tracing.push_call_id(&c);
        
    c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Init", strlen("MPI_Init")));

    PMPI_Barrier(MPI_COMM_WORLD);

    if (enable_msg_tracing)
        ::tracing.handle_init(&c);

    c.end(mpifn_attr);
        
    if (enable_msg_tracing)
        ::tracing.pop_call_id(&c);
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Init_thread ================== */
_EXTERN_C_ int PMPI_Init_thread(int *argc, char ***argv, int required, int *provided);
_EXTERN_C_ int MPI_Init_thread(int *argc, char ***argv, int required, int *provided) { 
    int _wrap_py_return_val = 0;
{
    WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Init_thread(argc, argv, required, provided);
WRAP_MPI_CALL_POSTFIX

    bool run_init_evts = Caliper::is_initialized();
    
    Caliper c;

    //   Run mpi init events here if Caliper was initialized before MPI_Init
    // Otherwise they will run via the Caliper initialization above.
    if (run_init_evts)
        MpiEvents::events.mpi_init_evt(&c);
    
    // cheat a bit: put begin/ends around a barrier here
    
    if (enable_msg_tracing)
        ::tracing.push_call_id(&c);
        
    c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Init_thread", strlen("MPI_Init_thread")));

    PMPI_Barrier(MPI_COMM_WORLD);

    if (enable_msg_tracing)
        ::tracing.handle_init(&c);

    c.end(mpifn_attr);
        
    if (enable_msg_tracing)
        ::tracing.pop_call_id(&c);
}    return _wrap_py_return_val;
}



/* ================== C Wrappers for MPI_Finalize ================== */
_EXTERN_C_ int PMPI_Finalize();
_EXTERN_C_ int MPI_Finalize() { 
    int _wrap_py_return_val = 0;
{
    Caliper c;

    // cheat a bit: put begin/ends around a barrier here
    
    if (enable_msg_tracing)
        ::tracing.push_call_id(&c);
        
    c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Finalize", strlen("MPI_Finalize")));

    PMPI_Barrier(MPI_COMM_WORLD);

    if (enable_msg_tracing)
        ::tracing.handle_finalize(&c);

    c.end(mpifn_attr);
        
    if (enable_msg_tracing)
        ::tracing.pop_call_id(&c);

    MpiEvents::events.mpi_finalize_evt(&c);

    WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Finalize();
WRAP_MPI_CALL_POSTFIX
}    return _wrap_py_return_val;
}




//
// --- Generic wrapper for all other MPI functions
//

/* ================== C Wrappers for MPI_Abort ================== */
_EXTERN_C_ int PMPI_Abort(MPI_Comm comm, int errorcode);
_EXTERN_C_ int MPI_Abort(MPI_Comm comm, int errorcode) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Abort) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Abort", strlen("MPI_Abort")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Abort(comm, errorcode);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Abort(comm, errorcode);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Accumulate ================== */
_EXTERN_C_ int PMPI_Accumulate(const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Op op, MPI_Win win);
_EXTERN_C_ int MPI_Accumulate(const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Op op, MPI_Win win) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Accumulate) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Accumulate", strlen("MPI_Accumulate")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Accumulate(origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_count, target_datatype, op, win);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Accumulate(origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_count, target_datatype, op, win);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Add_error_class ================== */
_EXTERN_C_ int PMPI_Add_error_class(int *errorclass);
_EXTERN_C_ int MPI_Add_error_class(int *errorclass) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Add_error_class) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Add_error_class", strlen("MPI_Add_error_class")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Add_error_class(errorclass);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Add_error_class(errorclass);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Add_error_code ================== */
_EXTERN_C_ int PMPI_Add_error_code(int errorclass, int *errorcode);
_EXTERN_C_ int MPI_Add_error_code(int errorclass, int *errorcode) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Add_error_code) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Add_error_code", strlen("MPI_Add_error_code")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Add_error_code(errorclass, errorcode);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Add_error_code(errorclass, errorcode);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Add_error_string ================== */
_EXTERN_C_ int PMPI_Add_error_string(int errorcode, const char *string);
_EXTERN_C_ int MPI_Add_error_string(int errorcode, const char *string) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Add_error_string) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Add_error_string", strlen("MPI_Add_error_string")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Add_error_string(errorcode, string);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Add_error_string(errorcode, string);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Address ================== */
_EXTERN_C_ int PMPI_Address(void *location, MPI_Aint *address);
_EXTERN_C_ int MPI_Address(void *location, MPI_Aint *address) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Address) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Address", strlen("MPI_Address")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Address(location, address);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Address(location, address);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Alloc_mem ================== */
_EXTERN_C_ int PMPI_Alloc_mem(MPI_Aint size, MPI_Info info, void *baseptr);
_EXTERN_C_ int MPI_Alloc_mem(MPI_Aint size, MPI_Info info, void *baseptr) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Alloc_mem) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Alloc_mem", strlen("MPI_Alloc_mem")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Alloc_mem(size, info, baseptr);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Alloc_mem(size, info, baseptr);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Alltoallw ================== */
_EXTERN_C_ int PMPI_Alltoallw(const void *sendbuf, const int sendcounts[], const int sdispls[], const MPI_Datatype sendtypes[], void *recvbuf, const int recvcounts[], const int rdispls[], const MPI_Datatype recvtypes[], MPI_Comm comm);
_EXTERN_C_ int MPI_Alltoallw(const void *sendbuf, const int sendcounts[], const int sdispls[], const MPI_Datatype sendtypes[], void *recvbuf, const int recvcounts[], const int rdispls[], const MPI_Datatype recvtypes[], MPI_Comm comm) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Alltoallw) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Alltoallw", strlen("MPI_Alltoallw")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Alltoallw(sendbuf, sendcounts, sdispls, sendtypes, recvbuf, recvcounts, rdispls, recvtypes, comm);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Alltoallw(sendbuf, sendcounts, sdispls, sendtypes, recvbuf, recvcounts, rdispls, recvtypes, comm);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Attr_delete ================== */
_EXTERN_C_ int PMPI_Attr_delete(MPI_Comm comm, int keyval);
_EXTERN_C_ int MPI_Attr_delete(MPI_Comm comm, int keyval) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Attr_delete) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Attr_delete", strlen("MPI_Attr_delete")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Attr_delete(comm, keyval);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Attr_delete(comm, keyval);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Attr_get ================== */
_EXTERN_C_ int PMPI_Attr_get(MPI_Comm comm, int keyval, void *attribute_val, int *flag);
_EXTERN_C_ int MPI_Attr_get(MPI_Comm comm, int keyval, void *attribute_val, int *flag) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Attr_get) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Attr_get", strlen("MPI_Attr_get")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Attr_get(comm, keyval, attribute_val, flag);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Attr_get(comm, keyval, attribute_val, flag);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Attr_put ================== */
_EXTERN_C_ int PMPI_Attr_put(MPI_Comm comm, int keyval, void *attribute_val);
_EXTERN_C_ int MPI_Attr_put(MPI_Comm comm, int keyval, void *attribute_val) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Attr_put) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Attr_put", strlen("MPI_Attr_put")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Attr_put(comm, keyval, attribute_val);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Attr_put(comm, keyval, attribute_val);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Buffer_attach ================== */
_EXTERN_C_ int PMPI_Buffer_attach(void *buffer, int size);
_EXTERN_C_ int MPI_Buffer_attach(void *buffer, int size) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Buffer_attach) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Buffer_attach", strlen("MPI_Buffer_attach")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Buffer_attach(buffer, size);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Buffer_attach(buffer, size);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Buffer_detach ================== */
_EXTERN_C_ int PMPI_Buffer_detach(void *buffer, int *size);
_EXTERN_C_ int MPI_Buffer_detach(void *buffer, int *size) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Buffer_detach) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Buffer_detach", strlen("MPI_Buffer_detach")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Buffer_detach(buffer, size);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Buffer_detach(buffer, size);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Cancel ================== */
_EXTERN_C_ int PMPI_Cancel(MPI_Request *request);
_EXTERN_C_ int MPI_Cancel(MPI_Request *request) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Cancel) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Cancel", strlen("MPI_Cancel")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Cancel(request);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Cancel(request);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Cart_coords ================== */
_EXTERN_C_ int PMPI_Cart_coords(MPI_Comm comm, int rank, int maxdims, int coords[]);
_EXTERN_C_ int MPI_Cart_coords(MPI_Comm comm, int rank, int maxdims, int coords[]) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Cart_coords) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Cart_coords", strlen("MPI_Cart_coords")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Cart_coords(comm, rank, maxdims, coords);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Cart_coords(comm, rank, maxdims, coords);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Cart_create ================== */
_EXTERN_C_ int PMPI_Cart_create(MPI_Comm old_comm, int ndims, const int dims[], const int periods[], int reorder, MPI_Comm *comm_cart);
_EXTERN_C_ int MPI_Cart_create(MPI_Comm old_comm, int ndims, const int dims[], const int periods[], int reorder, MPI_Comm *comm_cart) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Cart_create) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Cart_create", strlen("MPI_Cart_create")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Cart_create(old_comm, ndims, dims, periods, reorder, comm_cart);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Cart_create(old_comm, ndims, dims, periods, reorder, comm_cart);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Cart_get ================== */
_EXTERN_C_ int PMPI_Cart_get(MPI_Comm comm, int maxdims, int dims[], int periods[], int coords[]);
_EXTERN_C_ int MPI_Cart_get(MPI_Comm comm, int maxdims, int dims[], int periods[], int coords[]) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Cart_get) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Cart_get", strlen("MPI_Cart_get")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Cart_get(comm, maxdims, dims, periods, coords);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Cart_get(comm, maxdims, dims, periods, coords);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Cart_map ================== */
_EXTERN_C_ int PMPI_Cart_map(MPI_Comm comm, int ndims, const int dims[], const int periods[], int *newrank);
_EXTERN_C_ int MPI_Cart_map(MPI_Comm comm, int ndims, const int dims[], const int periods[], int *newrank) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Cart_map) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Cart_map", strlen("MPI_Cart_map")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Cart_map(comm, ndims, dims, periods, newrank);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Cart_map(comm, ndims, dims, periods, newrank);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Cart_rank ================== */
_EXTERN_C_ int PMPI_Cart_rank(MPI_Comm comm, const int coords[], int *rank);
_EXTERN_C_ int MPI_Cart_rank(MPI_Comm comm, const int coords[], int *rank) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Cart_rank) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Cart_rank", strlen("MPI_Cart_rank")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Cart_rank(comm, coords, rank);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Cart_rank(comm, coords, rank);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Cart_shift ================== */
_EXTERN_C_ int PMPI_Cart_shift(MPI_Comm comm, int direction, int disp, int *rank_source, int *rank_dest);
_EXTERN_C_ int MPI_Cart_shift(MPI_Comm comm, int direction, int disp, int *rank_source, int *rank_dest) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Cart_shift) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Cart_shift", strlen("MPI_Cart_shift")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Cart_shift(comm, direction, disp, rank_source, rank_dest);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Cart_shift(comm, direction, disp, rank_source, rank_dest);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Cart_sub ================== */
_EXTERN_C_ int PMPI_Cart_sub(MPI_Comm comm, const int remain_dims[], MPI_Comm *new_comm);
_EXTERN_C_ int MPI_Cart_sub(MPI_Comm comm, const int remain_dims[], MPI_Comm *new_comm) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Cart_sub) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Cart_sub", strlen("MPI_Cart_sub")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Cart_sub(comm, remain_dims, new_comm);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Cart_sub(comm, remain_dims, new_comm);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Cartdim_get ================== */
_EXTERN_C_ int PMPI_Cartdim_get(MPI_Comm comm, int *ndims);
_EXTERN_C_ int MPI_Cartdim_get(MPI_Comm comm, int *ndims) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Cartdim_get) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Cartdim_get", strlen("MPI_Cartdim_get")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Cartdim_get(comm, ndims);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Cartdim_get(comm, ndims);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Close_port ================== */
_EXTERN_C_ int PMPI_Close_port(const char *port_name);
_EXTERN_C_ int MPI_Close_port(const char *port_name) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Close_port) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Close_port", strlen("MPI_Close_port")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Close_port(port_name);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Close_port(port_name);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Comm_accept ================== */
_EXTERN_C_ int PMPI_Comm_accept(const char *port_name, MPI_Info info, int root, MPI_Comm comm, MPI_Comm *newcomm);
_EXTERN_C_ int MPI_Comm_accept(const char *port_name, MPI_Info info, int root, MPI_Comm comm, MPI_Comm *newcomm) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Comm_accept) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Comm_accept", strlen("MPI_Comm_accept")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Comm_accept(port_name, info, root, comm, newcomm);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Comm_accept(port_name, info, root, comm, newcomm);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Comm_call_errhandler ================== */
_EXTERN_C_ int PMPI_Comm_call_errhandler(MPI_Comm comm, int errorcode);
_EXTERN_C_ int MPI_Comm_call_errhandler(MPI_Comm comm, int errorcode) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Comm_call_errhandler) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Comm_call_errhandler", strlen("MPI_Comm_call_errhandler")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Comm_call_errhandler(comm, errorcode);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Comm_call_errhandler(comm, errorcode);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Comm_compare ================== */
_EXTERN_C_ int PMPI_Comm_compare(MPI_Comm comm1, MPI_Comm comm2, int *result);
_EXTERN_C_ int MPI_Comm_compare(MPI_Comm comm1, MPI_Comm comm2, int *result) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Comm_compare) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Comm_compare", strlen("MPI_Comm_compare")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Comm_compare(comm1, comm2, result);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Comm_compare(comm1, comm2, result);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Comm_connect ================== */
_EXTERN_C_ int PMPI_Comm_connect(const char *port_name, MPI_Info info, int root, MPI_Comm comm, MPI_Comm *newcomm);
_EXTERN_C_ int MPI_Comm_connect(const char *port_name, MPI_Info info, int root, MPI_Comm comm, MPI_Comm *newcomm) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Comm_connect) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Comm_connect", strlen("MPI_Comm_connect")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Comm_connect(port_name, info, root, comm, newcomm);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Comm_connect(port_name, info, root, comm, newcomm);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Comm_create ================== */
_EXTERN_C_ int PMPI_Comm_create(MPI_Comm comm, MPI_Group group, MPI_Comm *newcomm);
_EXTERN_C_ int MPI_Comm_create(MPI_Comm comm, MPI_Group group, MPI_Comm *newcomm) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Comm_create) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Comm_create", strlen("MPI_Comm_create")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Comm_create(comm, group, newcomm);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Comm_create(comm, group, newcomm);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Comm_create_errhandler ================== */
_EXTERN_C_ int PMPI_Comm_create_errhandler(MPI_Comm_errhandler_function *function, MPI_Errhandler *errhandler);
_EXTERN_C_ int MPI_Comm_create_errhandler(MPI_Comm_errhandler_function *function, MPI_Errhandler *errhandler) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Comm_create_errhandler) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Comm_create_errhandler", strlen("MPI_Comm_create_errhandler")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Comm_create_errhandler(function, errhandler);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Comm_create_errhandler(function, errhandler);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Comm_create_group ================== */
_EXTERN_C_ int PMPI_Comm_create_group(MPI_Comm comm, MPI_Group group, int tag, MPI_Comm *newcomm);
_EXTERN_C_ int MPI_Comm_create_group(MPI_Comm comm, MPI_Group group, int tag, MPI_Comm *newcomm) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Comm_create_group) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Comm_create_group", strlen("MPI_Comm_create_group")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Comm_create_group(comm, group, tag, newcomm);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Comm_create_group(comm, group, tag, newcomm);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Comm_create_keyval ================== */
_EXTERN_C_ int PMPI_Comm_create_keyval(MPI_Comm_copy_attr_function *comm_copy_attr_fn, MPI_Comm_delete_attr_function *comm_delete_attr_fn, int *comm_keyval, void *extra_state);
_EXTERN_C_ int MPI_Comm_create_keyval(MPI_Comm_copy_attr_function *comm_copy_attr_fn, MPI_Comm_delete_attr_function *comm_delete_attr_fn, int *comm_keyval, void *extra_state) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Comm_create_keyval) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Comm_create_keyval", strlen("MPI_Comm_create_keyval")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Comm_create_keyval(comm_copy_attr_fn, comm_delete_attr_fn, comm_keyval, extra_state);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Comm_create_keyval(comm_copy_attr_fn, comm_delete_attr_fn, comm_keyval, extra_state);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Comm_delete_attr ================== */
_EXTERN_C_ int PMPI_Comm_delete_attr(MPI_Comm comm, int comm_keyval);
_EXTERN_C_ int MPI_Comm_delete_attr(MPI_Comm comm, int comm_keyval) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Comm_delete_attr) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Comm_delete_attr", strlen("MPI_Comm_delete_attr")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Comm_delete_attr(comm, comm_keyval);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Comm_delete_attr(comm, comm_keyval);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Comm_disconnect ================== */
_EXTERN_C_ int PMPI_Comm_disconnect(MPI_Comm *comm);
_EXTERN_C_ int MPI_Comm_disconnect(MPI_Comm *comm) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Comm_disconnect) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Comm_disconnect", strlen("MPI_Comm_disconnect")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Comm_disconnect(comm);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Comm_disconnect(comm);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Comm_dup ================== */
_EXTERN_C_ int PMPI_Comm_dup(MPI_Comm comm, MPI_Comm *newcomm);
_EXTERN_C_ int MPI_Comm_dup(MPI_Comm comm, MPI_Comm *newcomm) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Comm_dup) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Comm_dup", strlen("MPI_Comm_dup")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Comm_dup(comm, newcomm);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Comm_dup(comm, newcomm);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Comm_dup_with_info ================== */
_EXTERN_C_ int PMPI_Comm_dup_with_info(MPI_Comm comm, MPI_Info info, MPI_Comm *newcomm);
_EXTERN_C_ int MPI_Comm_dup_with_info(MPI_Comm comm, MPI_Info info, MPI_Comm *newcomm) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Comm_dup_with_info) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Comm_dup_with_info", strlen("MPI_Comm_dup_with_info")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Comm_dup_with_info(comm, info, newcomm);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Comm_dup_with_info(comm, info, newcomm);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Comm_free ================== */
_EXTERN_C_ int PMPI_Comm_free(MPI_Comm *comm);
_EXTERN_C_ int MPI_Comm_free(MPI_Comm *comm) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Comm_free) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Comm_free", strlen("MPI_Comm_free")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Comm_free(comm);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Comm_free(comm);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Comm_free_keyval ================== */
_EXTERN_C_ int PMPI_Comm_free_keyval(int *comm_keyval);
_EXTERN_C_ int MPI_Comm_free_keyval(int *comm_keyval) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Comm_free_keyval) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Comm_free_keyval", strlen("MPI_Comm_free_keyval")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Comm_free_keyval(comm_keyval);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Comm_free_keyval(comm_keyval);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Comm_get_attr ================== */
_EXTERN_C_ int PMPI_Comm_get_attr(MPI_Comm comm, int comm_keyval, void *attribute_val, int *flag);
_EXTERN_C_ int MPI_Comm_get_attr(MPI_Comm comm, int comm_keyval, void *attribute_val, int *flag) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Comm_get_attr) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Comm_get_attr", strlen("MPI_Comm_get_attr")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Comm_get_attr(comm, comm_keyval, attribute_val, flag);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Comm_get_attr(comm, comm_keyval, attribute_val, flag);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Comm_get_errhandler ================== */
_EXTERN_C_ int PMPI_Comm_get_errhandler(MPI_Comm comm, MPI_Errhandler *erhandler);
_EXTERN_C_ int MPI_Comm_get_errhandler(MPI_Comm comm, MPI_Errhandler *erhandler) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Comm_get_errhandler) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Comm_get_errhandler", strlen("MPI_Comm_get_errhandler")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Comm_get_errhandler(comm, erhandler);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Comm_get_errhandler(comm, erhandler);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Comm_get_info ================== */
_EXTERN_C_ int PMPI_Comm_get_info(MPI_Comm comm, MPI_Info *info_used);
_EXTERN_C_ int MPI_Comm_get_info(MPI_Comm comm, MPI_Info *info_used) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Comm_get_info) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Comm_get_info", strlen("MPI_Comm_get_info")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Comm_get_info(comm, info_used);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Comm_get_info(comm, info_used);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Comm_get_name ================== */
_EXTERN_C_ int PMPI_Comm_get_name(MPI_Comm comm, char *comm_name, int *resultlen);
_EXTERN_C_ int MPI_Comm_get_name(MPI_Comm comm, char *comm_name, int *resultlen) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Comm_get_name) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Comm_get_name", strlen("MPI_Comm_get_name")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Comm_get_name(comm, comm_name, resultlen);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Comm_get_name(comm, comm_name, resultlen);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Comm_get_parent ================== */
_EXTERN_C_ int PMPI_Comm_get_parent(MPI_Comm *parent);
_EXTERN_C_ int MPI_Comm_get_parent(MPI_Comm *parent) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Comm_get_parent) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Comm_get_parent", strlen("MPI_Comm_get_parent")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Comm_get_parent(parent);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Comm_get_parent(parent);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Comm_group ================== */
_EXTERN_C_ int PMPI_Comm_group(MPI_Comm comm, MPI_Group *group);
_EXTERN_C_ int MPI_Comm_group(MPI_Comm comm, MPI_Group *group) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Comm_group) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Comm_group", strlen("MPI_Comm_group")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Comm_group(comm, group);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Comm_group(comm, group);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Comm_idup ================== */
_EXTERN_C_ int PMPI_Comm_idup(MPI_Comm comm, MPI_Comm *newcomm, MPI_Request *request);
_EXTERN_C_ int MPI_Comm_idup(MPI_Comm comm, MPI_Comm *newcomm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Comm_idup) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Comm_idup", strlen("MPI_Comm_idup")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Comm_idup(comm, newcomm, request);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Comm_idup(comm, newcomm, request);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Comm_join ================== */
_EXTERN_C_ int PMPI_Comm_join(int fd, MPI_Comm *intercomm);
_EXTERN_C_ int MPI_Comm_join(int fd, MPI_Comm *intercomm) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Comm_join) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Comm_join", strlen("MPI_Comm_join")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Comm_join(fd, intercomm);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Comm_join(fd, intercomm);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Comm_rank ================== */
_EXTERN_C_ int PMPI_Comm_rank(MPI_Comm comm, int *rank);
_EXTERN_C_ int MPI_Comm_rank(MPI_Comm comm, int *rank) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Comm_rank) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Comm_rank", strlen("MPI_Comm_rank")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Comm_rank(comm, rank);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Comm_rank(comm, rank);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Comm_remote_group ================== */
_EXTERN_C_ int PMPI_Comm_remote_group(MPI_Comm comm, MPI_Group *group);
_EXTERN_C_ int MPI_Comm_remote_group(MPI_Comm comm, MPI_Group *group) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Comm_remote_group) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Comm_remote_group", strlen("MPI_Comm_remote_group")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Comm_remote_group(comm, group);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Comm_remote_group(comm, group);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Comm_remote_size ================== */
_EXTERN_C_ int PMPI_Comm_remote_size(MPI_Comm comm, int *size);
_EXTERN_C_ int MPI_Comm_remote_size(MPI_Comm comm, int *size) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Comm_remote_size) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Comm_remote_size", strlen("MPI_Comm_remote_size")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Comm_remote_size(comm, size);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Comm_remote_size(comm, size);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Comm_set_attr ================== */
_EXTERN_C_ int PMPI_Comm_set_attr(MPI_Comm comm, int comm_keyval, void *attribute_val);
_EXTERN_C_ int MPI_Comm_set_attr(MPI_Comm comm, int comm_keyval, void *attribute_val) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Comm_set_attr) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Comm_set_attr", strlen("MPI_Comm_set_attr")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Comm_set_attr(comm, comm_keyval, attribute_val);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Comm_set_attr(comm, comm_keyval, attribute_val);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Comm_set_errhandler ================== */
_EXTERN_C_ int PMPI_Comm_set_errhandler(MPI_Comm comm, MPI_Errhandler errhandler);
_EXTERN_C_ int MPI_Comm_set_errhandler(MPI_Comm comm, MPI_Errhandler errhandler) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Comm_set_errhandler) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Comm_set_errhandler", strlen("MPI_Comm_set_errhandler")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Comm_set_errhandler(comm, errhandler);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Comm_set_errhandler(comm, errhandler);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Comm_set_info ================== */
_EXTERN_C_ int PMPI_Comm_set_info(MPI_Comm comm, MPI_Info info);
_EXTERN_C_ int MPI_Comm_set_info(MPI_Comm comm, MPI_Info info) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Comm_set_info) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Comm_set_info", strlen("MPI_Comm_set_info")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Comm_set_info(comm, info);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Comm_set_info(comm, info);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Comm_set_name ================== */
_EXTERN_C_ int PMPI_Comm_set_name(MPI_Comm comm, const char *comm_name);
_EXTERN_C_ int MPI_Comm_set_name(MPI_Comm comm, const char *comm_name) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Comm_set_name) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Comm_set_name", strlen("MPI_Comm_set_name")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Comm_set_name(comm, comm_name);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Comm_set_name(comm, comm_name);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Comm_size ================== */
_EXTERN_C_ int PMPI_Comm_size(MPI_Comm comm, int *size);
_EXTERN_C_ int MPI_Comm_size(MPI_Comm comm, int *size) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Comm_size) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Comm_size", strlen("MPI_Comm_size")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Comm_size(comm, size);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Comm_size(comm, size);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Comm_split ================== */
_EXTERN_C_ int PMPI_Comm_split(MPI_Comm comm, int color, int key, MPI_Comm *newcomm);
_EXTERN_C_ int MPI_Comm_split(MPI_Comm comm, int color, int key, MPI_Comm *newcomm) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Comm_split) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Comm_split", strlen("MPI_Comm_split")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Comm_split(comm, color, key, newcomm);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Comm_split(comm, color, key, newcomm);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Comm_split_type ================== */
_EXTERN_C_ int PMPI_Comm_split_type(MPI_Comm comm, int split_type, int key, MPI_Info info, MPI_Comm *newcomm);
_EXTERN_C_ int MPI_Comm_split_type(MPI_Comm comm, int split_type, int key, MPI_Info info, MPI_Comm *newcomm) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Comm_split_type) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Comm_split_type", strlen("MPI_Comm_split_type")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Comm_split_type(comm, split_type, key, info, newcomm);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Comm_split_type(comm, split_type, key, info, newcomm);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Comm_test_inter ================== */
_EXTERN_C_ int PMPI_Comm_test_inter(MPI_Comm comm, int *flag);
_EXTERN_C_ int MPI_Comm_test_inter(MPI_Comm comm, int *flag) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Comm_test_inter) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Comm_test_inter", strlen("MPI_Comm_test_inter")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Comm_test_inter(comm, flag);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Comm_test_inter(comm, flag);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Compare_and_swap ================== */
_EXTERN_C_ int PMPI_Compare_and_swap(const void *origin_addr, const void *compare_addr, void *result_addr, MPI_Datatype datatype, int target_rank, MPI_Aint target_disp, MPI_Win win);
_EXTERN_C_ int MPI_Compare_and_swap(const void *origin_addr, const void *compare_addr, void *result_addr, MPI_Datatype datatype, int target_rank, MPI_Aint target_disp, MPI_Win win) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Compare_and_swap) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Compare_and_swap", strlen("MPI_Compare_and_swap")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Compare_and_swap(origin_addr, compare_addr, result_addr, datatype, target_rank, target_disp, win);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Compare_and_swap(origin_addr, compare_addr, result_addr, datatype, target_rank, target_disp, win);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Dims_create ================== */
_EXTERN_C_ int PMPI_Dims_create(int nnodes, int ndims, int dims[]);
_EXTERN_C_ int MPI_Dims_create(int nnodes, int ndims, int dims[]) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Dims_create) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Dims_create", strlen("MPI_Dims_create")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Dims_create(nnodes, ndims, dims);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Dims_create(nnodes, ndims, dims);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Dist_graph_create ================== */
_EXTERN_C_ int PMPI_Dist_graph_create(MPI_Comm comm_old, int n, const int nodes[], const int degrees[], const int targets[], const int weights[], MPI_Info info, int reorder, MPI_Comm *newcomm);
_EXTERN_C_ int MPI_Dist_graph_create(MPI_Comm comm_old, int n, const int nodes[], const int degrees[], const int targets[], const int weights[], MPI_Info info, int reorder, MPI_Comm *newcomm) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Dist_graph_create) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Dist_graph_create", strlen("MPI_Dist_graph_create")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Dist_graph_create(comm_old, n, nodes, degrees, targets, weights, info, reorder, newcomm);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Dist_graph_create(comm_old, n, nodes, degrees, targets, weights, info, reorder, newcomm);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Dist_graph_create_adjacent ================== */
_EXTERN_C_ int PMPI_Dist_graph_create_adjacent(MPI_Comm comm_old, int indegree, const int sources[], const int sourceweights[], int outdegree, const int destinations[], const int destweights[], MPI_Info info, int reorder, MPI_Comm *comm_dist_graph);
_EXTERN_C_ int MPI_Dist_graph_create_adjacent(MPI_Comm comm_old, int indegree, const int sources[], const int sourceweights[], int outdegree, const int destinations[], const int destweights[], MPI_Info info, int reorder, MPI_Comm *comm_dist_graph) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Dist_graph_create_adjacent) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Dist_graph_create_adjacent", strlen("MPI_Dist_graph_create_adjacent")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Dist_graph_create_adjacent(comm_old, indegree, sources, sourceweights, outdegree, destinations, destweights, info, reorder, comm_dist_graph);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Dist_graph_create_adjacent(comm_old, indegree, sources, sourceweights, outdegree, destinations, destweights, info, reorder, comm_dist_graph);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Dist_graph_neighbors ================== */
_EXTERN_C_ int PMPI_Dist_graph_neighbors(MPI_Comm comm, int maxindegree, int sources[], int sourceweights[], int maxoutdegree, int destinations[], int destweights[]);
_EXTERN_C_ int MPI_Dist_graph_neighbors(MPI_Comm comm, int maxindegree, int sources[], int sourceweights[], int maxoutdegree, int destinations[], int destweights[]) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Dist_graph_neighbors) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Dist_graph_neighbors", strlen("MPI_Dist_graph_neighbors")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Dist_graph_neighbors(comm, maxindegree, sources, sourceweights, maxoutdegree, destinations, destweights);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Dist_graph_neighbors(comm, maxindegree, sources, sourceweights, maxoutdegree, destinations, destweights);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Dist_graph_neighbors_count ================== */
_EXTERN_C_ int PMPI_Dist_graph_neighbors_count(MPI_Comm comm, int *inneighbors, int *outneighbors, int *weighted);
_EXTERN_C_ int MPI_Dist_graph_neighbors_count(MPI_Comm comm, int *inneighbors, int *outneighbors, int *weighted) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Dist_graph_neighbors_count) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Dist_graph_neighbors_count", strlen("MPI_Dist_graph_neighbors_count")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Dist_graph_neighbors_count(comm, inneighbors, outneighbors, weighted);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Dist_graph_neighbors_count(comm, inneighbors, outneighbors, weighted);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Errhandler_create ================== */
_EXTERN_C_ int PMPI_Errhandler_create(MPI_Handler_function *function, MPI_Errhandler *errhandler);
_EXTERN_C_ int MPI_Errhandler_create(MPI_Handler_function *function, MPI_Errhandler *errhandler) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Errhandler_create) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Errhandler_create", strlen("MPI_Errhandler_create")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Errhandler_create(function, errhandler);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Errhandler_create(function, errhandler);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Errhandler_free ================== */
_EXTERN_C_ int PMPI_Errhandler_free(MPI_Errhandler *errhandler);
_EXTERN_C_ int MPI_Errhandler_free(MPI_Errhandler *errhandler) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Errhandler_free) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Errhandler_free", strlen("MPI_Errhandler_free")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Errhandler_free(errhandler);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Errhandler_free(errhandler);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Errhandler_get ================== */
_EXTERN_C_ int PMPI_Errhandler_get(MPI_Comm comm, MPI_Errhandler *errhandler);
_EXTERN_C_ int MPI_Errhandler_get(MPI_Comm comm, MPI_Errhandler *errhandler) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Errhandler_get) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Errhandler_get", strlen("MPI_Errhandler_get")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Errhandler_get(comm, errhandler);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Errhandler_get(comm, errhandler);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Errhandler_set ================== */
_EXTERN_C_ int PMPI_Errhandler_set(MPI_Comm comm, MPI_Errhandler errhandler);
_EXTERN_C_ int MPI_Errhandler_set(MPI_Comm comm, MPI_Errhandler errhandler) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Errhandler_set) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Errhandler_set", strlen("MPI_Errhandler_set")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Errhandler_set(comm, errhandler);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Errhandler_set(comm, errhandler);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Error_class ================== */
_EXTERN_C_ int PMPI_Error_class(int errorcode, int *errorclass);
_EXTERN_C_ int MPI_Error_class(int errorcode, int *errorclass) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Error_class) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Error_class", strlen("MPI_Error_class")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Error_class(errorcode, errorclass);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Error_class(errorcode, errorclass);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Error_string ================== */
_EXTERN_C_ int PMPI_Error_string(int errorcode, char *string, int *resultlen);
_EXTERN_C_ int MPI_Error_string(int errorcode, char *string, int *resultlen) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Error_string) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Error_string", strlen("MPI_Error_string")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Error_string(errorcode, string, resultlen);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Error_string(errorcode, string, resultlen);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Fetch_and_op ================== */
_EXTERN_C_ int PMPI_Fetch_and_op(const void *origin_addr, void *result_addr, MPI_Datatype datatype, int target_rank, MPI_Aint target_disp, MPI_Op op, MPI_Win win);
_EXTERN_C_ int MPI_Fetch_and_op(const void *origin_addr, void *result_addr, MPI_Datatype datatype, int target_rank, MPI_Aint target_disp, MPI_Op op, MPI_Win win) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Fetch_and_op) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Fetch_and_op", strlen("MPI_Fetch_and_op")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Fetch_and_op(origin_addr, result_addr, datatype, target_rank, target_disp, op, win);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Fetch_and_op(origin_addr, result_addr, datatype, target_rank, target_disp, op, win);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_call_errhandler ================== */
_EXTERN_C_ int PMPI_File_call_errhandler(MPI_File fh, int errorcode);
_EXTERN_C_ int MPI_File_call_errhandler(MPI_File fh, int errorcode) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_File_call_errhandler) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_File_call_errhandler", strlen("MPI_File_call_errhandler")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_call_errhandler(fh, errorcode);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_call_errhandler(fh, errorcode);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_close ================== */
_EXTERN_C_ int PMPI_File_close(MPI_File *fh);
_EXTERN_C_ int MPI_File_close(MPI_File *fh) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_File_close) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_File_close", strlen("MPI_File_close")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_close(fh);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_close(fh);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_create_errhandler ================== */
_EXTERN_C_ int PMPI_File_create_errhandler(MPI_File_errhandler_function *function, MPI_Errhandler *errhandler);
_EXTERN_C_ int MPI_File_create_errhandler(MPI_File_errhandler_function *function, MPI_Errhandler *errhandler) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_File_create_errhandler) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_File_create_errhandler", strlen("MPI_File_create_errhandler")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_create_errhandler(function, errhandler);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_create_errhandler(function, errhandler);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_delete ================== */
_EXTERN_C_ int PMPI_File_delete(const char *filename, MPI_Info info);
_EXTERN_C_ int MPI_File_delete(const char *filename, MPI_Info info) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_File_delete) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_File_delete", strlen("MPI_File_delete")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_delete(filename, info);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_delete(filename, info);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_get_amode ================== */
_EXTERN_C_ int PMPI_File_get_amode(MPI_File fh, int *amode);
_EXTERN_C_ int MPI_File_get_amode(MPI_File fh, int *amode) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_File_get_amode) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_File_get_amode", strlen("MPI_File_get_amode")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_get_amode(fh, amode);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_get_amode(fh, amode);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_get_atomicity ================== */
_EXTERN_C_ int PMPI_File_get_atomicity(MPI_File fh, int *flag);
_EXTERN_C_ int MPI_File_get_atomicity(MPI_File fh, int *flag) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_File_get_atomicity) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_File_get_atomicity", strlen("MPI_File_get_atomicity")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_get_atomicity(fh, flag);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_get_atomicity(fh, flag);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_get_byte_offset ================== */
_EXTERN_C_ int PMPI_File_get_byte_offset(MPI_File fh, MPI_Offset offset, MPI_Offset *disp);
_EXTERN_C_ int MPI_File_get_byte_offset(MPI_File fh, MPI_Offset offset, MPI_Offset *disp) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_File_get_byte_offset) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_File_get_byte_offset", strlen("MPI_File_get_byte_offset")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_get_byte_offset(fh, offset, disp);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_get_byte_offset(fh, offset, disp);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_get_errhandler ================== */
_EXTERN_C_ int PMPI_File_get_errhandler(MPI_File file, MPI_Errhandler *errhandler);
_EXTERN_C_ int MPI_File_get_errhandler(MPI_File file, MPI_Errhandler *errhandler) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_File_get_errhandler) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_File_get_errhandler", strlen("MPI_File_get_errhandler")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_get_errhandler(file, errhandler);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_get_errhandler(file, errhandler);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_get_group ================== */
_EXTERN_C_ int PMPI_File_get_group(MPI_File fh, MPI_Group *group);
_EXTERN_C_ int MPI_File_get_group(MPI_File fh, MPI_Group *group) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_File_get_group) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_File_get_group", strlen("MPI_File_get_group")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_get_group(fh, group);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_get_group(fh, group);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_get_info ================== */
_EXTERN_C_ int PMPI_File_get_info(MPI_File fh, MPI_Info *info_used);
_EXTERN_C_ int MPI_File_get_info(MPI_File fh, MPI_Info *info_used) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_File_get_info) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_File_get_info", strlen("MPI_File_get_info")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_get_info(fh, info_used);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_get_info(fh, info_used);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_get_position ================== */
_EXTERN_C_ int PMPI_File_get_position(MPI_File fh, MPI_Offset *offset);
_EXTERN_C_ int MPI_File_get_position(MPI_File fh, MPI_Offset *offset) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_File_get_position) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_File_get_position", strlen("MPI_File_get_position")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_get_position(fh, offset);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_get_position(fh, offset);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_get_position_shared ================== */
_EXTERN_C_ int PMPI_File_get_position_shared(MPI_File fh, MPI_Offset *offset);
_EXTERN_C_ int MPI_File_get_position_shared(MPI_File fh, MPI_Offset *offset) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_File_get_position_shared) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_File_get_position_shared", strlen("MPI_File_get_position_shared")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_get_position_shared(fh, offset);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_get_position_shared(fh, offset);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_get_size ================== */
_EXTERN_C_ int PMPI_File_get_size(MPI_File fh, MPI_Offset *size);
_EXTERN_C_ int MPI_File_get_size(MPI_File fh, MPI_Offset *size) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_File_get_size) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_File_get_size", strlen("MPI_File_get_size")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_get_size(fh, size);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_get_size(fh, size);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_get_type_extent ================== */
_EXTERN_C_ int PMPI_File_get_type_extent(MPI_File fh, MPI_Datatype datatype, MPI_Aint *extent);
_EXTERN_C_ int MPI_File_get_type_extent(MPI_File fh, MPI_Datatype datatype, MPI_Aint *extent) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_File_get_type_extent) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_File_get_type_extent", strlen("MPI_File_get_type_extent")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_get_type_extent(fh, datatype, extent);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_get_type_extent(fh, datatype, extent);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_get_view ================== */
_EXTERN_C_ int PMPI_File_get_view(MPI_File fh, MPI_Offset *disp, MPI_Datatype *etype, MPI_Datatype *filetype, char *datarep);
_EXTERN_C_ int MPI_File_get_view(MPI_File fh, MPI_Offset *disp, MPI_Datatype *etype, MPI_Datatype *filetype, char *datarep) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_File_get_view) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_File_get_view", strlen("MPI_File_get_view")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_get_view(fh, disp, etype, filetype, datarep);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_get_view(fh, disp, etype, filetype, datarep);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_iread ================== */
_EXTERN_C_ int PMPI_File_iread(MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Request *request);
_EXTERN_C_ int MPI_File_iread(MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Request *request) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_File_iread) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_File_iread", strlen("MPI_File_iread")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_iread(fh, buf, count, datatype, request);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_iread(fh, buf, count, datatype, request);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_iread_at ================== */
_EXTERN_C_ int PMPI_File_iread_at(MPI_File fh, MPI_Offset offset, void *buf, int count, MPI_Datatype datatype, MPI_Request *request);
_EXTERN_C_ int MPI_File_iread_at(MPI_File fh, MPI_Offset offset, void *buf, int count, MPI_Datatype datatype, MPI_Request *request) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_File_iread_at) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_File_iread_at", strlen("MPI_File_iread_at")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_iread_at(fh, offset, buf, count, datatype, request);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_iread_at(fh, offset, buf, count, datatype, request);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_iread_shared ================== */
_EXTERN_C_ int PMPI_File_iread_shared(MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Request *request);
_EXTERN_C_ int MPI_File_iread_shared(MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Request *request) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_File_iread_shared) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_File_iread_shared", strlen("MPI_File_iread_shared")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_iread_shared(fh, buf, count, datatype, request);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_iread_shared(fh, buf, count, datatype, request);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_iwrite ================== */
_EXTERN_C_ int PMPI_File_iwrite(MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Request *request);
_EXTERN_C_ int MPI_File_iwrite(MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Request *request) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_File_iwrite) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_File_iwrite", strlen("MPI_File_iwrite")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_iwrite(fh, buf, count, datatype, request);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_iwrite(fh, buf, count, datatype, request);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_iwrite_at ================== */
_EXTERN_C_ int PMPI_File_iwrite_at(MPI_File fh, MPI_Offset offset, const void *buf, int count, MPI_Datatype datatype, MPI_Request *request);
_EXTERN_C_ int MPI_File_iwrite_at(MPI_File fh, MPI_Offset offset, const void *buf, int count, MPI_Datatype datatype, MPI_Request *request) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_File_iwrite_at) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_File_iwrite_at", strlen("MPI_File_iwrite_at")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_iwrite_at(fh, offset, buf, count, datatype, request);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_iwrite_at(fh, offset, buf, count, datatype, request);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_iwrite_shared ================== */
_EXTERN_C_ int PMPI_File_iwrite_shared(MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Request *request);
_EXTERN_C_ int MPI_File_iwrite_shared(MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Request *request) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_File_iwrite_shared) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_File_iwrite_shared", strlen("MPI_File_iwrite_shared")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_iwrite_shared(fh, buf, count, datatype, request);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_iwrite_shared(fh, buf, count, datatype, request);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_open ================== */
_EXTERN_C_ int PMPI_File_open(MPI_Comm comm, const char *filename, int amode, MPI_Info info, MPI_File *fh);
_EXTERN_C_ int MPI_File_open(MPI_Comm comm, const char *filename, int amode, MPI_Info info, MPI_File *fh) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_File_open) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_File_open", strlen("MPI_File_open")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_open(comm, filename, amode, info, fh);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_open(comm, filename, amode, info, fh);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_preallocate ================== */
_EXTERN_C_ int PMPI_File_preallocate(MPI_File fh, MPI_Offset size);
_EXTERN_C_ int MPI_File_preallocate(MPI_File fh, MPI_Offset size) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_File_preallocate) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_File_preallocate", strlen("MPI_File_preallocate")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_preallocate(fh, size);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_preallocate(fh, size);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_read ================== */
_EXTERN_C_ int PMPI_File_read(MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Status *status);
_EXTERN_C_ int MPI_File_read(MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Status *status) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_File_read) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_File_read", strlen("MPI_File_read")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_read(fh, buf, count, datatype, status);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_read(fh, buf, count, datatype, status);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_read_all ================== */
_EXTERN_C_ int PMPI_File_read_all(MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Status *status);
_EXTERN_C_ int MPI_File_read_all(MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Status *status) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_File_read_all) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_File_read_all", strlen("MPI_File_read_all")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_read_all(fh, buf, count, datatype, status);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_read_all(fh, buf, count, datatype, status);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_read_all_begin ================== */
_EXTERN_C_ int PMPI_File_read_all_begin(MPI_File fh, void *buf, int count, MPI_Datatype datatype);
_EXTERN_C_ int MPI_File_read_all_begin(MPI_File fh, void *buf, int count, MPI_Datatype datatype) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_File_read_all_begin) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_File_read_all_begin", strlen("MPI_File_read_all_begin")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_read_all_begin(fh, buf, count, datatype);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_read_all_begin(fh, buf, count, datatype);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_read_all_end ================== */
_EXTERN_C_ int PMPI_File_read_all_end(MPI_File fh, void *buf, MPI_Status *status);
_EXTERN_C_ int MPI_File_read_all_end(MPI_File fh, void *buf, MPI_Status *status) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_File_read_all_end) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_File_read_all_end", strlen("MPI_File_read_all_end")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_read_all_end(fh, buf, status);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_read_all_end(fh, buf, status);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_read_at ================== */
_EXTERN_C_ int PMPI_File_read_at(MPI_File fh, MPI_Offset offset, void *buf, int count, MPI_Datatype datatype, MPI_Status *status);
_EXTERN_C_ int MPI_File_read_at(MPI_File fh, MPI_Offset offset, void *buf, int count, MPI_Datatype datatype, MPI_Status *status) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_File_read_at) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_File_read_at", strlen("MPI_File_read_at")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_read_at(fh, offset, buf, count, datatype, status);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_read_at(fh, offset, buf, count, datatype, status);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_read_at_all ================== */
_EXTERN_C_ int PMPI_File_read_at_all(MPI_File fh, MPI_Offset offset, void *buf, int count, MPI_Datatype datatype, MPI_Status *status);
_EXTERN_C_ int MPI_File_read_at_all(MPI_File fh, MPI_Offset offset, void *buf, int count, MPI_Datatype datatype, MPI_Status *status) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_File_read_at_all) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_File_read_at_all", strlen("MPI_File_read_at_all")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_read_at_all(fh, offset, buf, count, datatype, status);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_read_at_all(fh, offset, buf, count, datatype, status);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_read_at_all_begin ================== */
_EXTERN_C_ int PMPI_File_read_at_all_begin(MPI_File fh, MPI_Offset offset, void *buf, int count, MPI_Datatype datatype);
_EXTERN_C_ int MPI_File_read_at_all_begin(MPI_File fh, MPI_Offset offset, void *buf, int count, MPI_Datatype datatype) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_File_read_at_all_begin) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_File_read_at_all_begin", strlen("MPI_File_read_at_all_begin")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_read_at_all_begin(fh, offset, buf, count, datatype);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_read_at_all_begin(fh, offset, buf, count, datatype);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_read_at_all_end ================== */
_EXTERN_C_ int PMPI_File_read_at_all_end(MPI_File fh, void *buf, MPI_Status *status);
_EXTERN_C_ int MPI_File_read_at_all_end(MPI_File fh, void *buf, MPI_Status *status) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_File_read_at_all_end) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_File_read_at_all_end", strlen("MPI_File_read_at_all_end")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_read_at_all_end(fh, buf, status);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_read_at_all_end(fh, buf, status);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_read_ordered ================== */
_EXTERN_C_ int PMPI_File_read_ordered(MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Status *status);
_EXTERN_C_ int MPI_File_read_ordered(MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Status *status) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_File_read_ordered) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_File_read_ordered", strlen("MPI_File_read_ordered")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_read_ordered(fh, buf, count, datatype, status);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_read_ordered(fh, buf, count, datatype, status);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_read_ordered_begin ================== */
_EXTERN_C_ int PMPI_File_read_ordered_begin(MPI_File fh, void *buf, int count, MPI_Datatype datatype);
_EXTERN_C_ int MPI_File_read_ordered_begin(MPI_File fh, void *buf, int count, MPI_Datatype datatype) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_File_read_ordered_begin) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_File_read_ordered_begin", strlen("MPI_File_read_ordered_begin")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_read_ordered_begin(fh, buf, count, datatype);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_read_ordered_begin(fh, buf, count, datatype);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_read_ordered_end ================== */
_EXTERN_C_ int PMPI_File_read_ordered_end(MPI_File fh, void *buf, MPI_Status *status);
_EXTERN_C_ int MPI_File_read_ordered_end(MPI_File fh, void *buf, MPI_Status *status) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_File_read_ordered_end) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_File_read_ordered_end", strlen("MPI_File_read_ordered_end")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_read_ordered_end(fh, buf, status);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_read_ordered_end(fh, buf, status);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_read_shared ================== */
_EXTERN_C_ int PMPI_File_read_shared(MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Status *status);
_EXTERN_C_ int MPI_File_read_shared(MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Status *status) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_File_read_shared) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_File_read_shared", strlen("MPI_File_read_shared")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_read_shared(fh, buf, count, datatype, status);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_read_shared(fh, buf, count, datatype, status);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_seek ================== */
_EXTERN_C_ int PMPI_File_seek(MPI_File fh, MPI_Offset offset, int whence);
_EXTERN_C_ int MPI_File_seek(MPI_File fh, MPI_Offset offset, int whence) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_File_seek) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_File_seek", strlen("MPI_File_seek")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_seek(fh, offset, whence);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_seek(fh, offset, whence);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_seek_shared ================== */
_EXTERN_C_ int PMPI_File_seek_shared(MPI_File fh, MPI_Offset offset, int whence);
_EXTERN_C_ int MPI_File_seek_shared(MPI_File fh, MPI_Offset offset, int whence) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_File_seek_shared) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_File_seek_shared", strlen("MPI_File_seek_shared")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_seek_shared(fh, offset, whence);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_seek_shared(fh, offset, whence);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_set_atomicity ================== */
_EXTERN_C_ int PMPI_File_set_atomicity(MPI_File fh, int flag);
_EXTERN_C_ int MPI_File_set_atomicity(MPI_File fh, int flag) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_File_set_atomicity) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_File_set_atomicity", strlen("MPI_File_set_atomicity")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_set_atomicity(fh, flag);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_set_atomicity(fh, flag);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_set_errhandler ================== */
_EXTERN_C_ int PMPI_File_set_errhandler(MPI_File file, MPI_Errhandler errhandler);
_EXTERN_C_ int MPI_File_set_errhandler(MPI_File file, MPI_Errhandler errhandler) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_File_set_errhandler) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_File_set_errhandler", strlen("MPI_File_set_errhandler")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_set_errhandler(file, errhandler);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_set_errhandler(file, errhandler);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_set_info ================== */
_EXTERN_C_ int PMPI_File_set_info(MPI_File fh, MPI_Info info);
_EXTERN_C_ int MPI_File_set_info(MPI_File fh, MPI_Info info) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_File_set_info) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_File_set_info", strlen("MPI_File_set_info")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_set_info(fh, info);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_set_info(fh, info);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_set_size ================== */
_EXTERN_C_ int PMPI_File_set_size(MPI_File fh, MPI_Offset size);
_EXTERN_C_ int MPI_File_set_size(MPI_File fh, MPI_Offset size) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_File_set_size) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_File_set_size", strlen("MPI_File_set_size")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_set_size(fh, size);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_set_size(fh, size);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_set_view ================== */
_EXTERN_C_ int PMPI_File_set_view(MPI_File fh, MPI_Offset disp, MPI_Datatype etype, MPI_Datatype filetype, const char *datarep, MPI_Info info);
_EXTERN_C_ int MPI_File_set_view(MPI_File fh, MPI_Offset disp, MPI_Datatype etype, MPI_Datatype filetype, const char *datarep, MPI_Info info) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_File_set_view) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_File_set_view", strlen("MPI_File_set_view")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_set_view(fh, disp, etype, filetype, datarep, info);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_set_view(fh, disp, etype, filetype, datarep, info);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_sync ================== */
_EXTERN_C_ int PMPI_File_sync(MPI_File fh);
_EXTERN_C_ int MPI_File_sync(MPI_File fh) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_File_sync) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_File_sync", strlen("MPI_File_sync")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_sync(fh);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_sync(fh);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_write ================== */
_EXTERN_C_ int PMPI_File_write(MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status);
_EXTERN_C_ int MPI_File_write(MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_File_write) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_File_write", strlen("MPI_File_write")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_write(fh, buf, count, datatype, status);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_write(fh, buf, count, datatype, status);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_write_all ================== */
_EXTERN_C_ int PMPI_File_write_all(MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status);
_EXTERN_C_ int MPI_File_write_all(MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_File_write_all) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_File_write_all", strlen("MPI_File_write_all")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_write_all(fh, buf, count, datatype, status);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_write_all(fh, buf, count, datatype, status);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_write_all_begin ================== */
_EXTERN_C_ int PMPI_File_write_all_begin(MPI_File fh, const void *buf, int count, MPI_Datatype datatype);
_EXTERN_C_ int MPI_File_write_all_begin(MPI_File fh, const void *buf, int count, MPI_Datatype datatype) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_File_write_all_begin) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_File_write_all_begin", strlen("MPI_File_write_all_begin")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_write_all_begin(fh, buf, count, datatype);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_write_all_begin(fh, buf, count, datatype);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_write_all_end ================== */
_EXTERN_C_ int PMPI_File_write_all_end(MPI_File fh, const void *buf, MPI_Status *status);
_EXTERN_C_ int MPI_File_write_all_end(MPI_File fh, const void *buf, MPI_Status *status) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_File_write_all_end) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_File_write_all_end", strlen("MPI_File_write_all_end")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_write_all_end(fh, buf, status);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_write_all_end(fh, buf, status);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_write_at ================== */
_EXTERN_C_ int PMPI_File_write_at(MPI_File fh, MPI_Offset offset, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status);
_EXTERN_C_ int MPI_File_write_at(MPI_File fh, MPI_Offset offset, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_File_write_at) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_File_write_at", strlen("MPI_File_write_at")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_write_at(fh, offset, buf, count, datatype, status);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_write_at(fh, offset, buf, count, datatype, status);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_write_at_all ================== */
_EXTERN_C_ int PMPI_File_write_at_all(MPI_File fh, MPI_Offset offset, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status);
_EXTERN_C_ int MPI_File_write_at_all(MPI_File fh, MPI_Offset offset, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_File_write_at_all) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_File_write_at_all", strlen("MPI_File_write_at_all")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_write_at_all(fh, offset, buf, count, datatype, status);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_write_at_all(fh, offset, buf, count, datatype, status);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_write_at_all_begin ================== */
_EXTERN_C_ int PMPI_File_write_at_all_begin(MPI_File fh, MPI_Offset offset, const void *buf, int count, MPI_Datatype datatype);
_EXTERN_C_ int MPI_File_write_at_all_begin(MPI_File fh, MPI_Offset offset, const void *buf, int count, MPI_Datatype datatype) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_File_write_at_all_begin) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_File_write_at_all_begin", strlen("MPI_File_write_at_all_begin")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_write_at_all_begin(fh, offset, buf, count, datatype);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_write_at_all_begin(fh, offset, buf, count, datatype);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_write_at_all_end ================== */
_EXTERN_C_ int PMPI_File_write_at_all_end(MPI_File fh, const void *buf, MPI_Status *status);
_EXTERN_C_ int MPI_File_write_at_all_end(MPI_File fh, const void *buf, MPI_Status *status) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_File_write_at_all_end) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_File_write_at_all_end", strlen("MPI_File_write_at_all_end")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_write_at_all_end(fh, buf, status);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_write_at_all_end(fh, buf, status);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_write_ordered ================== */
_EXTERN_C_ int PMPI_File_write_ordered(MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status);
_EXTERN_C_ int MPI_File_write_ordered(MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_File_write_ordered) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_File_write_ordered", strlen("MPI_File_write_ordered")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_write_ordered(fh, buf, count, datatype, status);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_write_ordered(fh, buf, count, datatype, status);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_write_ordered_begin ================== */
_EXTERN_C_ int PMPI_File_write_ordered_begin(MPI_File fh, const void *buf, int count, MPI_Datatype datatype);
_EXTERN_C_ int MPI_File_write_ordered_begin(MPI_File fh, const void *buf, int count, MPI_Datatype datatype) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_File_write_ordered_begin) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_File_write_ordered_begin", strlen("MPI_File_write_ordered_begin")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_write_ordered_begin(fh, buf, count, datatype);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_write_ordered_begin(fh, buf, count, datatype);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_write_ordered_end ================== */
_EXTERN_C_ int PMPI_File_write_ordered_end(MPI_File fh, const void *buf, MPI_Status *status);
_EXTERN_C_ int MPI_File_write_ordered_end(MPI_File fh, const void *buf, MPI_Status *status) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_File_write_ordered_end) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_File_write_ordered_end", strlen("MPI_File_write_ordered_end")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_write_ordered_end(fh, buf, status);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_write_ordered_end(fh, buf, status);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_write_shared ================== */
_EXTERN_C_ int PMPI_File_write_shared(MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status);
_EXTERN_C_ int MPI_File_write_shared(MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_File_write_shared) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_File_write_shared", strlen("MPI_File_write_shared")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_write_shared(fh, buf, count, datatype, status);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_File_write_shared(fh, buf, count, datatype, status);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Finalized ================== */
_EXTERN_C_ int PMPI_Finalized(int *flag);
_EXTERN_C_ int MPI_Finalized(int *flag) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Finalized) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Finalized", strlen("MPI_Finalized")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Finalized(flag);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Finalized(flag);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Free_mem ================== */
_EXTERN_C_ int PMPI_Free_mem(void *base);
_EXTERN_C_ int MPI_Free_mem(void *base) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Free_mem) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Free_mem", strlen("MPI_Free_mem")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Free_mem(base);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Free_mem(base);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Get ================== */
_EXTERN_C_ int PMPI_Get(void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Win win);
_EXTERN_C_ int MPI_Get(void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Win win) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Get) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Get", strlen("MPI_Get")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Get(origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_count, target_datatype, win);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Get(origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_count, target_datatype, win);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Get_accumulate ================== */
_EXTERN_C_ int PMPI_Get_accumulate(const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, void *result_addr, int result_count, MPI_Datatype result_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Op op, MPI_Win win);
_EXTERN_C_ int MPI_Get_accumulate(const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, void *result_addr, int result_count, MPI_Datatype result_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Op op, MPI_Win win) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Get_accumulate) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Get_accumulate", strlen("MPI_Get_accumulate")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Get_accumulate(origin_addr, origin_count, origin_datatype, result_addr, result_count, result_datatype, target_rank, target_disp, target_count, target_datatype, op, win);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Get_accumulate(origin_addr, origin_count, origin_datatype, result_addr, result_count, result_datatype, target_rank, target_disp, target_count, target_datatype, op, win);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Get_address ================== */
_EXTERN_C_ int PMPI_Get_address(const void *location, MPI_Aint *address);
_EXTERN_C_ int MPI_Get_address(const void *location, MPI_Aint *address) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Get_address) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Get_address", strlen("MPI_Get_address")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Get_address(location, address);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Get_address(location, address);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Get_count ================== */
_EXTERN_C_ int PMPI_Get_count(const MPI_Status *status, MPI_Datatype datatype, int *count);
_EXTERN_C_ int MPI_Get_count(const MPI_Status *status, MPI_Datatype datatype, int *count) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Get_count) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Get_count", strlen("MPI_Get_count")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Get_count(status, datatype, count);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Get_count(status, datatype, count);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Get_elements ================== */
_EXTERN_C_ int PMPI_Get_elements(const MPI_Status *status, MPI_Datatype datatype, int *count);
_EXTERN_C_ int MPI_Get_elements(const MPI_Status *status, MPI_Datatype datatype, int *count) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Get_elements) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Get_elements", strlen("MPI_Get_elements")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Get_elements(status, datatype, count);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Get_elements(status, datatype, count);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Get_elements_x ================== */
_EXTERN_C_ int PMPI_Get_elements_x(const MPI_Status *status, MPI_Datatype datatype, MPI_Count *count);
_EXTERN_C_ int MPI_Get_elements_x(const MPI_Status *status, MPI_Datatype datatype, MPI_Count *count) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Get_elements_x) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Get_elements_x", strlen("MPI_Get_elements_x")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Get_elements_x(status, datatype, count);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Get_elements_x(status, datatype, count);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Get_library_version ================== */
_EXTERN_C_ int PMPI_Get_library_version(char *version, int *resultlen);
_EXTERN_C_ int MPI_Get_library_version(char *version, int *resultlen) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Get_library_version) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Get_library_version", strlen("MPI_Get_library_version")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Get_library_version(version, resultlen);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Get_library_version(version, resultlen);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Get_processor_name ================== */
_EXTERN_C_ int PMPI_Get_processor_name(char *name, int *resultlen);
_EXTERN_C_ int MPI_Get_processor_name(char *name, int *resultlen) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Get_processor_name) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Get_processor_name", strlen("MPI_Get_processor_name")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Get_processor_name(name, resultlen);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Get_processor_name(name, resultlen);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Get_version ================== */
_EXTERN_C_ int PMPI_Get_version(int *version, int *subversion);
_EXTERN_C_ int MPI_Get_version(int *version, int *subversion) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Get_version) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Get_version", strlen("MPI_Get_version")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Get_version(version, subversion);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Get_version(version, subversion);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Graph_create ================== */
_EXTERN_C_ int PMPI_Graph_create(MPI_Comm comm_old, int nnodes, const int index[], const int edges[], int reorder, MPI_Comm *comm_graph);
_EXTERN_C_ int MPI_Graph_create(MPI_Comm comm_old, int nnodes, const int index[], const int edges[], int reorder, MPI_Comm *comm_graph) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Graph_create) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Graph_create", strlen("MPI_Graph_create")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Graph_create(comm_old, nnodes, index, edges, reorder, comm_graph);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Graph_create(comm_old, nnodes, index, edges, reorder, comm_graph);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Graph_get ================== */
_EXTERN_C_ int PMPI_Graph_get(MPI_Comm comm, int maxindex, int maxedges, int index[], int edges[]);
_EXTERN_C_ int MPI_Graph_get(MPI_Comm comm, int maxindex, int maxedges, int index[], int edges[]) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Graph_get) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Graph_get", strlen("MPI_Graph_get")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Graph_get(comm, maxindex, maxedges, index, edges);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Graph_get(comm, maxindex, maxedges, index, edges);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Graph_map ================== */
_EXTERN_C_ int PMPI_Graph_map(MPI_Comm comm, int nnodes, const int index[], const int edges[], int *newrank);
_EXTERN_C_ int MPI_Graph_map(MPI_Comm comm, int nnodes, const int index[], const int edges[], int *newrank) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Graph_map) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Graph_map", strlen("MPI_Graph_map")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Graph_map(comm, nnodes, index, edges, newrank);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Graph_map(comm, nnodes, index, edges, newrank);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Graph_neighbors ================== */
_EXTERN_C_ int PMPI_Graph_neighbors(MPI_Comm comm, int rank, int maxneighbors, int neighbors[]);
_EXTERN_C_ int MPI_Graph_neighbors(MPI_Comm comm, int rank, int maxneighbors, int neighbors[]) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Graph_neighbors) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Graph_neighbors", strlen("MPI_Graph_neighbors")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Graph_neighbors(comm, rank, maxneighbors, neighbors);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Graph_neighbors(comm, rank, maxneighbors, neighbors);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Graph_neighbors_count ================== */
_EXTERN_C_ int PMPI_Graph_neighbors_count(MPI_Comm comm, int rank, int *nneighbors);
_EXTERN_C_ int MPI_Graph_neighbors_count(MPI_Comm comm, int rank, int *nneighbors) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Graph_neighbors_count) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Graph_neighbors_count", strlen("MPI_Graph_neighbors_count")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Graph_neighbors_count(comm, rank, nneighbors);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Graph_neighbors_count(comm, rank, nneighbors);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Graphdims_get ================== */
_EXTERN_C_ int PMPI_Graphdims_get(MPI_Comm comm, int *nnodes, int *nedges);
_EXTERN_C_ int MPI_Graphdims_get(MPI_Comm comm, int *nnodes, int *nedges) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Graphdims_get) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Graphdims_get", strlen("MPI_Graphdims_get")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Graphdims_get(comm, nnodes, nedges);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Graphdims_get(comm, nnodes, nedges);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Grequest_complete ================== */
_EXTERN_C_ int PMPI_Grequest_complete(MPI_Request request);
_EXTERN_C_ int MPI_Grequest_complete(MPI_Request request) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Grequest_complete) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Grequest_complete", strlen("MPI_Grequest_complete")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Grequest_complete(request);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Grequest_complete(request);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Grequest_start ================== */
_EXTERN_C_ int PMPI_Grequest_start(MPI_Grequest_query_function *query_fn, MPI_Grequest_free_function *free_fn, MPI_Grequest_cancel_function *cancel_fn, void *extra_state, MPI_Request *request);
_EXTERN_C_ int MPI_Grequest_start(MPI_Grequest_query_function *query_fn, MPI_Grequest_free_function *free_fn, MPI_Grequest_cancel_function *cancel_fn, void *extra_state, MPI_Request *request) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Grequest_start) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Grequest_start", strlen("MPI_Grequest_start")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Grequest_start(query_fn, free_fn, cancel_fn, extra_state, request);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Grequest_start(query_fn, free_fn, cancel_fn, extra_state, request);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Group_compare ================== */
_EXTERN_C_ int PMPI_Group_compare(MPI_Group group1, MPI_Group group2, int *result);
_EXTERN_C_ int MPI_Group_compare(MPI_Group group1, MPI_Group group2, int *result) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Group_compare) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Group_compare", strlen("MPI_Group_compare")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Group_compare(group1, group2, result);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Group_compare(group1, group2, result);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Group_difference ================== */
_EXTERN_C_ int PMPI_Group_difference(MPI_Group group1, MPI_Group group2, MPI_Group *newgroup);
_EXTERN_C_ int MPI_Group_difference(MPI_Group group1, MPI_Group group2, MPI_Group *newgroup) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Group_difference) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Group_difference", strlen("MPI_Group_difference")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Group_difference(group1, group2, newgroup);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Group_difference(group1, group2, newgroup);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Group_excl ================== */
_EXTERN_C_ int PMPI_Group_excl(MPI_Group group, int n, const int ranks[], MPI_Group *newgroup);
_EXTERN_C_ int MPI_Group_excl(MPI_Group group, int n, const int ranks[], MPI_Group *newgroup) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Group_excl) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Group_excl", strlen("MPI_Group_excl")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Group_excl(group, n, ranks, newgroup);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Group_excl(group, n, ranks, newgroup);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Group_free ================== */
_EXTERN_C_ int PMPI_Group_free(MPI_Group *group);
_EXTERN_C_ int MPI_Group_free(MPI_Group *group) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Group_free) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Group_free", strlen("MPI_Group_free")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Group_free(group);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Group_free(group);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Group_incl ================== */
_EXTERN_C_ int PMPI_Group_incl(MPI_Group group, int n, const int ranks[], MPI_Group *newgroup);
_EXTERN_C_ int MPI_Group_incl(MPI_Group group, int n, const int ranks[], MPI_Group *newgroup) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Group_incl) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Group_incl", strlen("MPI_Group_incl")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Group_incl(group, n, ranks, newgroup);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Group_incl(group, n, ranks, newgroup);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Group_intersection ================== */
_EXTERN_C_ int PMPI_Group_intersection(MPI_Group group1, MPI_Group group2, MPI_Group *newgroup);
_EXTERN_C_ int MPI_Group_intersection(MPI_Group group1, MPI_Group group2, MPI_Group *newgroup) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Group_intersection) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Group_intersection", strlen("MPI_Group_intersection")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Group_intersection(group1, group2, newgroup);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Group_intersection(group1, group2, newgroup);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Group_range_excl ================== */
_EXTERN_C_ int PMPI_Group_range_excl(MPI_Group group, int n, int ranges[][3], MPI_Group *newgroup);
_EXTERN_C_ int MPI_Group_range_excl(MPI_Group group, int n, int ranges[][3], MPI_Group *newgroup) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Group_range_excl) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Group_range_excl", strlen("MPI_Group_range_excl")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Group_range_excl(group, n, ranges, newgroup);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Group_range_excl(group, n, ranges, newgroup);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Group_range_incl ================== */
_EXTERN_C_ int PMPI_Group_range_incl(MPI_Group group, int n, int ranges[][3], MPI_Group *newgroup);
_EXTERN_C_ int MPI_Group_range_incl(MPI_Group group, int n, int ranges[][3], MPI_Group *newgroup) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Group_range_incl) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Group_range_incl", strlen("MPI_Group_range_incl")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Group_range_incl(group, n, ranges, newgroup);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Group_range_incl(group, n, ranges, newgroup);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Group_rank ================== */
_EXTERN_C_ int PMPI_Group_rank(MPI_Group group, int *rank);
_EXTERN_C_ int MPI_Group_rank(MPI_Group group, int *rank) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Group_rank) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Group_rank", strlen("MPI_Group_rank")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Group_rank(group, rank);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Group_rank(group, rank);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Group_size ================== */
_EXTERN_C_ int PMPI_Group_size(MPI_Group group, int *size);
_EXTERN_C_ int MPI_Group_size(MPI_Group group, int *size) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Group_size) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Group_size", strlen("MPI_Group_size")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Group_size(group, size);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Group_size(group, size);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Group_translate_ranks ================== */
_EXTERN_C_ int PMPI_Group_translate_ranks(MPI_Group group1, int n, const int ranks1[], MPI_Group group2, int ranks2[]);
_EXTERN_C_ int MPI_Group_translate_ranks(MPI_Group group1, int n, const int ranks1[], MPI_Group group2, int ranks2[]) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Group_translate_ranks) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Group_translate_ranks", strlen("MPI_Group_translate_ranks")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Group_translate_ranks(group1, n, ranks1, group2, ranks2);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Group_translate_ranks(group1, n, ranks1, group2, ranks2);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Group_union ================== */
_EXTERN_C_ int PMPI_Group_union(MPI_Group group1, MPI_Group group2, MPI_Group *newgroup);
_EXTERN_C_ int MPI_Group_union(MPI_Group group1, MPI_Group group2, MPI_Group *newgroup) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Group_union) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Group_union", strlen("MPI_Group_union")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Group_union(group1, group2, newgroup);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Group_union(group1, group2, newgroup);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Iallgather ================== */
_EXTERN_C_ int PMPI_Iallgather(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Iallgather(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Iallgather) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Iallgather", strlen("MPI_Iallgather")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Iallgather(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm, request);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Iallgather(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm, request);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Iallgatherv ================== */
_EXTERN_C_ int PMPI_Iallgatherv(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Iallgatherv(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Iallgatherv) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Iallgatherv", strlen("MPI_Iallgatherv")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Iallgatherv(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, comm, request);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Iallgatherv(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, comm, request);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Iallreduce ================== */
_EXTERN_C_ int PMPI_Iallreduce(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Iallreduce(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Iallreduce) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Iallreduce", strlen("MPI_Iallreduce")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Iallreduce(sendbuf, recvbuf, count, datatype, op, comm, request);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Iallreduce(sendbuf, recvbuf, count, datatype, op, comm, request);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Ialltoall ================== */
_EXTERN_C_ int PMPI_Ialltoall(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Ialltoall(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Ialltoall) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Ialltoall", strlen("MPI_Ialltoall")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Ialltoall(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm, request);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Ialltoall(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm, request);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Ialltoallv ================== */
_EXTERN_C_ int PMPI_Ialltoallv(const void *sendbuf, const int sendcounts[], const int sdispls[], MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int rdispls[], MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Ialltoallv(const void *sendbuf, const int sendcounts[], const int sdispls[], MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int rdispls[], MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Ialltoallv) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Ialltoallv", strlen("MPI_Ialltoallv")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Ialltoallv(sendbuf, sendcounts, sdispls, sendtype, recvbuf, recvcounts, rdispls, recvtype, comm, request);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Ialltoallv(sendbuf, sendcounts, sdispls, sendtype, recvbuf, recvcounts, rdispls, recvtype, comm, request);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Ialltoallw ================== */
_EXTERN_C_ int PMPI_Ialltoallw(const void *sendbuf, const int sendcounts[], const int sdispls[], const MPI_Datatype sendtypes[], void *recvbuf, const int recvcounts[], const int rdispls[], const MPI_Datatype recvtypes[], MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Ialltoallw(const void *sendbuf, const int sendcounts[], const int sdispls[], const MPI_Datatype sendtypes[], void *recvbuf, const int recvcounts[], const int rdispls[], const MPI_Datatype recvtypes[], MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Ialltoallw) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Ialltoallw", strlen("MPI_Ialltoallw")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Ialltoallw(sendbuf, sendcounts, sdispls, sendtypes, recvbuf, recvcounts, rdispls, recvtypes, comm, request);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Ialltoallw(sendbuf, sendcounts, sdispls, sendtypes, recvbuf, recvcounts, rdispls, recvtypes, comm, request);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Ibarrier ================== */
_EXTERN_C_ int PMPI_Ibarrier(MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Ibarrier(MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Ibarrier) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Ibarrier", strlen("MPI_Ibarrier")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Ibarrier(comm, request);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Ibarrier(comm, request);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Ibcast ================== */
_EXTERN_C_ int PMPI_Ibcast(void *buffer, int count, MPI_Datatype datatype, int root, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Ibcast(void *buffer, int count, MPI_Datatype datatype, int root, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Ibcast) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Ibcast", strlen("MPI_Ibcast")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Ibcast(buffer, count, datatype, root, comm, request);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Ibcast(buffer, count, datatype, root, comm, request);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Iexscan ================== */
_EXTERN_C_ int PMPI_Iexscan(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Iexscan(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Iexscan) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Iexscan", strlen("MPI_Iexscan")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Iexscan(sendbuf, recvbuf, count, datatype, op, comm, request);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Iexscan(sendbuf, recvbuf, count, datatype, op, comm, request);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Igather ================== */
_EXTERN_C_ int PMPI_Igather(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Igather(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Igather) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Igather", strlen("MPI_Igather")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Igather(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm, request);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Igather(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm, request);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Igatherv ================== */
_EXTERN_C_ int PMPI_Igatherv(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, int root, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Igatherv(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, int root, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Igatherv) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Igatherv", strlen("MPI_Igatherv")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Igatherv(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, root, comm, request);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Igatherv(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, root, comm, request);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Improbe ================== */
_EXTERN_C_ int PMPI_Improbe(int source, int tag, MPI_Comm comm, int *flag, MPI_Message *message, MPI_Status *status);
_EXTERN_C_ int MPI_Improbe(int source, int tag, MPI_Comm comm, int *flag, MPI_Message *message, MPI_Status *status) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Improbe) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Improbe", strlen("MPI_Improbe")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Improbe(source, tag, comm, flag, message, status);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Improbe(source, tag, comm, flag, message, status);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Imrecv ================== */
_EXTERN_C_ int PMPI_Imrecv(void *buf, int count, MPI_Datatype type, MPI_Message *message, MPI_Request *request);
_EXTERN_C_ int MPI_Imrecv(void *buf, int count, MPI_Datatype type, MPI_Message *message, MPI_Request *request) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Imrecv) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Imrecv", strlen("MPI_Imrecv")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Imrecv(buf, count, type, message, request);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Imrecv(buf, count, type, message, request);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Ineighbor_allgather ================== */
_EXTERN_C_ int PMPI_Ineighbor_allgather(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Ineighbor_allgather(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Ineighbor_allgather) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Ineighbor_allgather", strlen("MPI_Ineighbor_allgather")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Ineighbor_allgather(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm, request);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Ineighbor_allgather(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm, request);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Ineighbor_allgatherv ================== */
_EXTERN_C_ int PMPI_Ineighbor_allgatherv(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Ineighbor_allgatherv(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Ineighbor_allgatherv) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Ineighbor_allgatherv", strlen("MPI_Ineighbor_allgatherv")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Ineighbor_allgatherv(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, comm, request);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Ineighbor_allgatherv(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, comm, request);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Ineighbor_alltoall ================== */
_EXTERN_C_ int PMPI_Ineighbor_alltoall(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Ineighbor_alltoall(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Ineighbor_alltoall) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Ineighbor_alltoall", strlen("MPI_Ineighbor_alltoall")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Ineighbor_alltoall(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm, request);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Ineighbor_alltoall(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm, request);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Ineighbor_alltoallv ================== */
_EXTERN_C_ int PMPI_Ineighbor_alltoallv(const void *sendbuf, const int sendcounts[], const int sdispls[], MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int rdispls[], MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Ineighbor_alltoallv(const void *sendbuf, const int sendcounts[], const int sdispls[], MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int rdispls[], MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Ineighbor_alltoallv) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Ineighbor_alltoallv", strlen("MPI_Ineighbor_alltoallv")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Ineighbor_alltoallv(sendbuf, sendcounts, sdispls, sendtype, recvbuf, recvcounts, rdispls, recvtype, comm, request);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Ineighbor_alltoallv(sendbuf, sendcounts, sdispls, sendtype, recvbuf, recvcounts, rdispls, recvtype, comm, request);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Ineighbor_alltoallw ================== */
_EXTERN_C_ int PMPI_Ineighbor_alltoallw(const void *sendbuf, const int sendcounts[], const MPI_Aint sdispls[], const MPI_Datatype sendtypes[], void *recvbuf, const int recvcounts[], const MPI_Aint rdispls[], const MPI_Datatype recvtypes[], MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Ineighbor_alltoallw(const void *sendbuf, const int sendcounts[], const MPI_Aint sdispls[], const MPI_Datatype sendtypes[], void *recvbuf, const int recvcounts[], const MPI_Aint rdispls[], const MPI_Datatype recvtypes[], MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Ineighbor_alltoallw) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Ineighbor_alltoallw", strlen("MPI_Ineighbor_alltoallw")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Ineighbor_alltoallw(sendbuf, sendcounts, sdispls, sendtypes, recvbuf, recvcounts, rdispls, recvtypes, comm, request);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Ineighbor_alltoallw(sendbuf, sendcounts, sdispls, sendtypes, recvbuf, recvcounts, rdispls, recvtypes, comm, request);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Info_create ================== */
_EXTERN_C_ int PMPI_Info_create(MPI_Info *info);
_EXTERN_C_ int MPI_Info_create(MPI_Info *info) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Info_create) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Info_create", strlen("MPI_Info_create")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Info_create(info);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Info_create(info);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Info_delete ================== */
_EXTERN_C_ int PMPI_Info_delete(MPI_Info info, const char *key);
_EXTERN_C_ int MPI_Info_delete(MPI_Info info, const char *key) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Info_delete) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Info_delete", strlen("MPI_Info_delete")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Info_delete(info, key);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Info_delete(info, key);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Info_dup ================== */
_EXTERN_C_ int PMPI_Info_dup(MPI_Info info, MPI_Info *newinfo);
_EXTERN_C_ int MPI_Info_dup(MPI_Info info, MPI_Info *newinfo) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Info_dup) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Info_dup", strlen("MPI_Info_dup")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Info_dup(info, newinfo);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Info_dup(info, newinfo);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Info_free ================== */
_EXTERN_C_ int PMPI_Info_free(MPI_Info *info);
_EXTERN_C_ int MPI_Info_free(MPI_Info *info) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Info_free) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Info_free", strlen("MPI_Info_free")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Info_free(info);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Info_free(info);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Info_get ================== */
_EXTERN_C_ int PMPI_Info_get(MPI_Info info, const char *key, int valuelen, char *value, int *flag);
_EXTERN_C_ int MPI_Info_get(MPI_Info info, const char *key, int valuelen, char *value, int *flag) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Info_get) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Info_get", strlen("MPI_Info_get")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Info_get(info, key, valuelen, value, flag);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Info_get(info, key, valuelen, value, flag);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Info_get_nkeys ================== */
_EXTERN_C_ int PMPI_Info_get_nkeys(MPI_Info info, int *nkeys);
_EXTERN_C_ int MPI_Info_get_nkeys(MPI_Info info, int *nkeys) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Info_get_nkeys) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Info_get_nkeys", strlen("MPI_Info_get_nkeys")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Info_get_nkeys(info, nkeys);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Info_get_nkeys(info, nkeys);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Info_get_nthkey ================== */
_EXTERN_C_ int PMPI_Info_get_nthkey(MPI_Info info, int n, char *key);
_EXTERN_C_ int MPI_Info_get_nthkey(MPI_Info info, int n, char *key) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Info_get_nthkey) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Info_get_nthkey", strlen("MPI_Info_get_nthkey")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Info_get_nthkey(info, n, key);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Info_get_nthkey(info, n, key);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Info_get_valuelen ================== */
_EXTERN_C_ int PMPI_Info_get_valuelen(MPI_Info info, const char *key, int *valuelen, int *flag);
_EXTERN_C_ int MPI_Info_get_valuelen(MPI_Info info, const char *key, int *valuelen, int *flag) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Info_get_valuelen) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Info_get_valuelen", strlen("MPI_Info_get_valuelen")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Info_get_valuelen(info, key, valuelen, flag);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Info_get_valuelen(info, key, valuelen, flag);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Info_set ================== */
_EXTERN_C_ int PMPI_Info_set(MPI_Info info, const char *key, const char *value);
_EXTERN_C_ int MPI_Info_set(MPI_Info info, const char *key, const char *value) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Info_set) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Info_set", strlen("MPI_Info_set")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Info_set(info, key, value);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Info_set(info, key, value);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Initialized ================== */
_EXTERN_C_ int PMPI_Initialized(int *flag);
_EXTERN_C_ int MPI_Initialized(int *flag) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Initialized) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Initialized", strlen("MPI_Initialized")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Initialized(flag);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Initialized(flag);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Intercomm_create ================== */
_EXTERN_C_ int PMPI_Intercomm_create(MPI_Comm local_comm, int local_leader, MPI_Comm bridge_comm, int remote_leader, int tag, MPI_Comm *newintercomm);
_EXTERN_C_ int MPI_Intercomm_create(MPI_Comm local_comm, int local_leader, MPI_Comm bridge_comm, int remote_leader, int tag, MPI_Comm *newintercomm) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Intercomm_create) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Intercomm_create", strlen("MPI_Intercomm_create")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Intercomm_create(local_comm, local_leader, bridge_comm, remote_leader, tag, newintercomm);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Intercomm_create(local_comm, local_leader, bridge_comm, remote_leader, tag, newintercomm);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Intercomm_merge ================== */
_EXTERN_C_ int PMPI_Intercomm_merge(MPI_Comm intercomm, int high, MPI_Comm *newintercomm);
_EXTERN_C_ int MPI_Intercomm_merge(MPI_Comm intercomm, int high, MPI_Comm *newintercomm) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Intercomm_merge) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Intercomm_merge", strlen("MPI_Intercomm_merge")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Intercomm_merge(intercomm, high, newintercomm);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Intercomm_merge(intercomm, high, newintercomm);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Iprobe ================== */
_EXTERN_C_ int PMPI_Iprobe(int source, int tag, MPI_Comm comm, int *flag, MPI_Status *status);
_EXTERN_C_ int MPI_Iprobe(int source, int tag, MPI_Comm comm, int *flag, MPI_Status *status) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Iprobe) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Iprobe", strlen("MPI_Iprobe")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Iprobe(source, tag, comm, flag, status);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Iprobe(source, tag, comm, flag, status);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Ireduce ================== */
_EXTERN_C_ int PMPI_Ireduce(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, int root, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Ireduce(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, int root, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Ireduce) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Ireduce", strlen("MPI_Ireduce")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Ireduce(sendbuf, recvbuf, count, datatype, op, root, comm, request);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Ireduce(sendbuf, recvbuf, count, datatype, op, root, comm, request);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Ireduce_scatter ================== */
_EXTERN_C_ int PMPI_Ireduce_scatter(const void *sendbuf, void *recvbuf, const int recvcounts[], MPI_Datatype datatype, MPI_Op op, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Ireduce_scatter(const void *sendbuf, void *recvbuf, const int recvcounts[], MPI_Datatype datatype, MPI_Op op, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Ireduce_scatter) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Ireduce_scatter", strlen("MPI_Ireduce_scatter")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Ireduce_scatter(sendbuf, recvbuf, recvcounts, datatype, op, comm, request);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Ireduce_scatter(sendbuf, recvbuf, recvcounts, datatype, op, comm, request);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Ireduce_scatter_block ================== */
_EXTERN_C_ int PMPI_Ireduce_scatter_block(const void *sendbuf, void *recvbuf, int recvcount, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Ireduce_scatter_block(const void *sendbuf, void *recvbuf, int recvcount, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Ireduce_scatter_block) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Ireduce_scatter_block", strlen("MPI_Ireduce_scatter_block")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Ireduce_scatter_block(sendbuf, recvbuf, recvcount, datatype, op, comm, request);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Ireduce_scatter_block(sendbuf, recvbuf, recvcount, datatype, op, comm, request);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Is_thread_main ================== */
_EXTERN_C_ int PMPI_Is_thread_main(int *flag);
_EXTERN_C_ int MPI_Is_thread_main(int *flag) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Is_thread_main) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Is_thread_main", strlen("MPI_Is_thread_main")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Is_thread_main(flag);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Is_thread_main(flag);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Iscan ================== */
_EXTERN_C_ int PMPI_Iscan(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Iscan(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Iscan) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Iscan", strlen("MPI_Iscan")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Iscan(sendbuf, recvbuf, count, datatype, op, comm, request);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Iscan(sendbuf, recvbuf, count, datatype, op, comm, request);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Iscatter ================== */
_EXTERN_C_ int PMPI_Iscatter(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Iscatter(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Iscatter) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Iscatter", strlen("MPI_Iscatter")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Iscatter(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm, request);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Iscatter(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm, request);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Iscatterv ================== */
_EXTERN_C_ int PMPI_Iscatterv(const void *sendbuf, const int sendcounts[], const int displs[], MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Iscatterv(const void *sendbuf, const int sendcounts[], const int displs[], MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Iscatterv) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Iscatterv", strlen("MPI_Iscatterv")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Iscatterv(sendbuf, sendcounts, displs, sendtype, recvbuf, recvcount, recvtype, root, comm, request);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Iscatterv(sendbuf, sendcounts, displs, sendtype, recvbuf, recvcount, recvtype, root, comm, request);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Keyval_create ================== */
_EXTERN_C_ int PMPI_Keyval_create(MPI_Copy_function *copy_fn, MPI_Delete_function *delete_fn, int *keyval, void *extra_state);
_EXTERN_C_ int MPI_Keyval_create(MPI_Copy_function *copy_fn, MPI_Delete_function *delete_fn, int *keyval, void *extra_state) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Keyval_create) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Keyval_create", strlen("MPI_Keyval_create")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Keyval_create(copy_fn, delete_fn, keyval, extra_state);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Keyval_create(copy_fn, delete_fn, keyval, extra_state);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Keyval_free ================== */
_EXTERN_C_ int PMPI_Keyval_free(int *keyval);
_EXTERN_C_ int MPI_Keyval_free(int *keyval) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Keyval_free) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Keyval_free", strlen("MPI_Keyval_free")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Keyval_free(keyval);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Keyval_free(keyval);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Lookup_name ================== */
_EXTERN_C_ int PMPI_Lookup_name(const char *service_name, MPI_Info info, char *port_name);
_EXTERN_C_ int MPI_Lookup_name(const char *service_name, MPI_Info info, char *port_name) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Lookup_name) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Lookup_name", strlen("MPI_Lookup_name")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Lookup_name(service_name, info, port_name);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Lookup_name(service_name, info, port_name);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Mprobe ================== */
_EXTERN_C_ int PMPI_Mprobe(int source, int tag, MPI_Comm comm, MPI_Message *message, MPI_Status *status);
_EXTERN_C_ int MPI_Mprobe(int source, int tag, MPI_Comm comm, MPI_Message *message, MPI_Status *status) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Mprobe) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Mprobe", strlen("MPI_Mprobe")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Mprobe(source, tag, comm, message, status);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Mprobe(source, tag, comm, message, status);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Mrecv ================== */
_EXTERN_C_ int PMPI_Mrecv(void *buf, int count, MPI_Datatype type, MPI_Message *message, MPI_Status *status);
_EXTERN_C_ int MPI_Mrecv(void *buf, int count, MPI_Datatype type, MPI_Message *message, MPI_Status *status) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Mrecv) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Mrecv", strlen("MPI_Mrecv")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Mrecv(buf, count, type, message, status);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Mrecv(buf, count, type, message, status);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Neighbor_allgather ================== */
_EXTERN_C_ int PMPI_Neighbor_allgather(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm);
_EXTERN_C_ int MPI_Neighbor_allgather(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Neighbor_allgather) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Neighbor_allgather", strlen("MPI_Neighbor_allgather")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Neighbor_allgather(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Neighbor_allgather(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Neighbor_allgatherv ================== */
_EXTERN_C_ int PMPI_Neighbor_allgatherv(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, MPI_Comm comm);
_EXTERN_C_ int MPI_Neighbor_allgatherv(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, MPI_Comm comm) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Neighbor_allgatherv) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Neighbor_allgatherv", strlen("MPI_Neighbor_allgatherv")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Neighbor_allgatherv(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, comm);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Neighbor_allgatherv(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, comm);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Neighbor_alltoall ================== */
_EXTERN_C_ int PMPI_Neighbor_alltoall(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm);
_EXTERN_C_ int MPI_Neighbor_alltoall(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Neighbor_alltoall) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Neighbor_alltoall", strlen("MPI_Neighbor_alltoall")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Neighbor_alltoall(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Neighbor_alltoall(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Neighbor_alltoallv ================== */
_EXTERN_C_ int PMPI_Neighbor_alltoallv(const void *sendbuf, const int sendcounts[], const int sdispls[], MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int rdispls[], MPI_Datatype recvtype, MPI_Comm comm);
_EXTERN_C_ int MPI_Neighbor_alltoallv(const void *sendbuf, const int sendcounts[], const int sdispls[], MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int rdispls[], MPI_Datatype recvtype, MPI_Comm comm) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Neighbor_alltoallv) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Neighbor_alltoallv", strlen("MPI_Neighbor_alltoallv")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Neighbor_alltoallv(sendbuf, sendcounts, sdispls, sendtype, recvbuf, recvcounts, rdispls, recvtype, comm);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Neighbor_alltoallv(sendbuf, sendcounts, sdispls, sendtype, recvbuf, recvcounts, rdispls, recvtype, comm);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Neighbor_alltoallw ================== */
_EXTERN_C_ int PMPI_Neighbor_alltoallw(const void *sendbuf, const int sendcounts[], const MPI_Aint sdispls[], const MPI_Datatype sendtypes[], void *recvbuf, const int recvcounts[], const MPI_Aint rdispls[], const MPI_Datatype recvtypes[], MPI_Comm comm);
_EXTERN_C_ int MPI_Neighbor_alltoallw(const void *sendbuf, const int sendcounts[], const MPI_Aint sdispls[], const MPI_Datatype sendtypes[], void *recvbuf, const int recvcounts[], const MPI_Aint rdispls[], const MPI_Datatype recvtypes[], MPI_Comm comm) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Neighbor_alltoallw) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Neighbor_alltoallw", strlen("MPI_Neighbor_alltoallw")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Neighbor_alltoallw(sendbuf, sendcounts, sdispls, sendtypes, recvbuf, recvcounts, rdispls, recvtypes, comm);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Neighbor_alltoallw(sendbuf, sendcounts, sdispls, sendtypes, recvbuf, recvcounts, rdispls, recvtypes, comm);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Op_commutative ================== */
_EXTERN_C_ int PMPI_Op_commutative(MPI_Op op, int *commute);
_EXTERN_C_ int MPI_Op_commutative(MPI_Op op, int *commute) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Op_commutative) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Op_commutative", strlen("MPI_Op_commutative")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Op_commutative(op, commute);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Op_commutative(op, commute);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Op_create ================== */
_EXTERN_C_ int PMPI_Op_create(MPI_User_function *function, int commute, MPI_Op *op);
_EXTERN_C_ int MPI_Op_create(MPI_User_function *function, int commute, MPI_Op *op) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Op_create) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Op_create", strlen("MPI_Op_create")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Op_create(function, commute, op);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Op_create(function, commute, op);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Op_free ================== */
_EXTERN_C_ int PMPI_Op_free(MPI_Op *op);
_EXTERN_C_ int MPI_Op_free(MPI_Op *op) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Op_free) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Op_free", strlen("MPI_Op_free")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Op_free(op);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Op_free(op);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Open_port ================== */
_EXTERN_C_ int PMPI_Open_port(MPI_Info info, char *port_name);
_EXTERN_C_ int MPI_Open_port(MPI_Info info, char *port_name) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Open_port) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Open_port", strlen("MPI_Open_port")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Open_port(info, port_name);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Open_port(info, port_name);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Pack ================== */
_EXTERN_C_ int PMPI_Pack(const void *inbuf, int incount, MPI_Datatype datatype, void *outbuf, int outsize, int *position, MPI_Comm comm);
_EXTERN_C_ int MPI_Pack(const void *inbuf, int incount, MPI_Datatype datatype, void *outbuf, int outsize, int *position, MPI_Comm comm) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Pack) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Pack", strlen("MPI_Pack")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Pack(inbuf, incount, datatype, outbuf, outsize, position, comm);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Pack(inbuf, incount, datatype, outbuf, outsize, position, comm);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Pack_external ================== */
_EXTERN_C_ int PMPI_Pack_external(const char datarep[], const void *inbuf, int incount, MPI_Datatype datatype, void *outbuf, MPI_Aint outsize, MPI_Aint *position);
_EXTERN_C_ int MPI_Pack_external(const char datarep[], const void *inbuf, int incount, MPI_Datatype datatype, void *outbuf, MPI_Aint outsize, MPI_Aint *position) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Pack_external) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Pack_external", strlen("MPI_Pack_external")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Pack_external(datarep, inbuf, incount, datatype, outbuf, outsize, position);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Pack_external(datarep, inbuf, incount, datatype, outbuf, outsize, position);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Pack_external_size ================== */
_EXTERN_C_ int PMPI_Pack_external_size(const char datarep[], int incount, MPI_Datatype datatype, MPI_Aint *size);
_EXTERN_C_ int MPI_Pack_external_size(const char datarep[], int incount, MPI_Datatype datatype, MPI_Aint *size) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Pack_external_size) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Pack_external_size", strlen("MPI_Pack_external_size")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Pack_external_size(datarep, incount, datatype, size);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Pack_external_size(datarep, incount, datatype, size);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Pack_size ================== */
_EXTERN_C_ int PMPI_Pack_size(int incount, MPI_Datatype datatype, MPI_Comm comm, int *size);
_EXTERN_C_ int MPI_Pack_size(int incount, MPI_Datatype datatype, MPI_Comm comm, int *size) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Pack_size) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Pack_size", strlen("MPI_Pack_size")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Pack_size(incount, datatype, comm, size);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Pack_size(incount, datatype, comm, size);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Pcontrol ================== */
_EXTERN_C_ int PMPI_Pcontrol(const int level, ...);
_EXTERN_C_ int MPI_Pcontrol(const int level, ...) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Pcontrol) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Pcontrol", strlen("MPI_Pcontrol")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Pcontrol(level);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Pcontrol(level);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Probe ================== */
_EXTERN_C_ int PMPI_Probe(int source, int tag, MPI_Comm comm, MPI_Status *status);
_EXTERN_C_ int MPI_Probe(int source, int tag, MPI_Comm comm, MPI_Status *status) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Probe) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Probe", strlen("MPI_Probe")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Probe(source, tag, comm, status);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Probe(source, tag, comm, status);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Publish_name ================== */
_EXTERN_C_ int PMPI_Publish_name(const char *service_name, MPI_Info info, const char *port_name);
_EXTERN_C_ int MPI_Publish_name(const char *service_name, MPI_Info info, const char *port_name) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Publish_name) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Publish_name", strlen("MPI_Publish_name")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Publish_name(service_name, info, port_name);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Publish_name(service_name, info, port_name);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Put ================== */
_EXTERN_C_ int PMPI_Put(const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Win win);
_EXTERN_C_ int MPI_Put(const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Win win) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Put) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Put", strlen("MPI_Put")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Put(origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_count, target_datatype, win);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Put(origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_count, target_datatype, win);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Query_thread ================== */
_EXTERN_C_ int PMPI_Query_thread(int *provided);
_EXTERN_C_ int MPI_Query_thread(int *provided) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Query_thread) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Query_thread", strlen("MPI_Query_thread")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Query_thread(provided);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Query_thread(provided);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Raccumulate ================== */
_EXTERN_C_ int PMPI_Raccumulate(const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Op op, MPI_Win win, MPI_Request *request);
_EXTERN_C_ int MPI_Raccumulate(const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Op op, MPI_Win win, MPI_Request *request) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Raccumulate) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Raccumulate", strlen("MPI_Raccumulate")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Raccumulate(origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_count, target_datatype, op, win, request);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Raccumulate(origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_count, target_datatype, op, win, request);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Reduce_local ================== */
_EXTERN_C_ int PMPI_Reduce_local(const void *inbuf, void *inoutbuf, int count, MPI_Datatype datatype, MPI_Op op);
_EXTERN_C_ int MPI_Reduce_local(const void *inbuf, void *inoutbuf, int count, MPI_Datatype datatype, MPI_Op op) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Reduce_local) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Reduce_local", strlen("MPI_Reduce_local")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Reduce_local(inbuf, inoutbuf, count, datatype, op);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Reduce_local(inbuf, inoutbuf, count, datatype, op);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Reduce_scatter_block ================== */
_EXTERN_C_ int PMPI_Reduce_scatter_block(const void *sendbuf, void *recvbuf, int recvcount, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm);
_EXTERN_C_ int MPI_Reduce_scatter_block(const void *sendbuf, void *recvbuf, int recvcount, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Reduce_scatter_block) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Reduce_scatter_block", strlen("MPI_Reduce_scatter_block")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Reduce_scatter_block(sendbuf, recvbuf, recvcount, datatype, op, comm);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Reduce_scatter_block(sendbuf, recvbuf, recvcount, datatype, op, comm);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Register_datarep ================== */
_EXTERN_C_ int PMPI_Register_datarep(const char *datarep, MPI_Datarep_conversion_function *read_conversion_fn, MPI_Datarep_conversion_function *write_conversion_fn, MPI_Datarep_extent_function *dtype_file_extent_fn, void *extra_state);
_EXTERN_C_ int MPI_Register_datarep(const char *datarep, MPI_Datarep_conversion_function *read_conversion_fn, MPI_Datarep_conversion_function *write_conversion_fn, MPI_Datarep_extent_function *dtype_file_extent_fn, void *extra_state) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Register_datarep) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Register_datarep", strlen("MPI_Register_datarep")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Register_datarep(datarep, read_conversion_fn, write_conversion_fn, dtype_file_extent_fn, extra_state);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Register_datarep(datarep, read_conversion_fn, write_conversion_fn, dtype_file_extent_fn, extra_state);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Request_get_status ================== */
_EXTERN_C_ int PMPI_Request_get_status(MPI_Request request, int *flag, MPI_Status *status);
_EXTERN_C_ int MPI_Request_get_status(MPI_Request request, int *flag, MPI_Status *status) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Request_get_status) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Request_get_status", strlen("MPI_Request_get_status")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Request_get_status(request, flag, status);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Request_get_status(request, flag, status);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Rget ================== */
_EXTERN_C_ int PMPI_Rget(void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Win win, MPI_Request *request);
_EXTERN_C_ int MPI_Rget(void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Win win, MPI_Request *request) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Rget) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Rget", strlen("MPI_Rget")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Rget(origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_count, target_datatype, win, request);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Rget(origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_count, target_datatype, win, request);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Rget_accumulate ================== */
_EXTERN_C_ int PMPI_Rget_accumulate(const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, void *result_addr, int result_count, MPI_Datatype result_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Op op, MPI_Win win, MPI_Request *request);
_EXTERN_C_ int MPI_Rget_accumulate(const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, void *result_addr, int result_count, MPI_Datatype result_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Op op, MPI_Win win, MPI_Request *request) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Rget_accumulate) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Rget_accumulate", strlen("MPI_Rget_accumulate")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Rget_accumulate(origin_addr, origin_count, origin_datatype, result_addr, result_count, result_datatype, target_rank, target_disp, target_count, target_datatype, op, win, request);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Rget_accumulate(origin_addr, origin_count, origin_datatype, result_addr, result_count, result_datatype, target_rank, target_disp, target_count, target_datatype, op, win, request);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Rput ================== */
_EXTERN_C_ int PMPI_Rput(const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_cout, MPI_Datatype target_datatype, MPI_Win win, MPI_Request *request);
_EXTERN_C_ int MPI_Rput(const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_cout, MPI_Datatype target_datatype, MPI_Win win, MPI_Request *request) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Rput) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Rput", strlen("MPI_Rput")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Rput(origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_cout, target_datatype, win, request);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Rput(origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_cout, target_datatype, win, request);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Status_set_cancelled ================== */
_EXTERN_C_ int PMPI_Status_set_cancelled(MPI_Status *status, int flag);
_EXTERN_C_ int MPI_Status_set_cancelled(MPI_Status *status, int flag) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Status_set_cancelled) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Status_set_cancelled", strlen("MPI_Status_set_cancelled")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Status_set_cancelled(status, flag);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Status_set_cancelled(status, flag);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Status_set_elements ================== */
_EXTERN_C_ int PMPI_Status_set_elements(MPI_Status *status, MPI_Datatype datatype, int count);
_EXTERN_C_ int MPI_Status_set_elements(MPI_Status *status, MPI_Datatype datatype, int count) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Status_set_elements) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Status_set_elements", strlen("MPI_Status_set_elements")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Status_set_elements(status, datatype, count);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Status_set_elements(status, datatype, count);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Status_set_elements_x ================== */
_EXTERN_C_ int PMPI_Status_set_elements_x(MPI_Status *status, MPI_Datatype datatype, MPI_Count count);
_EXTERN_C_ int MPI_Status_set_elements_x(MPI_Status *status, MPI_Datatype datatype, MPI_Count count) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Status_set_elements_x) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Status_set_elements_x", strlen("MPI_Status_set_elements_x")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Status_set_elements_x(status, datatype, count);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Status_set_elements_x(status, datatype, count);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Test_cancelled ================== */
_EXTERN_C_ int PMPI_Test_cancelled(const MPI_Status *status, int *flag);
_EXTERN_C_ int MPI_Test_cancelled(const MPI_Status *status, int *flag) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Test_cancelled) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Test_cancelled", strlen("MPI_Test_cancelled")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Test_cancelled(status, flag);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Test_cancelled(status, flag);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Topo_test ================== */
_EXTERN_C_ int PMPI_Topo_test(MPI_Comm comm, int *status);
_EXTERN_C_ int MPI_Topo_test(MPI_Comm comm, int *status) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Topo_test) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Topo_test", strlen("MPI_Topo_test")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Topo_test(comm, status);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Topo_test(comm, status);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Type_commit ================== */
_EXTERN_C_ int PMPI_Type_commit(MPI_Datatype *type);
_EXTERN_C_ int MPI_Type_commit(MPI_Datatype *type) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Type_commit) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Type_commit", strlen("MPI_Type_commit")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Type_commit(type);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Type_commit(type);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Type_contiguous ================== */
_EXTERN_C_ int PMPI_Type_contiguous(int count, MPI_Datatype oldtype, MPI_Datatype *newtype);
_EXTERN_C_ int MPI_Type_contiguous(int count, MPI_Datatype oldtype, MPI_Datatype *newtype) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Type_contiguous) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Type_contiguous", strlen("MPI_Type_contiguous")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Type_contiguous(count, oldtype, newtype);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Type_contiguous(count, oldtype, newtype);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Type_create_darray ================== */
_EXTERN_C_ int PMPI_Type_create_darray(int size, int rank, int ndims, const int gsize_array[], const int distrib_array[], const int darg_array[], const int psize_array[], int order, MPI_Datatype oldtype, MPI_Datatype *newtype);
_EXTERN_C_ int MPI_Type_create_darray(int size, int rank, int ndims, const int gsize_array[], const int distrib_array[], const int darg_array[], const int psize_array[], int order, MPI_Datatype oldtype, MPI_Datatype *newtype) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Type_create_darray) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Type_create_darray", strlen("MPI_Type_create_darray")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Type_create_darray(size, rank, ndims, gsize_array, distrib_array, darg_array, psize_array, order, oldtype, newtype);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Type_create_darray(size, rank, ndims, gsize_array, distrib_array, darg_array, psize_array, order, oldtype, newtype);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Type_create_f90_complex ================== */
_EXTERN_C_ int PMPI_Type_create_f90_complex(int p, int r, MPI_Datatype *newtype);
_EXTERN_C_ int MPI_Type_create_f90_complex(int p, int r, MPI_Datatype *newtype) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Type_create_f90_complex) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Type_create_f90_complex", strlen("MPI_Type_create_f90_complex")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Type_create_f90_complex(p, r, newtype);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Type_create_f90_complex(p, r, newtype);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Type_create_f90_integer ================== */
_EXTERN_C_ int PMPI_Type_create_f90_integer(int r, MPI_Datatype *newtype);
_EXTERN_C_ int MPI_Type_create_f90_integer(int r, MPI_Datatype *newtype) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Type_create_f90_integer) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Type_create_f90_integer", strlen("MPI_Type_create_f90_integer")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Type_create_f90_integer(r, newtype);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Type_create_f90_integer(r, newtype);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Type_create_f90_real ================== */
_EXTERN_C_ int PMPI_Type_create_f90_real(int p, int r, MPI_Datatype *newtype);
_EXTERN_C_ int MPI_Type_create_f90_real(int p, int r, MPI_Datatype *newtype) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Type_create_f90_real) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Type_create_f90_real", strlen("MPI_Type_create_f90_real")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Type_create_f90_real(p, r, newtype);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Type_create_f90_real(p, r, newtype);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Type_create_hindexed ================== */
_EXTERN_C_ int PMPI_Type_create_hindexed(int count, const int array_of_blocklengths[], const MPI_Aint array_of_displacements[], MPI_Datatype oldtype, MPI_Datatype *newtype);
_EXTERN_C_ int MPI_Type_create_hindexed(int count, const int array_of_blocklengths[], const MPI_Aint array_of_displacements[], MPI_Datatype oldtype, MPI_Datatype *newtype) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Type_create_hindexed) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Type_create_hindexed", strlen("MPI_Type_create_hindexed")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Type_create_hindexed(count, array_of_blocklengths, array_of_displacements, oldtype, newtype);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Type_create_hindexed(count, array_of_blocklengths, array_of_displacements, oldtype, newtype);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Type_create_hindexed_block ================== */
_EXTERN_C_ int PMPI_Type_create_hindexed_block(int count, int blocklength, const MPI_Aint array_of_displacements[], MPI_Datatype oldtype, MPI_Datatype *newtype);
_EXTERN_C_ int MPI_Type_create_hindexed_block(int count, int blocklength, const MPI_Aint array_of_displacements[], MPI_Datatype oldtype, MPI_Datatype *newtype) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Type_create_hindexed_block) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Type_create_hindexed_block", strlen("MPI_Type_create_hindexed_block")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Type_create_hindexed_block(count, blocklength, array_of_displacements, oldtype, newtype);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Type_create_hindexed_block(count, blocklength, array_of_displacements, oldtype, newtype);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Type_create_hvector ================== */
_EXTERN_C_ int PMPI_Type_create_hvector(int count, int blocklength, MPI_Aint stride, MPI_Datatype oldtype, MPI_Datatype *newtype);
_EXTERN_C_ int MPI_Type_create_hvector(int count, int blocklength, MPI_Aint stride, MPI_Datatype oldtype, MPI_Datatype *newtype) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Type_create_hvector) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Type_create_hvector", strlen("MPI_Type_create_hvector")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Type_create_hvector(count, blocklength, stride, oldtype, newtype);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Type_create_hvector(count, blocklength, stride, oldtype, newtype);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Type_create_indexed_block ================== */
_EXTERN_C_ int PMPI_Type_create_indexed_block(int count, int blocklength, const int array_of_displacements[], MPI_Datatype oldtype, MPI_Datatype *newtype);
_EXTERN_C_ int MPI_Type_create_indexed_block(int count, int blocklength, const int array_of_displacements[], MPI_Datatype oldtype, MPI_Datatype *newtype) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Type_create_indexed_block) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Type_create_indexed_block", strlen("MPI_Type_create_indexed_block")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Type_create_indexed_block(count, blocklength, array_of_displacements, oldtype, newtype);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Type_create_indexed_block(count, blocklength, array_of_displacements, oldtype, newtype);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Type_create_keyval ================== */
_EXTERN_C_ int PMPI_Type_create_keyval(MPI_Type_copy_attr_function *type_copy_attr_fn, MPI_Type_delete_attr_function *type_delete_attr_fn, int *type_keyval, void *extra_state);
_EXTERN_C_ int MPI_Type_create_keyval(MPI_Type_copy_attr_function *type_copy_attr_fn, MPI_Type_delete_attr_function *type_delete_attr_fn, int *type_keyval, void *extra_state) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Type_create_keyval) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Type_create_keyval", strlen("MPI_Type_create_keyval")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Type_create_keyval(type_copy_attr_fn, type_delete_attr_fn, type_keyval, extra_state);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Type_create_keyval(type_copy_attr_fn, type_delete_attr_fn, type_keyval, extra_state);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Type_create_resized ================== */
_EXTERN_C_ int PMPI_Type_create_resized(MPI_Datatype oldtype, MPI_Aint lb, MPI_Aint extent, MPI_Datatype *newtype);
_EXTERN_C_ int MPI_Type_create_resized(MPI_Datatype oldtype, MPI_Aint lb, MPI_Aint extent, MPI_Datatype *newtype) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Type_create_resized) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Type_create_resized", strlen("MPI_Type_create_resized")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Type_create_resized(oldtype, lb, extent, newtype);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Type_create_resized(oldtype, lb, extent, newtype);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Type_create_struct ================== */
_EXTERN_C_ int PMPI_Type_create_struct(int count, const int array_of_block_lengths[], const MPI_Aint array_of_displacements[], const MPI_Datatype array_of_types[], MPI_Datatype *newtype);
_EXTERN_C_ int MPI_Type_create_struct(int count, const int array_of_block_lengths[], const MPI_Aint array_of_displacements[], const MPI_Datatype array_of_types[], MPI_Datatype *newtype) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Type_create_struct) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Type_create_struct", strlen("MPI_Type_create_struct")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Type_create_struct(count, array_of_block_lengths, array_of_displacements, array_of_types, newtype);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Type_create_struct(count, array_of_block_lengths, array_of_displacements, array_of_types, newtype);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Type_create_subarray ================== */
_EXTERN_C_ int PMPI_Type_create_subarray(int ndims, const int size_array[], const int subsize_array[], const int start_array[], int order, MPI_Datatype oldtype, MPI_Datatype *newtype);
_EXTERN_C_ int MPI_Type_create_subarray(int ndims, const int size_array[], const int subsize_array[], const int start_array[], int order, MPI_Datatype oldtype, MPI_Datatype *newtype) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Type_create_subarray) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Type_create_subarray", strlen("MPI_Type_create_subarray")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Type_create_subarray(ndims, size_array, subsize_array, start_array, order, oldtype, newtype);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Type_create_subarray(ndims, size_array, subsize_array, start_array, order, oldtype, newtype);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Type_delete_attr ================== */
_EXTERN_C_ int PMPI_Type_delete_attr(MPI_Datatype type, int type_keyval);
_EXTERN_C_ int MPI_Type_delete_attr(MPI_Datatype type, int type_keyval) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Type_delete_attr) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Type_delete_attr", strlen("MPI_Type_delete_attr")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Type_delete_attr(type, type_keyval);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Type_delete_attr(type, type_keyval);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Type_dup ================== */
_EXTERN_C_ int PMPI_Type_dup(MPI_Datatype type, MPI_Datatype *newtype);
_EXTERN_C_ int MPI_Type_dup(MPI_Datatype type, MPI_Datatype *newtype) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Type_dup) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Type_dup", strlen("MPI_Type_dup")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Type_dup(type, newtype);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Type_dup(type, newtype);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Type_extent ================== */
_EXTERN_C_ int PMPI_Type_extent(MPI_Datatype type, MPI_Aint *extent);
_EXTERN_C_ int MPI_Type_extent(MPI_Datatype type, MPI_Aint *extent) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Type_extent) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Type_extent", strlen("MPI_Type_extent")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Type_extent(type, extent);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Type_extent(type, extent);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Type_free ================== */
_EXTERN_C_ int PMPI_Type_free(MPI_Datatype *type);
_EXTERN_C_ int MPI_Type_free(MPI_Datatype *type) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Type_free) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Type_free", strlen("MPI_Type_free")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Type_free(type);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Type_free(type);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Type_free_keyval ================== */
_EXTERN_C_ int PMPI_Type_free_keyval(int *type_keyval);
_EXTERN_C_ int MPI_Type_free_keyval(int *type_keyval) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Type_free_keyval) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Type_free_keyval", strlen("MPI_Type_free_keyval")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Type_free_keyval(type_keyval);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Type_free_keyval(type_keyval);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Type_get_attr ================== */
_EXTERN_C_ int PMPI_Type_get_attr(MPI_Datatype type, int type_keyval, void *attribute_val, int *flag);
_EXTERN_C_ int MPI_Type_get_attr(MPI_Datatype type, int type_keyval, void *attribute_val, int *flag) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Type_get_attr) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Type_get_attr", strlen("MPI_Type_get_attr")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Type_get_attr(type, type_keyval, attribute_val, flag);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Type_get_attr(type, type_keyval, attribute_val, flag);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Type_get_contents ================== */
_EXTERN_C_ int PMPI_Type_get_contents(MPI_Datatype mtype, int max_integers, int max_addresses, int max_datatypes, int array_of_integers[], MPI_Aint array_of_addresses[], MPI_Datatype array_of_datatypes[]);
_EXTERN_C_ int MPI_Type_get_contents(MPI_Datatype mtype, int max_integers, int max_addresses, int max_datatypes, int array_of_integers[], MPI_Aint array_of_addresses[], MPI_Datatype array_of_datatypes[]) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Type_get_contents) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Type_get_contents", strlen("MPI_Type_get_contents")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Type_get_contents(mtype, max_integers, max_addresses, max_datatypes, array_of_integers, array_of_addresses, array_of_datatypes);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Type_get_contents(mtype, max_integers, max_addresses, max_datatypes, array_of_integers, array_of_addresses, array_of_datatypes);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Type_get_envelope ================== */
_EXTERN_C_ int PMPI_Type_get_envelope(MPI_Datatype type, int *num_integers, int *num_addresses, int *num_datatypes, int *combiner);
_EXTERN_C_ int MPI_Type_get_envelope(MPI_Datatype type, int *num_integers, int *num_addresses, int *num_datatypes, int *combiner) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Type_get_envelope) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Type_get_envelope", strlen("MPI_Type_get_envelope")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Type_get_envelope(type, num_integers, num_addresses, num_datatypes, combiner);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Type_get_envelope(type, num_integers, num_addresses, num_datatypes, combiner);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Type_get_extent ================== */
_EXTERN_C_ int PMPI_Type_get_extent(MPI_Datatype type, MPI_Aint *lb, MPI_Aint *extent);
_EXTERN_C_ int MPI_Type_get_extent(MPI_Datatype type, MPI_Aint *lb, MPI_Aint *extent) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Type_get_extent) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Type_get_extent", strlen("MPI_Type_get_extent")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Type_get_extent(type, lb, extent);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Type_get_extent(type, lb, extent);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Type_get_extent_x ================== */
_EXTERN_C_ int PMPI_Type_get_extent_x(MPI_Datatype type, MPI_Count *lb, MPI_Count *extent);
_EXTERN_C_ int MPI_Type_get_extent_x(MPI_Datatype type, MPI_Count *lb, MPI_Count *extent) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Type_get_extent_x) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Type_get_extent_x", strlen("MPI_Type_get_extent_x")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Type_get_extent_x(type, lb, extent);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Type_get_extent_x(type, lb, extent);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Type_get_name ================== */
_EXTERN_C_ int PMPI_Type_get_name(MPI_Datatype type, char *type_name, int *resultlen);
_EXTERN_C_ int MPI_Type_get_name(MPI_Datatype type, char *type_name, int *resultlen) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Type_get_name) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Type_get_name", strlen("MPI_Type_get_name")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Type_get_name(type, type_name, resultlen);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Type_get_name(type, type_name, resultlen);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Type_get_true_extent ================== */
_EXTERN_C_ int PMPI_Type_get_true_extent(MPI_Datatype datatype, MPI_Aint *true_lb, MPI_Aint *true_extent);
_EXTERN_C_ int MPI_Type_get_true_extent(MPI_Datatype datatype, MPI_Aint *true_lb, MPI_Aint *true_extent) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Type_get_true_extent) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Type_get_true_extent", strlen("MPI_Type_get_true_extent")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Type_get_true_extent(datatype, true_lb, true_extent);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Type_get_true_extent(datatype, true_lb, true_extent);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Type_get_true_extent_x ================== */
_EXTERN_C_ int PMPI_Type_get_true_extent_x(MPI_Datatype datatype, MPI_Count *true_lb, MPI_Count *true_extent);
_EXTERN_C_ int MPI_Type_get_true_extent_x(MPI_Datatype datatype, MPI_Count *true_lb, MPI_Count *true_extent) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Type_get_true_extent_x) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Type_get_true_extent_x", strlen("MPI_Type_get_true_extent_x")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Type_get_true_extent_x(datatype, true_lb, true_extent);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Type_get_true_extent_x(datatype, true_lb, true_extent);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Type_hindexed ================== */
_EXTERN_C_ int PMPI_Type_hindexed(int count, int array_of_blocklengths[], MPI_Aint array_of_displacements[], MPI_Datatype oldtype, MPI_Datatype *newtype);
_EXTERN_C_ int MPI_Type_hindexed(int count, int array_of_blocklengths[], MPI_Aint array_of_displacements[], MPI_Datatype oldtype, MPI_Datatype *newtype) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Type_hindexed) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Type_hindexed", strlen("MPI_Type_hindexed")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Type_hindexed(count, array_of_blocklengths, array_of_displacements, oldtype, newtype);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Type_hindexed(count, array_of_blocklengths, array_of_displacements, oldtype, newtype);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Type_hvector ================== */
_EXTERN_C_ int PMPI_Type_hvector(int count, int blocklength, MPI_Aint stride, MPI_Datatype oldtype, MPI_Datatype *newtype);
_EXTERN_C_ int MPI_Type_hvector(int count, int blocklength, MPI_Aint stride, MPI_Datatype oldtype, MPI_Datatype *newtype) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Type_hvector) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Type_hvector", strlen("MPI_Type_hvector")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Type_hvector(count, blocklength, stride, oldtype, newtype);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Type_hvector(count, blocklength, stride, oldtype, newtype);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Type_indexed ================== */
_EXTERN_C_ int PMPI_Type_indexed(int count, const int array_of_blocklengths[], const int array_of_displacements[], MPI_Datatype oldtype, MPI_Datatype *newtype);
_EXTERN_C_ int MPI_Type_indexed(int count, const int array_of_blocklengths[], const int array_of_displacements[], MPI_Datatype oldtype, MPI_Datatype *newtype) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Type_indexed) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Type_indexed", strlen("MPI_Type_indexed")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Type_indexed(count, array_of_blocklengths, array_of_displacements, oldtype, newtype);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Type_indexed(count, array_of_blocklengths, array_of_displacements, oldtype, newtype);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Type_lb ================== */
_EXTERN_C_ int PMPI_Type_lb(MPI_Datatype type, MPI_Aint *lb);
_EXTERN_C_ int MPI_Type_lb(MPI_Datatype type, MPI_Aint *lb) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Type_lb) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Type_lb", strlen("MPI_Type_lb")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Type_lb(type, lb);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Type_lb(type, lb);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Type_match_size ================== */
_EXTERN_C_ int PMPI_Type_match_size(int typeclass, int size, MPI_Datatype *type);
_EXTERN_C_ int MPI_Type_match_size(int typeclass, int size, MPI_Datatype *type) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Type_match_size) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Type_match_size", strlen("MPI_Type_match_size")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Type_match_size(typeclass, size, type);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Type_match_size(typeclass, size, type);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Type_set_attr ================== */
_EXTERN_C_ int PMPI_Type_set_attr(MPI_Datatype type, int type_keyval, void *attr_val);
_EXTERN_C_ int MPI_Type_set_attr(MPI_Datatype type, int type_keyval, void *attr_val) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Type_set_attr) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Type_set_attr", strlen("MPI_Type_set_attr")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Type_set_attr(type, type_keyval, attr_val);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Type_set_attr(type, type_keyval, attr_val);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Type_set_name ================== */
_EXTERN_C_ int PMPI_Type_set_name(MPI_Datatype type, const char *type_name);
_EXTERN_C_ int MPI_Type_set_name(MPI_Datatype type, const char *type_name) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Type_set_name) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Type_set_name", strlen("MPI_Type_set_name")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Type_set_name(type, type_name);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Type_set_name(type, type_name);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Type_size ================== */
_EXTERN_C_ int PMPI_Type_size(MPI_Datatype type, int *size);
_EXTERN_C_ int MPI_Type_size(MPI_Datatype type, int *size) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Type_size) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Type_size", strlen("MPI_Type_size")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Type_size(type, size);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Type_size(type, size);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Type_size_x ================== */
_EXTERN_C_ int PMPI_Type_size_x(MPI_Datatype type, MPI_Count *size);
_EXTERN_C_ int MPI_Type_size_x(MPI_Datatype type, MPI_Count *size) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Type_size_x) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Type_size_x", strlen("MPI_Type_size_x")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Type_size_x(type, size);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Type_size_x(type, size);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Type_struct ================== */
_EXTERN_C_ int PMPI_Type_struct(int count, int array_of_blocklengths[], MPI_Aint array_of_displacements[], MPI_Datatype array_of_types[], MPI_Datatype *newtype);
_EXTERN_C_ int MPI_Type_struct(int count, int array_of_blocklengths[], MPI_Aint array_of_displacements[], MPI_Datatype array_of_types[], MPI_Datatype *newtype) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Type_struct) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Type_struct", strlen("MPI_Type_struct")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Type_struct(count, array_of_blocklengths, array_of_displacements, array_of_types, newtype);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Type_struct(count, array_of_blocklengths, array_of_displacements, array_of_types, newtype);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Type_ub ================== */
_EXTERN_C_ int PMPI_Type_ub(MPI_Datatype mtype, MPI_Aint *ub);
_EXTERN_C_ int MPI_Type_ub(MPI_Datatype mtype, MPI_Aint *ub) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Type_ub) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Type_ub", strlen("MPI_Type_ub")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Type_ub(mtype, ub);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Type_ub(mtype, ub);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Type_vector ================== */
_EXTERN_C_ int PMPI_Type_vector(int count, int blocklength, int stride, MPI_Datatype oldtype, MPI_Datatype *newtype);
_EXTERN_C_ int MPI_Type_vector(int count, int blocklength, int stride, MPI_Datatype oldtype, MPI_Datatype *newtype) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Type_vector) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Type_vector", strlen("MPI_Type_vector")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Type_vector(count, blocklength, stride, oldtype, newtype);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Type_vector(count, blocklength, stride, oldtype, newtype);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Unpack ================== */
_EXTERN_C_ int PMPI_Unpack(const void *inbuf, int insize, int *position, void *outbuf, int outcount, MPI_Datatype datatype, MPI_Comm comm);
_EXTERN_C_ int MPI_Unpack(const void *inbuf, int insize, int *position, void *outbuf, int outcount, MPI_Datatype datatype, MPI_Comm comm) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Unpack) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Unpack", strlen("MPI_Unpack")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Unpack(inbuf, insize, position, outbuf, outcount, datatype, comm);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Unpack(inbuf, insize, position, outbuf, outcount, datatype, comm);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Unpack_external ================== */
_EXTERN_C_ int PMPI_Unpack_external(const char datarep[], const void *inbuf, MPI_Aint insize, MPI_Aint *position, void *outbuf, int outcount, MPI_Datatype datatype);
_EXTERN_C_ int MPI_Unpack_external(const char datarep[], const void *inbuf, MPI_Aint insize, MPI_Aint *position, void *outbuf, int outcount, MPI_Datatype datatype) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Unpack_external) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Unpack_external", strlen("MPI_Unpack_external")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Unpack_external(datarep, inbuf, insize, position, outbuf, outcount, datatype);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Unpack_external(datarep, inbuf, insize, position, outbuf, outcount, datatype);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Unpublish_name ================== */
_EXTERN_C_ int PMPI_Unpublish_name(const char *service_name, MPI_Info info, const char *port_name);
_EXTERN_C_ int MPI_Unpublish_name(const char *service_name, MPI_Info info, const char *port_name) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Unpublish_name) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Unpublish_name", strlen("MPI_Unpublish_name")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Unpublish_name(service_name, info, port_name);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Unpublish_name(service_name, info, port_name);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Win_allocate ================== */
_EXTERN_C_ int PMPI_Win_allocate(MPI_Aint size, int disp_unit, MPI_Info info, MPI_Comm comm, void *baseptr, MPI_Win *win);
_EXTERN_C_ int MPI_Win_allocate(MPI_Aint size, int disp_unit, MPI_Info info, MPI_Comm comm, void *baseptr, MPI_Win *win) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Win_allocate) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Win_allocate", strlen("MPI_Win_allocate")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Win_allocate(size, disp_unit, info, comm, baseptr, win);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Win_allocate(size, disp_unit, info, comm, baseptr, win);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Win_allocate_shared ================== */
_EXTERN_C_ int PMPI_Win_allocate_shared(MPI_Aint size, int disp_unit, MPI_Info info, MPI_Comm comm, void *baseptr, MPI_Win *win);
_EXTERN_C_ int MPI_Win_allocate_shared(MPI_Aint size, int disp_unit, MPI_Info info, MPI_Comm comm, void *baseptr, MPI_Win *win) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Win_allocate_shared) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Win_allocate_shared", strlen("MPI_Win_allocate_shared")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Win_allocate_shared(size, disp_unit, info, comm, baseptr, win);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Win_allocate_shared(size, disp_unit, info, comm, baseptr, win);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Win_attach ================== */
_EXTERN_C_ int PMPI_Win_attach(MPI_Win win, void *base, MPI_Aint size);
_EXTERN_C_ int MPI_Win_attach(MPI_Win win, void *base, MPI_Aint size) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Win_attach) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Win_attach", strlen("MPI_Win_attach")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Win_attach(win, base, size);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Win_attach(win, base, size);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Win_call_errhandler ================== */
_EXTERN_C_ int PMPI_Win_call_errhandler(MPI_Win win, int errorcode);
_EXTERN_C_ int MPI_Win_call_errhandler(MPI_Win win, int errorcode) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Win_call_errhandler) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Win_call_errhandler", strlen("MPI_Win_call_errhandler")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Win_call_errhandler(win, errorcode);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Win_call_errhandler(win, errorcode);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Win_complete ================== */
_EXTERN_C_ int PMPI_Win_complete(MPI_Win win);
_EXTERN_C_ int MPI_Win_complete(MPI_Win win) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Win_complete) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Win_complete", strlen("MPI_Win_complete")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Win_complete(win);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Win_complete(win);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Win_create ================== */
_EXTERN_C_ int PMPI_Win_create(void *base, MPI_Aint size, int disp_unit, MPI_Info info, MPI_Comm comm, MPI_Win *win);
_EXTERN_C_ int MPI_Win_create(void *base, MPI_Aint size, int disp_unit, MPI_Info info, MPI_Comm comm, MPI_Win *win) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Win_create) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Win_create", strlen("MPI_Win_create")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Win_create(base, size, disp_unit, info, comm, win);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Win_create(base, size, disp_unit, info, comm, win);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Win_create_dynamic ================== */
_EXTERN_C_ int PMPI_Win_create_dynamic(MPI_Info info, MPI_Comm comm, MPI_Win *win);
_EXTERN_C_ int MPI_Win_create_dynamic(MPI_Info info, MPI_Comm comm, MPI_Win *win) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Win_create_dynamic) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Win_create_dynamic", strlen("MPI_Win_create_dynamic")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Win_create_dynamic(info, comm, win);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Win_create_dynamic(info, comm, win);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Win_create_errhandler ================== */
_EXTERN_C_ int PMPI_Win_create_errhandler(MPI_Win_errhandler_function *function, MPI_Errhandler *errhandler);
_EXTERN_C_ int MPI_Win_create_errhandler(MPI_Win_errhandler_function *function, MPI_Errhandler *errhandler) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Win_create_errhandler) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Win_create_errhandler", strlen("MPI_Win_create_errhandler")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Win_create_errhandler(function, errhandler);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Win_create_errhandler(function, errhandler);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Win_create_keyval ================== */
_EXTERN_C_ int PMPI_Win_create_keyval(MPI_Win_copy_attr_function *win_copy_attr_fn, MPI_Win_delete_attr_function *win_delete_attr_fn, int *win_keyval, void *extra_state);
_EXTERN_C_ int MPI_Win_create_keyval(MPI_Win_copy_attr_function *win_copy_attr_fn, MPI_Win_delete_attr_function *win_delete_attr_fn, int *win_keyval, void *extra_state) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Win_create_keyval) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Win_create_keyval", strlen("MPI_Win_create_keyval")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Win_create_keyval(win_copy_attr_fn, win_delete_attr_fn, win_keyval, extra_state);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Win_create_keyval(win_copy_attr_fn, win_delete_attr_fn, win_keyval, extra_state);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Win_delete_attr ================== */
_EXTERN_C_ int PMPI_Win_delete_attr(MPI_Win win, int win_keyval);
_EXTERN_C_ int MPI_Win_delete_attr(MPI_Win win, int win_keyval) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Win_delete_attr) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Win_delete_attr", strlen("MPI_Win_delete_attr")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Win_delete_attr(win, win_keyval);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Win_delete_attr(win, win_keyval);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Win_detach ================== */
_EXTERN_C_ int PMPI_Win_detach(MPI_Win win, const void *base);
_EXTERN_C_ int MPI_Win_detach(MPI_Win win, const void *base) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Win_detach) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Win_detach", strlen("MPI_Win_detach")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Win_detach(win, base);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Win_detach(win, base);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Win_fence ================== */
_EXTERN_C_ int PMPI_Win_fence(int assert, MPI_Win win);
_EXTERN_C_ int MPI_Win_fence(int assert, MPI_Win win) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Win_fence) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Win_fence", strlen("MPI_Win_fence")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Win_fence(assert, win);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Win_fence(assert, win);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Win_flush ================== */
_EXTERN_C_ int PMPI_Win_flush(int rank, MPI_Win win);
_EXTERN_C_ int MPI_Win_flush(int rank, MPI_Win win) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Win_flush) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Win_flush", strlen("MPI_Win_flush")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Win_flush(rank, win);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Win_flush(rank, win);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Win_flush_all ================== */
_EXTERN_C_ int PMPI_Win_flush_all(MPI_Win win);
_EXTERN_C_ int MPI_Win_flush_all(MPI_Win win) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Win_flush_all) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Win_flush_all", strlen("MPI_Win_flush_all")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Win_flush_all(win);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Win_flush_all(win);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Win_flush_local ================== */
_EXTERN_C_ int PMPI_Win_flush_local(int rank, MPI_Win win);
_EXTERN_C_ int MPI_Win_flush_local(int rank, MPI_Win win) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Win_flush_local) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Win_flush_local", strlen("MPI_Win_flush_local")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Win_flush_local(rank, win);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Win_flush_local(rank, win);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Win_flush_local_all ================== */
_EXTERN_C_ int PMPI_Win_flush_local_all(MPI_Win win);
_EXTERN_C_ int MPI_Win_flush_local_all(MPI_Win win) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Win_flush_local_all) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Win_flush_local_all", strlen("MPI_Win_flush_local_all")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Win_flush_local_all(win);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Win_flush_local_all(win);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Win_free ================== */
_EXTERN_C_ int PMPI_Win_free(MPI_Win *win);
_EXTERN_C_ int MPI_Win_free(MPI_Win *win) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Win_free) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Win_free", strlen("MPI_Win_free")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Win_free(win);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Win_free(win);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Win_free_keyval ================== */
_EXTERN_C_ int PMPI_Win_free_keyval(int *win_keyval);
_EXTERN_C_ int MPI_Win_free_keyval(int *win_keyval) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Win_free_keyval) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Win_free_keyval", strlen("MPI_Win_free_keyval")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Win_free_keyval(win_keyval);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Win_free_keyval(win_keyval);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Win_get_attr ================== */
_EXTERN_C_ int PMPI_Win_get_attr(MPI_Win win, int win_keyval, void *attribute_val, int *flag);
_EXTERN_C_ int MPI_Win_get_attr(MPI_Win win, int win_keyval, void *attribute_val, int *flag) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Win_get_attr) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Win_get_attr", strlen("MPI_Win_get_attr")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Win_get_attr(win, win_keyval, attribute_val, flag);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Win_get_attr(win, win_keyval, attribute_val, flag);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Win_get_errhandler ================== */
_EXTERN_C_ int PMPI_Win_get_errhandler(MPI_Win win, MPI_Errhandler *errhandler);
_EXTERN_C_ int MPI_Win_get_errhandler(MPI_Win win, MPI_Errhandler *errhandler) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Win_get_errhandler) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Win_get_errhandler", strlen("MPI_Win_get_errhandler")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Win_get_errhandler(win, errhandler);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Win_get_errhandler(win, errhandler);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Win_get_group ================== */
_EXTERN_C_ int PMPI_Win_get_group(MPI_Win win, MPI_Group *group);
_EXTERN_C_ int MPI_Win_get_group(MPI_Win win, MPI_Group *group) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Win_get_group) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Win_get_group", strlen("MPI_Win_get_group")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Win_get_group(win, group);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Win_get_group(win, group);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Win_get_info ================== */
_EXTERN_C_ int PMPI_Win_get_info(MPI_Win win, MPI_Info *info_used);
_EXTERN_C_ int MPI_Win_get_info(MPI_Win win, MPI_Info *info_used) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Win_get_info) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Win_get_info", strlen("MPI_Win_get_info")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Win_get_info(win, info_used);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Win_get_info(win, info_used);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Win_get_name ================== */
_EXTERN_C_ int PMPI_Win_get_name(MPI_Win win, char *win_name, int *resultlen);
_EXTERN_C_ int MPI_Win_get_name(MPI_Win win, char *win_name, int *resultlen) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Win_get_name) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Win_get_name", strlen("MPI_Win_get_name")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Win_get_name(win, win_name, resultlen);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Win_get_name(win, win_name, resultlen);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Win_lock ================== */
_EXTERN_C_ int PMPI_Win_lock(int lock_type, int rank, int assert, MPI_Win win);
_EXTERN_C_ int MPI_Win_lock(int lock_type, int rank, int assert, MPI_Win win) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Win_lock) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Win_lock", strlen("MPI_Win_lock")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Win_lock(lock_type, rank, assert, win);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Win_lock(lock_type, rank, assert, win);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Win_lock_all ================== */
_EXTERN_C_ int PMPI_Win_lock_all(int assert, MPI_Win win);
_EXTERN_C_ int MPI_Win_lock_all(int assert, MPI_Win win) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Win_lock_all) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Win_lock_all", strlen("MPI_Win_lock_all")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Win_lock_all(assert, win);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Win_lock_all(assert, win);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Win_post ================== */
_EXTERN_C_ int PMPI_Win_post(MPI_Group group, int assert, MPI_Win win);
_EXTERN_C_ int MPI_Win_post(MPI_Group group, int assert, MPI_Win win) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Win_post) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Win_post", strlen("MPI_Win_post")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Win_post(group, assert, win);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Win_post(group, assert, win);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Win_set_attr ================== */
_EXTERN_C_ int PMPI_Win_set_attr(MPI_Win win, int win_keyval, void *attribute_val);
_EXTERN_C_ int MPI_Win_set_attr(MPI_Win win, int win_keyval, void *attribute_val) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Win_set_attr) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Win_set_attr", strlen("MPI_Win_set_attr")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Win_set_attr(win, win_keyval, attribute_val);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Win_set_attr(win, win_keyval, attribute_val);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Win_set_errhandler ================== */
_EXTERN_C_ int PMPI_Win_set_errhandler(MPI_Win win, MPI_Errhandler errhandler);
_EXTERN_C_ int MPI_Win_set_errhandler(MPI_Win win, MPI_Errhandler errhandler) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Win_set_errhandler) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Win_set_errhandler", strlen("MPI_Win_set_errhandler")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Win_set_errhandler(win, errhandler);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Win_set_errhandler(win, errhandler);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Win_set_info ================== */
_EXTERN_C_ int PMPI_Win_set_info(MPI_Win win, MPI_Info info);
_EXTERN_C_ int MPI_Win_set_info(MPI_Win win, MPI_Info info) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Win_set_info) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Win_set_info", strlen("MPI_Win_set_info")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Win_set_info(win, info);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Win_set_info(win, info);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Win_set_name ================== */
_EXTERN_C_ int PMPI_Win_set_name(MPI_Win win, const char *win_name);
_EXTERN_C_ int MPI_Win_set_name(MPI_Win win, const char *win_name) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Win_set_name) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Win_set_name", strlen("MPI_Win_set_name")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Win_set_name(win, win_name);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Win_set_name(win, win_name);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Win_shared_query ================== */
_EXTERN_C_ int PMPI_Win_shared_query(MPI_Win win, int rank, MPI_Aint *size, int *disp_unit, void *baseptr);
_EXTERN_C_ int MPI_Win_shared_query(MPI_Win win, int rank, MPI_Aint *size, int *disp_unit, void *baseptr) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Win_shared_query) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Win_shared_query", strlen("MPI_Win_shared_query")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Win_shared_query(win, rank, size, disp_unit, baseptr);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Win_shared_query(win, rank, size, disp_unit, baseptr);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Win_start ================== */
_EXTERN_C_ int PMPI_Win_start(MPI_Group group, int assert, MPI_Win win);
_EXTERN_C_ int MPI_Win_start(MPI_Group group, int assert, MPI_Win win) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Win_start) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Win_start", strlen("MPI_Win_start")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Win_start(group, assert, win);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Win_start(group, assert, win);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Win_sync ================== */
_EXTERN_C_ int PMPI_Win_sync(MPI_Win win);
_EXTERN_C_ int MPI_Win_sync(MPI_Win win) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Win_sync) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Win_sync", strlen("MPI_Win_sync")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Win_sync(win);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Win_sync(win);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Win_test ================== */
_EXTERN_C_ int PMPI_Win_test(MPI_Win win, int *flag);
_EXTERN_C_ int MPI_Win_test(MPI_Win win, int *flag) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Win_test) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Win_test", strlen("MPI_Win_test")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Win_test(win, flag);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Win_test(win, flag);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Win_unlock ================== */
_EXTERN_C_ int PMPI_Win_unlock(int rank, MPI_Win win);
_EXTERN_C_ int MPI_Win_unlock(int rank, MPI_Win win) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Win_unlock) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Win_unlock", strlen("MPI_Win_unlock")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Win_unlock(rank, win);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Win_unlock(rank, win);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Win_unlock_all ================== */
_EXTERN_C_ int PMPI_Win_unlock_all(MPI_Win win);
_EXTERN_C_ int MPI_Win_unlock_all(MPI_Win win) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Win_unlock_all) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Win_unlock_all", strlen("MPI_Win_unlock_all")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Win_unlock_all(win);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Win_unlock_all(win);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Win_wait ================== */
_EXTERN_C_ int PMPI_Win_wait(MPI_Win win);
_EXTERN_C_ int MPI_Win_wait(MPI_Win win) { 
    int _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Win_wait) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Win_wait", strlen("MPI_Win_wait")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Win_wait(win);
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Win_wait(win);
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Wtick ================== */
_EXTERN_C_ double PMPI_Wtick();
_EXTERN_C_ double MPI_Wtick() { 
    double _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Wtick) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Wtick", strlen("MPI_Wtick")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Wtick();
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Wtick();
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Wtime ================== */
_EXTERN_C_ double PMPI_Wtime();
_EXTERN_C_ double MPI_Wtime() { 
    double _wrap_py_return_val = 0;
{
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    if (::enable_wrapper && ::enable_MPI_Wtime) {
#endif
        Caliper c;
        c.begin(mpifn_attr, Variant(CALI_TYPE_STRING, "MPI_Wtime", strlen("MPI_Wtime")));
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Wtime();
WRAP_MPI_CALL_POSTFIX
        c.end(mpifn_attr);
#ifndef CALIPER_MPIWRAP_USE_GOTCHA
    } else {
        WRAP_MPI_CALL_PREFIX
_wrap_py_return_val = PMPI_Wtime();
WRAP_MPI_CALL_POSTFIX
    }
#endif
}    return _wrap_py_return_val;
}




//
// --- Wrapper initialization
//

namespace cali
{

//
// --- Init function
//

void mpiwrap_init(Caliper* c, const std::string& whitelist, const std::string& blacklist)
{
    // --- register callbacks

    c->events().post_init_evt.connect(::post_init_cb);
    MpiEvents::events.mpi_init_evt.connect(::mpi_init_cb);

    // --- setup wrappers

    ::enable_wrapper = true;

    if (enable_msg_tracing)
        ::tracing.init(c);

    setup_filter(whitelist, blacklist);

#ifdef CALIPER_MPIWRAP_USE_GOTCHA
    Log(2).stream() << "mpiwrap: Using GOTCHA wrappers." << std::endl;

    std::vector<struct gotcha_binding_t> bindings;

    // we always wrap init & finalize
    bindings.push_back(wrap_MPI_Init_binding);
    bindings.push_back(wrap_MPI_Init_thread_binding);
    bindings.push_back(wrap_MPI_Finalize_binding);

    
    if (::enable_MPI_Abort)
        bindings.push_back(wrap_MPI_Abort_binding);
    
    if (::enable_MPI_Accumulate)
        bindings.push_back(wrap_MPI_Accumulate_binding);
    
    if (::enable_MPI_Add_error_class)
        bindings.push_back(wrap_MPI_Add_error_class_binding);
    
    if (::enable_MPI_Add_error_code)
        bindings.push_back(wrap_MPI_Add_error_code_binding);
    
    if (::enable_MPI_Add_error_string)
        bindings.push_back(wrap_MPI_Add_error_string_binding);
    
    if (::enable_MPI_Address)
        bindings.push_back(wrap_MPI_Address_binding);
    
    if (::enable_MPI_Allgather)
        bindings.push_back(wrap_MPI_Allgather_binding);
    
    if (::enable_MPI_Allgatherv)
        bindings.push_back(wrap_MPI_Allgatherv_binding);
    
    if (::enable_MPI_Alloc_mem)
        bindings.push_back(wrap_MPI_Alloc_mem_binding);
    
    if (::enable_MPI_Allreduce)
        bindings.push_back(wrap_MPI_Allreduce_binding);
    
    if (::enable_MPI_Alltoall)
        bindings.push_back(wrap_MPI_Alltoall_binding);
    
    if (::enable_MPI_Alltoallv)
        bindings.push_back(wrap_MPI_Alltoallv_binding);
    
    if (::enable_MPI_Alltoallw)
        bindings.push_back(wrap_MPI_Alltoallw_binding);
    
    if (::enable_MPI_Attr_delete)
        bindings.push_back(wrap_MPI_Attr_delete_binding);
    
    if (::enable_MPI_Attr_get)
        bindings.push_back(wrap_MPI_Attr_get_binding);
    
    if (::enable_MPI_Attr_put)
        bindings.push_back(wrap_MPI_Attr_put_binding);
    
    if (::enable_MPI_Barrier)
        bindings.push_back(wrap_MPI_Barrier_binding);
    
    if (::enable_MPI_Bcast)
        bindings.push_back(wrap_MPI_Bcast_binding);
    
    if (::enable_MPI_Bsend)
        bindings.push_back(wrap_MPI_Bsend_binding);
    
    if (::enable_MPI_Bsend_init)
        bindings.push_back(wrap_MPI_Bsend_init_binding);
    
    if (::enable_MPI_Buffer_attach)
        bindings.push_back(wrap_MPI_Buffer_attach_binding);
    
    if (::enable_MPI_Buffer_detach)
        bindings.push_back(wrap_MPI_Buffer_detach_binding);
    
    if (::enable_MPI_Cancel)
        bindings.push_back(wrap_MPI_Cancel_binding);
    
    if (::enable_MPI_Cart_coords)
        bindings.push_back(wrap_MPI_Cart_coords_binding);
    
    if (::enable_MPI_Cart_create)
        bindings.push_back(wrap_MPI_Cart_create_binding);
    
    if (::enable_MPI_Cart_get)
        bindings.push_back(wrap_MPI_Cart_get_binding);
    
    if (::enable_MPI_Cart_map)
        bindings.push_back(wrap_MPI_Cart_map_binding);
    
    if (::enable_MPI_Cart_rank)
        bindings.push_back(wrap_MPI_Cart_rank_binding);
    
    if (::enable_MPI_Cart_shift)
        bindings.push_back(wrap_MPI_Cart_shift_binding);
    
    if (::enable_MPI_Cart_sub)
        bindings.push_back(wrap_MPI_Cart_sub_binding);
    
    if (::enable_MPI_Cartdim_get)
        bindings.push_back(wrap_MPI_Cartdim_get_binding);
    
    if (::enable_MPI_Close_port)
        bindings.push_back(wrap_MPI_Close_port_binding);
    
    if (::enable_MPI_Comm_accept)
        bindings.push_back(wrap_MPI_Comm_accept_binding);
    
    if (::enable_MPI_Comm_call_errhandler)
        bindings.push_back(wrap_MPI_Comm_call_errhandler_binding);
    
    if (::enable_MPI_Comm_compare)
        bindings.push_back(wrap_MPI_Comm_compare_binding);
    
    if (::enable_MPI_Comm_connect)
        bindings.push_back(wrap_MPI_Comm_connect_binding);
    
    if (::enable_MPI_Comm_create)
        bindings.push_back(wrap_MPI_Comm_create_binding);
    
    if (::enable_MPI_Comm_create_errhandler)
        bindings.push_back(wrap_MPI_Comm_create_errhandler_binding);
    
    if (::enable_MPI_Comm_create_group)
        bindings.push_back(wrap_MPI_Comm_create_group_binding);
    
    if (::enable_MPI_Comm_create_keyval)
        bindings.push_back(wrap_MPI_Comm_create_keyval_binding);
    
    if (::enable_MPI_Comm_delete_attr)
        bindings.push_back(wrap_MPI_Comm_delete_attr_binding);
    
    if (::enable_MPI_Comm_disconnect)
        bindings.push_back(wrap_MPI_Comm_disconnect_binding);
    
    if (::enable_MPI_Comm_dup)
        bindings.push_back(wrap_MPI_Comm_dup_binding);
    
    if (::enable_MPI_Comm_dup_with_info)
        bindings.push_back(wrap_MPI_Comm_dup_with_info_binding);
    
    if (::enable_MPI_Comm_free)
        bindings.push_back(wrap_MPI_Comm_free_binding);
    
    if (::enable_MPI_Comm_free_keyval)
        bindings.push_back(wrap_MPI_Comm_free_keyval_binding);
    
    if (::enable_MPI_Comm_get_attr)
        bindings.push_back(wrap_MPI_Comm_get_attr_binding);
    
    if (::enable_MPI_Comm_get_errhandler)
        bindings.push_back(wrap_MPI_Comm_get_errhandler_binding);
    
    if (::enable_MPI_Comm_get_info)
        bindings.push_back(wrap_MPI_Comm_get_info_binding);
    
    if (::enable_MPI_Comm_get_name)
        bindings.push_back(wrap_MPI_Comm_get_name_binding);
    
    if (::enable_MPI_Comm_get_parent)
        bindings.push_back(wrap_MPI_Comm_get_parent_binding);
    
    if (::enable_MPI_Comm_group)
        bindings.push_back(wrap_MPI_Comm_group_binding);
    
    if (::enable_MPI_Comm_idup)
        bindings.push_back(wrap_MPI_Comm_idup_binding);
    
    if (::enable_MPI_Comm_join)
        bindings.push_back(wrap_MPI_Comm_join_binding);
    
    if (::enable_MPI_Comm_rank)
        bindings.push_back(wrap_MPI_Comm_rank_binding);
    
    if (::enable_MPI_Comm_remote_group)
        bindings.push_back(wrap_MPI_Comm_remote_group_binding);
    
    if (::enable_MPI_Comm_remote_size)
        bindings.push_back(wrap_MPI_Comm_remote_size_binding);
    
    if (::enable_MPI_Comm_set_attr)
        bindings.push_back(wrap_MPI_Comm_set_attr_binding);
    
    if (::enable_MPI_Comm_set_errhandler)
        bindings.push_back(wrap_MPI_Comm_set_errhandler_binding);
    
    if (::enable_MPI_Comm_set_info)
        bindings.push_back(wrap_MPI_Comm_set_info_binding);
    
    if (::enable_MPI_Comm_set_name)
        bindings.push_back(wrap_MPI_Comm_set_name_binding);
    
    if (::enable_MPI_Comm_size)
        bindings.push_back(wrap_MPI_Comm_size_binding);
    
    if (::enable_MPI_Comm_split)
        bindings.push_back(wrap_MPI_Comm_split_binding);
    
    if (::enable_MPI_Comm_split_type)
        bindings.push_back(wrap_MPI_Comm_split_type_binding);
    
    if (::enable_MPI_Comm_test_inter)
        bindings.push_back(wrap_MPI_Comm_test_inter_binding);
    
    if (::enable_MPI_Compare_and_swap)
        bindings.push_back(wrap_MPI_Compare_and_swap_binding);
    
    if (::enable_MPI_Dims_create)
        bindings.push_back(wrap_MPI_Dims_create_binding);
    
    if (::enable_MPI_Dist_graph_create)
        bindings.push_back(wrap_MPI_Dist_graph_create_binding);
    
    if (::enable_MPI_Dist_graph_create_adjacent)
        bindings.push_back(wrap_MPI_Dist_graph_create_adjacent_binding);
    
    if (::enable_MPI_Dist_graph_neighbors)
        bindings.push_back(wrap_MPI_Dist_graph_neighbors_binding);
    
    if (::enable_MPI_Dist_graph_neighbors_count)
        bindings.push_back(wrap_MPI_Dist_graph_neighbors_count_binding);
    
    if (::enable_MPI_Errhandler_create)
        bindings.push_back(wrap_MPI_Errhandler_create_binding);
    
    if (::enable_MPI_Errhandler_free)
        bindings.push_back(wrap_MPI_Errhandler_free_binding);
    
    if (::enable_MPI_Errhandler_get)
        bindings.push_back(wrap_MPI_Errhandler_get_binding);
    
    if (::enable_MPI_Errhandler_set)
        bindings.push_back(wrap_MPI_Errhandler_set_binding);
    
    if (::enable_MPI_Error_class)
        bindings.push_back(wrap_MPI_Error_class_binding);
    
    if (::enable_MPI_Error_string)
        bindings.push_back(wrap_MPI_Error_string_binding);
    
    if (::enable_MPI_Exscan)
        bindings.push_back(wrap_MPI_Exscan_binding);
    
    if (::enable_MPI_Fetch_and_op)
        bindings.push_back(wrap_MPI_Fetch_and_op_binding);
    
    if (::enable_MPI_File_call_errhandler)
        bindings.push_back(wrap_MPI_File_call_errhandler_binding);
    
    if (::enable_MPI_File_close)
        bindings.push_back(wrap_MPI_File_close_binding);
    
    if (::enable_MPI_File_create_errhandler)
        bindings.push_back(wrap_MPI_File_create_errhandler_binding);
    
    if (::enable_MPI_File_delete)
        bindings.push_back(wrap_MPI_File_delete_binding);
    
    if (::enable_MPI_File_get_amode)
        bindings.push_back(wrap_MPI_File_get_amode_binding);
    
    if (::enable_MPI_File_get_atomicity)
        bindings.push_back(wrap_MPI_File_get_atomicity_binding);
    
    if (::enable_MPI_File_get_byte_offset)
        bindings.push_back(wrap_MPI_File_get_byte_offset_binding);
    
    if (::enable_MPI_File_get_errhandler)
        bindings.push_back(wrap_MPI_File_get_errhandler_binding);
    
    if (::enable_MPI_File_get_group)
        bindings.push_back(wrap_MPI_File_get_group_binding);
    
    if (::enable_MPI_File_get_info)
        bindings.push_back(wrap_MPI_File_get_info_binding);
    
    if (::enable_MPI_File_get_position)
        bindings.push_back(wrap_MPI_File_get_position_binding);
    
    if (::enable_MPI_File_get_position_shared)
        bindings.push_back(wrap_MPI_File_get_position_shared_binding);
    
    if (::enable_MPI_File_get_size)
        bindings.push_back(wrap_MPI_File_get_size_binding);
    
    if (::enable_MPI_File_get_type_extent)
        bindings.push_back(wrap_MPI_File_get_type_extent_binding);
    
    if (::enable_MPI_File_get_view)
        bindings.push_back(wrap_MPI_File_get_view_binding);
    
    if (::enable_MPI_File_iread)
        bindings.push_back(wrap_MPI_File_iread_binding);
    
    if (::enable_MPI_File_iread_at)
        bindings.push_back(wrap_MPI_File_iread_at_binding);
    
    if (::enable_MPI_File_iread_shared)
        bindings.push_back(wrap_MPI_File_iread_shared_binding);
    
    if (::enable_MPI_File_iwrite)
        bindings.push_back(wrap_MPI_File_iwrite_binding);
    
    if (::enable_MPI_File_iwrite_at)
        bindings.push_back(wrap_MPI_File_iwrite_at_binding);
    
    if (::enable_MPI_File_iwrite_shared)
        bindings.push_back(wrap_MPI_File_iwrite_shared_binding);
    
    if (::enable_MPI_File_open)
        bindings.push_back(wrap_MPI_File_open_binding);
    
    if (::enable_MPI_File_preallocate)
        bindings.push_back(wrap_MPI_File_preallocate_binding);
    
    if (::enable_MPI_File_read)
        bindings.push_back(wrap_MPI_File_read_binding);
    
    if (::enable_MPI_File_read_all)
        bindings.push_back(wrap_MPI_File_read_all_binding);
    
    if (::enable_MPI_File_read_all_begin)
        bindings.push_back(wrap_MPI_File_read_all_begin_binding);
    
    if (::enable_MPI_File_read_all_end)
        bindings.push_back(wrap_MPI_File_read_all_end_binding);
    
    if (::enable_MPI_File_read_at)
        bindings.push_back(wrap_MPI_File_read_at_binding);
    
    if (::enable_MPI_File_read_at_all)
        bindings.push_back(wrap_MPI_File_read_at_all_binding);
    
    if (::enable_MPI_File_read_at_all_begin)
        bindings.push_back(wrap_MPI_File_read_at_all_begin_binding);
    
    if (::enable_MPI_File_read_at_all_end)
        bindings.push_back(wrap_MPI_File_read_at_all_end_binding);
    
    if (::enable_MPI_File_read_ordered)
        bindings.push_back(wrap_MPI_File_read_ordered_binding);
    
    if (::enable_MPI_File_read_ordered_begin)
        bindings.push_back(wrap_MPI_File_read_ordered_begin_binding);
    
    if (::enable_MPI_File_read_ordered_end)
        bindings.push_back(wrap_MPI_File_read_ordered_end_binding);
    
    if (::enable_MPI_File_read_shared)
        bindings.push_back(wrap_MPI_File_read_shared_binding);
    
    if (::enable_MPI_File_seek)
        bindings.push_back(wrap_MPI_File_seek_binding);
    
    if (::enable_MPI_File_seek_shared)
        bindings.push_back(wrap_MPI_File_seek_shared_binding);
    
    if (::enable_MPI_File_set_atomicity)
        bindings.push_back(wrap_MPI_File_set_atomicity_binding);
    
    if (::enable_MPI_File_set_errhandler)
        bindings.push_back(wrap_MPI_File_set_errhandler_binding);
    
    if (::enable_MPI_File_set_info)
        bindings.push_back(wrap_MPI_File_set_info_binding);
    
    if (::enable_MPI_File_set_size)
        bindings.push_back(wrap_MPI_File_set_size_binding);
    
    if (::enable_MPI_File_set_view)
        bindings.push_back(wrap_MPI_File_set_view_binding);
    
    if (::enable_MPI_File_sync)
        bindings.push_back(wrap_MPI_File_sync_binding);
    
    if (::enable_MPI_File_write)
        bindings.push_back(wrap_MPI_File_write_binding);
    
    if (::enable_MPI_File_write_all)
        bindings.push_back(wrap_MPI_File_write_all_binding);
    
    if (::enable_MPI_File_write_all_begin)
        bindings.push_back(wrap_MPI_File_write_all_begin_binding);
    
    if (::enable_MPI_File_write_all_end)
        bindings.push_back(wrap_MPI_File_write_all_end_binding);
    
    if (::enable_MPI_File_write_at)
        bindings.push_back(wrap_MPI_File_write_at_binding);
    
    if (::enable_MPI_File_write_at_all)
        bindings.push_back(wrap_MPI_File_write_at_all_binding);
    
    if (::enable_MPI_File_write_at_all_begin)
        bindings.push_back(wrap_MPI_File_write_at_all_begin_binding);
    
    if (::enable_MPI_File_write_at_all_end)
        bindings.push_back(wrap_MPI_File_write_at_all_end_binding);
    
    if (::enable_MPI_File_write_ordered)
        bindings.push_back(wrap_MPI_File_write_ordered_binding);
    
    if (::enable_MPI_File_write_ordered_begin)
        bindings.push_back(wrap_MPI_File_write_ordered_begin_binding);
    
    if (::enable_MPI_File_write_ordered_end)
        bindings.push_back(wrap_MPI_File_write_ordered_end_binding);
    
    if (::enable_MPI_File_write_shared)
        bindings.push_back(wrap_MPI_File_write_shared_binding);
    
    if (::enable_MPI_Finalized)
        bindings.push_back(wrap_MPI_Finalized_binding);
    
    if (::enable_MPI_Free_mem)
        bindings.push_back(wrap_MPI_Free_mem_binding);
    
    if (::enable_MPI_Gather)
        bindings.push_back(wrap_MPI_Gather_binding);
    
    if (::enable_MPI_Gatherv)
        bindings.push_back(wrap_MPI_Gatherv_binding);
    
    if (::enable_MPI_Get)
        bindings.push_back(wrap_MPI_Get_binding);
    
    if (::enable_MPI_Get_accumulate)
        bindings.push_back(wrap_MPI_Get_accumulate_binding);
    
    if (::enable_MPI_Get_address)
        bindings.push_back(wrap_MPI_Get_address_binding);
    
    if (::enable_MPI_Get_count)
        bindings.push_back(wrap_MPI_Get_count_binding);
    
    if (::enable_MPI_Get_elements)
        bindings.push_back(wrap_MPI_Get_elements_binding);
    
    if (::enable_MPI_Get_elements_x)
        bindings.push_back(wrap_MPI_Get_elements_x_binding);
    
    if (::enable_MPI_Get_library_version)
        bindings.push_back(wrap_MPI_Get_library_version_binding);
    
    if (::enable_MPI_Get_processor_name)
        bindings.push_back(wrap_MPI_Get_processor_name_binding);
    
    if (::enable_MPI_Get_version)
        bindings.push_back(wrap_MPI_Get_version_binding);
    
    if (::enable_MPI_Graph_create)
        bindings.push_back(wrap_MPI_Graph_create_binding);
    
    if (::enable_MPI_Graph_get)
        bindings.push_back(wrap_MPI_Graph_get_binding);
    
    if (::enable_MPI_Graph_map)
        bindings.push_back(wrap_MPI_Graph_map_binding);
    
    if (::enable_MPI_Graph_neighbors)
        bindings.push_back(wrap_MPI_Graph_neighbors_binding);
    
    if (::enable_MPI_Graph_neighbors_count)
        bindings.push_back(wrap_MPI_Graph_neighbors_count_binding);
    
    if (::enable_MPI_Graphdims_get)
        bindings.push_back(wrap_MPI_Graphdims_get_binding);
    
    if (::enable_MPI_Grequest_complete)
        bindings.push_back(wrap_MPI_Grequest_complete_binding);
    
    if (::enable_MPI_Grequest_start)
        bindings.push_back(wrap_MPI_Grequest_start_binding);
    
    if (::enable_MPI_Group_compare)
        bindings.push_back(wrap_MPI_Group_compare_binding);
    
    if (::enable_MPI_Group_difference)
        bindings.push_back(wrap_MPI_Group_difference_binding);
    
    if (::enable_MPI_Group_excl)
        bindings.push_back(wrap_MPI_Group_excl_binding);
    
    if (::enable_MPI_Group_free)
        bindings.push_back(wrap_MPI_Group_free_binding);
    
    if (::enable_MPI_Group_incl)
        bindings.push_back(wrap_MPI_Group_incl_binding);
    
    if (::enable_MPI_Group_intersection)
        bindings.push_back(wrap_MPI_Group_intersection_binding);
    
    if (::enable_MPI_Group_range_excl)
        bindings.push_back(wrap_MPI_Group_range_excl_binding);
    
    if (::enable_MPI_Group_range_incl)
        bindings.push_back(wrap_MPI_Group_range_incl_binding);
    
    if (::enable_MPI_Group_rank)
        bindings.push_back(wrap_MPI_Group_rank_binding);
    
    if (::enable_MPI_Group_size)
        bindings.push_back(wrap_MPI_Group_size_binding);
    
    if (::enable_MPI_Group_translate_ranks)
        bindings.push_back(wrap_MPI_Group_translate_ranks_binding);
    
    if (::enable_MPI_Group_union)
        bindings.push_back(wrap_MPI_Group_union_binding);
    
    if (::enable_MPI_Iallgather)
        bindings.push_back(wrap_MPI_Iallgather_binding);
    
    if (::enable_MPI_Iallgatherv)
        bindings.push_back(wrap_MPI_Iallgatherv_binding);
    
    if (::enable_MPI_Iallreduce)
        bindings.push_back(wrap_MPI_Iallreduce_binding);
    
    if (::enable_MPI_Ialltoall)
        bindings.push_back(wrap_MPI_Ialltoall_binding);
    
    if (::enable_MPI_Ialltoallv)
        bindings.push_back(wrap_MPI_Ialltoallv_binding);
    
    if (::enable_MPI_Ialltoallw)
        bindings.push_back(wrap_MPI_Ialltoallw_binding);
    
    if (::enable_MPI_Ibarrier)
        bindings.push_back(wrap_MPI_Ibarrier_binding);
    
    if (::enable_MPI_Ibcast)
        bindings.push_back(wrap_MPI_Ibcast_binding);
    
    if (::enable_MPI_Ibsend)
        bindings.push_back(wrap_MPI_Ibsend_binding);
    
    if (::enable_MPI_Iexscan)
        bindings.push_back(wrap_MPI_Iexscan_binding);
    
    if (::enable_MPI_Igather)
        bindings.push_back(wrap_MPI_Igather_binding);
    
    if (::enable_MPI_Igatherv)
        bindings.push_back(wrap_MPI_Igatherv_binding);
    
    if (::enable_MPI_Improbe)
        bindings.push_back(wrap_MPI_Improbe_binding);
    
    if (::enable_MPI_Imrecv)
        bindings.push_back(wrap_MPI_Imrecv_binding);
    
    if (::enable_MPI_Ineighbor_allgather)
        bindings.push_back(wrap_MPI_Ineighbor_allgather_binding);
    
    if (::enable_MPI_Ineighbor_allgatherv)
        bindings.push_back(wrap_MPI_Ineighbor_allgatherv_binding);
    
    if (::enable_MPI_Ineighbor_alltoall)
        bindings.push_back(wrap_MPI_Ineighbor_alltoall_binding);
    
    if (::enable_MPI_Ineighbor_alltoallv)
        bindings.push_back(wrap_MPI_Ineighbor_alltoallv_binding);
    
    if (::enable_MPI_Ineighbor_alltoallw)
        bindings.push_back(wrap_MPI_Ineighbor_alltoallw_binding);
    
    if (::enable_MPI_Info_create)
        bindings.push_back(wrap_MPI_Info_create_binding);
    
    if (::enable_MPI_Info_delete)
        bindings.push_back(wrap_MPI_Info_delete_binding);
    
    if (::enable_MPI_Info_dup)
        bindings.push_back(wrap_MPI_Info_dup_binding);
    
    if (::enable_MPI_Info_free)
        bindings.push_back(wrap_MPI_Info_free_binding);
    
    if (::enable_MPI_Info_get)
        bindings.push_back(wrap_MPI_Info_get_binding);
    
    if (::enable_MPI_Info_get_nkeys)
        bindings.push_back(wrap_MPI_Info_get_nkeys_binding);
    
    if (::enable_MPI_Info_get_nthkey)
        bindings.push_back(wrap_MPI_Info_get_nthkey_binding);
    
    if (::enable_MPI_Info_get_valuelen)
        bindings.push_back(wrap_MPI_Info_get_valuelen_binding);
    
    if (::enable_MPI_Info_set)
        bindings.push_back(wrap_MPI_Info_set_binding);
    
    if (::enable_MPI_Initialized)
        bindings.push_back(wrap_MPI_Initialized_binding);
    
    if (::enable_MPI_Intercomm_create)
        bindings.push_back(wrap_MPI_Intercomm_create_binding);
    
    if (::enable_MPI_Intercomm_merge)
        bindings.push_back(wrap_MPI_Intercomm_merge_binding);
    
    if (::enable_MPI_Iprobe)
        bindings.push_back(wrap_MPI_Iprobe_binding);
    
    if (::enable_MPI_Irecv)
        bindings.push_back(wrap_MPI_Irecv_binding);
    
    if (::enable_MPI_Ireduce)
        bindings.push_back(wrap_MPI_Ireduce_binding);
    
    if (::enable_MPI_Ireduce_scatter)
        bindings.push_back(wrap_MPI_Ireduce_scatter_binding);
    
    if (::enable_MPI_Ireduce_scatter_block)
        bindings.push_back(wrap_MPI_Ireduce_scatter_block_binding);
    
    if (::enable_MPI_Irsend)
        bindings.push_back(wrap_MPI_Irsend_binding);
    
    if (::enable_MPI_Is_thread_main)
        bindings.push_back(wrap_MPI_Is_thread_main_binding);
    
    if (::enable_MPI_Iscan)
        bindings.push_back(wrap_MPI_Iscan_binding);
    
    if (::enable_MPI_Iscatter)
        bindings.push_back(wrap_MPI_Iscatter_binding);
    
    if (::enable_MPI_Iscatterv)
        bindings.push_back(wrap_MPI_Iscatterv_binding);
    
    if (::enable_MPI_Isend)
        bindings.push_back(wrap_MPI_Isend_binding);
    
    if (::enable_MPI_Issend)
        bindings.push_back(wrap_MPI_Issend_binding);
    
    if (::enable_MPI_Keyval_create)
        bindings.push_back(wrap_MPI_Keyval_create_binding);
    
    if (::enable_MPI_Keyval_free)
        bindings.push_back(wrap_MPI_Keyval_free_binding);
    
    if (::enable_MPI_Lookup_name)
        bindings.push_back(wrap_MPI_Lookup_name_binding);
    
    if (::enable_MPI_Mprobe)
        bindings.push_back(wrap_MPI_Mprobe_binding);
    
    if (::enable_MPI_Mrecv)
        bindings.push_back(wrap_MPI_Mrecv_binding);
    
    if (::enable_MPI_Neighbor_allgather)
        bindings.push_back(wrap_MPI_Neighbor_allgather_binding);
    
    if (::enable_MPI_Neighbor_allgatherv)
        bindings.push_back(wrap_MPI_Neighbor_allgatherv_binding);
    
    if (::enable_MPI_Neighbor_alltoall)
        bindings.push_back(wrap_MPI_Neighbor_alltoall_binding);
    
    if (::enable_MPI_Neighbor_alltoallv)
        bindings.push_back(wrap_MPI_Neighbor_alltoallv_binding);
    
    if (::enable_MPI_Neighbor_alltoallw)
        bindings.push_back(wrap_MPI_Neighbor_alltoallw_binding);
    
    if (::enable_MPI_Op_commutative)
        bindings.push_back(wrap_MPI_Op_commutative_binding);
    
    if (::enable_MPI_Op_create)
        bindings.push_back(wrap_MPI_Op_create_binding);
    
    if (::enable_MPI_Op_free)
        bindings.push_back(wrap_MPI_Op_free_binding);
    
    if (::enable_MPI_Open_port)
        bindings.push_back(wrap_MPI_Open_port_binding);
    
    if (::enable_MPI_Pack)
        bindings.push_back(wrap_MPI_Pack_binding);
    
    if (::enable_MPI_Pack_external)
        bindings.push_back(wrap_MPI_Pack_external_binding);
    
    if (::enable_MPI_Pack_external_size)
        bindings.push_back(wrap_MPI_Pack_external_size_binding);
    
    if (::enable_MPI_Pack_size)
        bindings.push_back(wrap_MPI_Pack_size_binding);
    
    if (::enable_MPI_Pcontrol)
        bindings.push_back(wrap_MPI_Pcontrol_binding);
    
    if (::enable_MPI_Probe)
        bindings.push_back(wrap_MPI_Probe_binding);
    
    if (::enable_MPI_Publish_name)
        bindings.push_back(wrap_MPI_Publish_name_binding);
    
    if (::enable_MPI_Put)
        bindings.push_back(wrap_MPI_Put_binding);
    
    if (::enable_MPI_Query_thread)
        bindings.push_back(wrap_MPI_Query_thread_binding);
    
    if (::enable_MPI_Raccumulate)
        bindings.push_back(wrap_MPI_Raccumulate_binding);
    
    if (::enable_MPI_Recv)
        bindings.push_back(wrap_MPI_Recv_binding);
    
    if (::enable_MPI_Recv_init)
        bindings.push_back(wrap_MPI_Recv_init_binding);
    
    if (::enable_MPI_Reduce)
        bindings.push_back(wrap_MPI_Reduce_binding);
    
    if (::enable_MPI_Reduce_local)
        bindings.push_back(wrap_MPI_Reduce_local_binding);
    
    if (::enable_MPI_Reduce_scatter)
        bindings.push_back(wrap_MPI_Reduce_scatter_binding);
    
    if (::enable_MPI_Reduce_scatter_block)
        bindings.push_back(wrap_MPI_Reduce_scatter_block_binding);
    
    if (::enable_MPI_Register_datarep)
        bindings.push_back(wrap_MPI_Register_datarep_binding);
    
    if (::enable_MPI_Request_free)
        bindings.push_back(wrap_MPI_Request_free_binding);
    
    if (::enable_MPI_Request_get_status)
        bindings.push_back(wrap_MPI_Request_get_status_binding);
    
    if (::enable_MPI_Rget)
        bindings.push_back(wrap_MPI_Rget_binding);
    
    if (::enable_MPI_Rget_accumulate)
        bindings.push_back(wrap_MPI_Rget_accumulate_binding);
    
    if (::enable_MPI_Rput)
        bindings.push_back(wrap_MPI_Rput_binding);
    
    if (::enable_MPI_Rsend)
        bindings.push_back(wrap_MPI_Rsend_binding);
    
    if (::enable_MPI_Rsend_init)
        bindings.push_back(wrap_MPI_Rsend_init_binding);
    
    if (::enable_MPI_Scan)
        bindings.push_back(wrap_MPI_Scan_binding);
    
    if (::enable_MPI_Scatter)
        bindings.push_back(wrap_MPI_Scatter_binding);
    
    if (::enable_MPI_Scatterv)
        bindings.push_back(wrap_MPI_Scatterv_binding);
    
    if (::enable_MPI_Send)
        bindings.push_back(wrap_MPI_Send_binding);
    
    if (::enable_MPI_Send_init)
        bindings.push_back(wrap_MPI_Send_init_binding);
    
    if (::enable_MPI_Sendrecv)
        bindings.push_back(wrap_MPI_Sendrecv_binding);
    
    if (::enable_MPI_Sendrecv_replace)
        bindings.push_back(wrap_MPI_Sendrecv_replace_binding);
    
    if (::enable_MPI_Ssend)
        bindings.push_back(wrap_MPI_Ssend_binding);
    
    if (::enable_MPI_Ssend_init)
        bindings.push_back(wrap_MPI_Ssend_init_binding);
    
    if (::enable_MPI_Start)
        bindings.push_back(wrap_MPI_Start_binding);
    
    if (::enable_MPI_Startall)
        bindings.push_back(wrap_MPI_Startall_binding);
    
    if (::enable_MPI_Status_set_cancelled)
        bindings.push_back(wrap_MPI_Status_set_cancelled_binding);
    
    if (::enable_MPI_Status_set_elements)
        bindings.push_back(wrap_MPI_Status_set_elements_binding);
    
    if (::enable_MPI_Status_set_elements_x)
        bindings.push_back(wrap_MPI_Status_set_elements_x_binding);
    
    if (::enable_MPI_Test)
        bindings.push_back(wrap_MPI_Test_binding);
    
    if (::enable_MPI_Test_cancelled)
        bindings.push_back(wrap_MPI_Test_cancelled_binding);
    
    if (::enable_MPI_Testall)
        bindings.push_back(wrap_MPI_Testall_binding);
    
    if (::enable_MPI_Testany)
        bindings.push_back(wrap_MPI_Testany_binding);
    
    if (::enable_MPI_Testsome)
        bindings.push_back(wrap_MPI_Testsome_binding);
    
    if (::enable_MPI_Topo_test)
        bindings.push_back(wrap_MPI_Topo_test_binding);
    
    if (::enable_MPI_Type_commit)
        bindings.push_back(wrap_MPI_Type_commit_binding);
    
    if (::enable_MPI_Type_contiguous)
        bindings.push_back(wrap_MPI_Type_contiguous_binding);
    
    if (::enable_MPI_Type_create_darray)
        bindings.push_back(wrap_MPI_Type_create_darray_binding);
    
    if (::enable_MPI_Type_create_f90_complex)
        bindings.push_back(wrap_MPI_Type_create_f90_complex_binding);
    
    if (::enable_MPI_Type_create_f90_integer)
        bindings.push_back(wrap_MPI_Type_create_f90_integer_binding);
    
    if (::enable_MPI_Type_create_f90_real)
        bindings.push_back(wrap_MPI_Type_create_f90_real_binding);
    
    if (::enable_MPI_Type_create_hindexed)
        bindings.push_back(wrap_MPI_Type_create_hindexed_binding);
    
    if (::enable_MPI_Type_create_hindexed_block)
        bindings.push_back(wrap_MPI_Type_create_hindexed_block_binding);
    
    if (::enable_MPI_Type_create_hvector)
        bindings.push_back(wrap_MPI_Type_create_hvector_binding);
    
    if (::enable_MPI_Type_create_indexed_block)
        bindings.push_back(wrap_MPI_Type_create_indexed_block_binding);
    
    if (::enable_MPI_Type_create_keyval)
        bindings.push_back(wrap_MPI_Type_create_keyval_binding);
    
    if (::enable_MPI_Type_create_resized)
        bindings.push_back(wrap_MPI_Type_create_resized_binding);
    
    if (::enable_MPI_Type_create_struct)
        bindings.push_back(wrap_MPI_Type_create_struct_binding);
    
    if (::enable_MPI_Type_create_subarray)
        bindings.push_back(wrap_MPI_Type_create_subarray_binding);
    
    if (::enable_MPI_Type_delete_attr)
        bindings.push_back(wrap_MPI_Type_delete_attr_binding);
    
    if (::enable_MPI_Type_dup)
        bindings.push_back(wrap_MPI_Type_dup_binding);
    
    if (::enable_MPI_Type_extent)
        bindings.push_back(wrap_MPI_Type_extent_binding);
    
    if (::enable_MPI_Type_free)
        bindings.push_back(wrap_MPI_Type_free_binding);
    
    if (::enable_MPI_Type_free_keyval)
        bindings.push_back(wrap_MPI_Type_free_keyval_binding);
    
    if (::enable_MPI_Type_get_attr)
        bindings.push_back(wrap_MPI_Type_get_attr_binding);
    
    if (::enable_MPI_Type_get_contents)
        bindings.push_back(wrap_MPI_Type_get_contents_binding);
    
    if (::enable_MPI_Type_get_envelope)
        bindings.push_back(wrap_MPI_Type_get_envelope_binding);
    
    if (::enable_MPI_Type_get_extent)
        bindings.push_back(wrap_MPI_Type_get_extent_binding);
    
    if (::enable_MPI_Type_get_extent_x)
        bindings.push_back(wrap_MPI_Type_get_extent_x_binding);
    
    if (::enable_MPI_Type_get_name)
        bindings.push_back(wrap_MPI_Type_get_name_binding);
    
    if (::enable_MPI_Type_get_true_extent)
        bindings.push_back(wrap_MPI_Type_get_true_extent_binding);
    
    if (::enable_MPI_Type_get_true_extent_x)
        bindings.push_back(wrap_MPI_Type_get_true_extent_x_binding);
    
    if (::enable_MPI_Type_hindexed)
        bindings.push_back(wrap_MPI_Type_hindexed_binding);
    
    if (::enable_MPI_Type_hvector)
        bindings.push_back(wrap_MPI_Type_hvector_binding);
    
    if (::enable_MPI_Type_indexed)
        bindings.push_back(wrap_MPI_Type_indexed_binding);
    
    if (::enable_MPI_Type_lb)
        bindings.push_back(wrap_MPI_Type_lb_binding);
    
    if (::enable_MPI_Type_match_size)
        bindings.push_back(wrap_MPI_Type_match_size_binding);
    
    if (::enable_MPI_Type_set_attr)
        bindings.push_back(wrap_MPI_Type_set_attr_binding);
    
    if (::enable_MPI_Type_set_name)
        bindings.push_back(wrap_MPI_Type_set_name_binding);
    
    if (::enable_MPI_Type_size)
        bindings.push_back(wrap_MPI_Type_size_binding);
    
    if (::enable_MPI_Type_size_x)
        bindings.push_back(wrap_MPI_Type_size_x_binding);
    
    if (::enable_MPI_Type_struct)
        bindings.push_back(wrap_MPI_Type_struct_binding);
    
    if (::enable_MPI_Type_ub)
        bindings.push_back(wrap_MPI_Type_ub_binding);
    
    if (::enable_MPI_Type_vector)
        bindings.push_back(wrap_MPI_Type_vector_binding);
    
    if (::enable_MPI_Unpack)
        bindings.push_back(wrap_MPI_Unpack_binding);
    
    if (::enable_MPI_Unpack_external)
        bindings.push_back(wrap_MPI_Unpack_external_binding);
    
    if (::enable_MPI_Unpublish_name)
        bindings.push_back(wrap_MPI_Unpublish_name_binding);
    
    if (::enable_MPI_Wait)
        bindings.push_back(wrap_MPI_Wait_binding);
    
    if (::enable_MPI_Waitall)
        bindings.push_back(wrap_MPI_Waitall_binding);
    
    if (::enable_MPI_Waitany)
        bindings.push_back(wrap_MPI_Waitany_binding);
    
    if (::enable_MPI_Waitsome)
        bindings.push_back(wrap_MPI_Waitsome_binding);
    
    if (::enable_MPI_Win_allocate)
        bindings.push_back(wrap_MPI_Win_allocate_binding);
    
    if (::enable_MPI_Win_allocate_shared)
        bindings.push_back(wrap_MPI_Win_allocate_shared_binding);
    
    if (::enable_MPI_Win_attach)
        bindings.push_back(wrap_MPI_Win_attach_binding);
    
    if (::enable_MPI_Win_call_errhandler)
        bindings.push_back(wrap_MPI_Win_call_errhandler_binding);
    
    if (::enable_MPI_Win_complete)
        bindings.push_back(wrap_MPI_Win_complete_binding);
    
    if (::enable_MPI_Win_create)
        bindings.push_back(wrap_MPI_Win_create_binding);
    
    if (::enable_MPI_Win_create_dynamic)
        bindings.push_back(wrap_MPI_Win_create_dynamic_binding);
    
    if (::enable_MPI_Win_create_errhandler)
        bindings.push_back(wrap_MPI_Win_create_errhandler_binding);
    
    if (::enable_MPI_Win_create_keyval)
        bindings.push_back(wrap_MPI_Win_create_keyval_binding);
    
    if (::enable_MPI_Win_delete_attr)
        bindings.push_back(wrap_MPI_Win_delete_attr_binding);
    
    if (::enable_MPI_Win_detach)
        bindings.push_back(wrap_MPI_Win_detach_binding);
    
    if (::enable_MPI_Win_fence)
        bindings.push_back(wrap_MPI_Win_fence_binding);
    
    if (::enable_MPI_Win_flush)
        bindings.push_back(wrap_MPI_Win_flush_binding);
    
    if (::enable_MPI_Win_flush_all)
        bindings.push_back(wrap_MPI_Win_flush_all_binding);
    
    if (::enable_MPI_Win_flush_local)
        bindings.push_back(wrap_MPI_Win_flush_local_binding);
    
    if (::enable_MPI_Win_flush_local_all)
        bindings.push_back(wrap_MPI_Win_flush_local_all_binding);
    
    if (::enable_MPI_Win_free)
        bindings.push_back(wrap_MPI_Win_free_binding);
    
    if (::enable_MPI_Win_free_keyval)
        bindings.push_back(wrap_MPI_Win_free_keyval_binding);
    
    if (::enable_MPI_Win_get_attr)
        bindings.push_back(wrap_MPI_Win_get_attr_binding);
    
    if (::enable_MPI_Win_get_errhandler)
        bindings.push_back(wrap_MPI_Win_get_errhandler_binding);
    
    if (::enable_MPI_Win_get_group)
        bindings.push_back(wrap_MPI_Win_get_group_binding);
    
    if (::enable_MPI_Win_get_info)
        bindings.push_back(wrap_MPI_Win_get_info_binding);
    
    if (::enable_MPI_Win_get_name)
        bindings.push_back(wrap_MPI_Win_get_name_binding);
    
    if (::enable_MPI_Win_lock)
        bindings.push_back(wrap_MPI_Win_lock_binding);
    
    if (::enable_MPI_Win_lock_all)
        bindings.push_back(wrap_MPI_Win_lock_all_binding);
    
    if (::enable_MPI_Win_post)
        bindings.push_back(wrap_MPI_Win_post_binding);
    
    if (::enable_MPI_Win_set_attr)
        bindings.push_back(wrap_MPI_Win_set_attr_binding);
    
    if (::enable_MPI_Win_set_errhandler)
        bindings.push_back(wrap_MPI_Win_set_errhandler_binding);
    
    if (::enable_MPI_Win_set_info)
        bindings.push_back(wrap_MPI_Win_set_info_binding);
    
    if (::enable_MPI_Win_set_name)
        bindings.push_back(wrap_MPI_Win_set_name_binding);
    
    if (::enable_MPI_Win_shared_query)
        bindings.push_back(wrap_MPI_Win_shared_query_binding);
    
    if (::enable_MPI_Win_start)
        bindings.push_back(wrap_MPI_Win_start_binding);
    
    if (::enable_MPI_Win_sync)
        bindings.push_back(wrap_MPI_Win_sync_binding);
    
    if (::enable_MPI_Win_test)
        bindings.push_back(wrap_MPI_Win_test_binding);
    
    if (::enable_MPI_Win_unlock)
        bindings.push_back(wrap_MPI_Win_unlock_binding);
    
    if (::enable_MPI_Win_unlock_all)
        bindings.push_back(wrap_MPI_Win_unlock_all_binding);
    
    if (::enable_MPI_Win_wait)
        bindings.push_back(wrap_MPI_Win_wait_binding);
    
    if (::enable_MPI_Wtick)
        bindings.push_back(wrap_MPI_Wtick_binding);
    
    if (::enable_MPI_Wtime)
        bindings.push_back(wrap_MPI_Wtime_binding);
    

    gotcha_wrap(bindings.data(), bindings.size(), "caliper/mpi");
#else
    Log(2).stream() << "mpiwrap: Using PMPI wrappers." << std::endl;
#endif
}

}
